@@ -154,6 +154,20 @@ setClass("CsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
          validity = function(object) .Call(Csparse_validate, object)
          )
 
+if(FALSE) { ## in theory.. would be neat for  new("dgCMatrix", Dim = c(3L,3L))
+setMethod("initialize", "CsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[2L] + 1L)
+    .Object
+})
+
+setMethod("initialize", "RsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[1L] + 1L)
+    .Object
+})
+}# not yet (fails)
+
 setClass("RsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
 	 slots = c(p = "integer", j = "integer"),
 	 prototype = prototype(p = 0L),# to be valid
@@ -737,7 +751,7 @@ setClassUnion("geMatrix", members = c("dgeMatrix", "lgeMatrix", "ngeMatrix"))
 
 ## --- "General" (not Matrix at all) ----
 
-## e.g. for "Arith" methods:
+## e.g. for "Arith" methods, NB: --> see "numericVector" below (incl "integer")
 setClassUnion("numLike", members = c("numeric", "logical"))
 
 ##setClassUnion("numIndex", members = "numeric")
@@ -814,19 +828,10 @@ setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
 	      members = c("logical", "integer", "numeric",
 			  "complex", "raw", "character"))
 
+## NB: --> see "numLike" above
+if(FALSE) # not used anywhere
 setClassUnion("numericVector", members = c("logical", "integer", "numeric"))
 
-setClassUnion("Mnumeric", members = c("numericVector", "Matrix"))
-## not "matrix" as that extends "vector" and contains "character", "structure" ...
-
-setValidity("Mnumeric",
-            function(object) {
-                if(is.numeric(object) ||
-                   is.logical(object) ||
-                   inherits(object, "Matrix")) return(TRUE)
-                ## else
-                "Not a valid 'Mnumeric' class object"
-		})
 
 
 
