@@ -30,7 +30,5 @@ setAs("lgCMatrix", "lgeMatrix",
 setAs("lgCMatrix", "matrix", function(from) .Call(lgC_to_matrix, from))
 ## not this: .Call(Csparse_to_matrix, from)), since it goes via dense -> double precision
 
+setAs("matrix", "lgCMatrix", .m2lgC)
 
-## TODO (maybe): write  matrix_to_lcsc()  in ../src/lgCMatrix.c
-setAs("matrix", "lgCMatrix",
-      function(from) as(as(from, "lgTMatrix"), "lgCMatrix"))
