@@ -1,23 +1,24 @@
 ###       Functions that are used in several parts of the nlme library
 ###                 but do not belong to any specific part
 ###
+### Copyright 2006-2016  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
-# Copyright 2006-2013 The R Core team
-#
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 2 of the License, or
-#  (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
-#
+###
+###  This program is free software; you can redistribute it and/or modify
+###  it under the terms of the GNU General Public License as published by
+###  the Free Software Foundation; either version 2 of the License, or
+###  (at your option) any later version.
+###
+###  This program is distributed in the hope that it will be useful,
+###  but WITHOUT ANY WARRANTY; without even the implied warranty of
+###  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+###  GNU General Public License for more details.
+###
+###  A copy of the GNU General Public License is available at
+###  http://www.r-project.org/Licenses/
+
+svd.d <- function(x) La.svd(x, nu=0L, nv=0L)$d
 
 allCoef <-
   ## Combines different coefficient vectors into one vector, keeping track
@@ -26,13 +27,13 @@ allCoef <-
 {
   dots <- list(...)
   theta <- lapply(dots, extract)
-  len <- unlist(lapply(theta, length))
+  len <- lengths(theta)
   num <- seq_along(len)
-  if (sum(len) > 0) {
-    which <- outer(rep(num, len), num, "==")
-  } else {
-    which <- array(FALSE, c(1, length(len)))
-  }
+  which <-
+    if (sum(len) > 0)
+      outer(rep(num, len), num, "==")
+    else
+      array(FALSE, c(1, length(len)))
   cnames <- unlist(as.list(sys.call()[-1]))
   dimnames(which) <- list(NULL, cnames[cnames != substitute(extract)])
   theta <- unlist(theta)
@@ -56,8 +57,7 @@ asOneFormula <-
   ## list of formulas, except for the names in omit
   function(..., omit = c(".", "pi"))
 {
-  dots <- list(...)
-  names <- unique(allVarsRec((dots)))
+  names <- unique(allVarsRec(list(...)))
   names <- names[is.na(match(names, omit))]
   if (length(names))
     eval(parse(text = paste("~", paste(names, collapse = "+")))[[1]])
@@ -97,14 +97,13 @@ compareFits <-
   out
 }
 
-fdHess <- function(pars, fun, ..., .relStep = (.Machine$double.eps)^(1/3),
+fdHess <- function(pars, fun, ..., .relStep = .Machine$double.eps^(1/3),
                    minAbsPar = 0)
   ## Use a Koschal design to establish a second order model for the response
 {
   pars <- as.numeric(pars)
   npar <- length(pars)
-  incr <- ifelse( abs(pars) <= minAbsPar, minAbsPar * .relStep,
-                 abs(pars) * .relStep )
+  incr <- pmax(abs(pars), minAbsPar) * .relStep
   baseInd <- diag(npar)
   frac <- c(1, incr, incr^2)
   cols <- list(0, baseInd, -baseInd)
@@ -119,11 +118,13 @@ fdHess <- function(pars, fun, ..., .relStep = (.Machine$double.eps)^(1/3),
   for ( i in seq_along(pars)[ - npar ] ) {
     Xcols <- c( Xcols, list( indMat[ , i ] * indMat[ , -(1:i) ] ) )
   }
-  coefs <- solve( do.call( "cbind", Xcols ) , apply(shifted, 2, fun, ...) )/frac
+  coefs <- solve(do.call("cbind", Xcols),
+                 apply(shifted, 2, fun, ...)) / frac
   Hess <- diag( coefs[ 1 + npar + seq_along(pars) ], ncol = npar )
-  Hess[ row( Hess ) > col ( Hess ) ] <- coefs[ -(1:(1 + 2 * npar)) ]
-  list( mean = coefs[ 1 ], gradient = coefs[ 1 + seq_along(pars) ],
-       Hessian = ( Hess + t(Hess) ) )
+  Hess[ row(Hess) > col (Hess) ] <- coefs[ -(1:(1 + 2 * npar)) ]
+  list(mean     = coefs[ 1 ],
+       gradient = coefs[ 1 + seq_along(pars) ],
+       Hessian   = (Hess + t(Hess)) )
 }
 
 gapply <-
@@ -247,13 +248,11 @@ gsummary <-
       aux <- table(x)
       names(aux)[match(max(aux), aux)]
     }
-    if (data.class(FUN) == "function") {	# single function given
+    if (is.function(FUN)) {	# single function given
       FUN <- list(numeric = FUN, ordered = Mode, factor = Mode)
     } else {
-      if (!(is.list(FUN) &&
-	   all(sapply(FUN, data.class) == "function"))) {
+      if (!(is.list(FUN) && all(sapply(FUN, is.function))))
 	stop("'FUN' can only be a function or a list of functions")
-      }
       auxFUN <- list(numeric = mean, ordered = Mode, factor = Mode)
       aux <- names(auxFUN)[is.na(match(names(auxFUN), names(FUN)))]
       if (length(aux) > 0) FUN[aux] <- auxFUN[aux]
