@@ -14,10 +14,12 @@ extern "C" {
 #include <Rdefines.h> /* Rinternals.h + GET_SLOT etc */
 #include <R_ext/RS.h> /* for Memzero() */
 
+// NB: For  'FCONE'  etc (for LTO), the "includer" will  #include "Lapack-etc.h"
+// --
+
 #define imax2(x, y) ((x < y) ? y : x)
 #define imin2(x, y) ((x < y) ? x : y)
 
-
 // must come after <R.h> above, for clang (2015-08-05)
 #ifdef __GNUC__
 # undef alloca
@@ -47,6 +49,7 @@ extern "C" {
 # endif
 #endif
 
+
 #define Alloca(n, t)   (t *) alloca( (size_t) ( (n) * sizeof(t) ) )
 
 #define SMALL_4_Alloca 10000
@@ -98,6 +101,7 @@ SEXP as_det_obj(double val, int log, int sign);
 SEXP get_factors(SEXP obj, char *nm);
 SEXP set_factors(SEXP obj, SEXP val, char *nm);
 SEXP R_set_factors(SEXP obj, SEXP val, SEXP name, SEXP warn);
+SEXP R_empty_factors(SEXP obj, SEXP warn);
 
 #if 0
 SEXP dgCMatrix_set_Dim(SEXP x, int nrow);
