@@ -12,8 +12,8 @@ SEXP get_SuiteSparse_version() {
     return ans;
 }
 
-cholmod_common c;
-cholmod_common cl;
+cholmod_common c; // for cholmod_<method>  (..)
+cholmod_common cl;// for cholmod_l_<method>(..)
 
 SEXP chm_common_env;
 static SEXP dboundSym, grow0Sym, grow1Sym, grow2Sym, maxrankSym,
@@ -48,27 +48,33 @@ void CHM_store_common() {
 }
 
 void CHM_restore_common() {
-    SEXP rho = chm_common_env;
-    c.dbound = asReal(findVarInFrame(rho, dboundSym));
-    c.grow0 = asReal(findVarInFrame(rho, grow0Sym));
-    c.grow1 = asReal(findVarInFrame(rho, grow1Sym));
-    c.grow2 = asInteger(findVarInFrame(rho, grow2Sym));
-    c.maxrank = asInteger(findVarInFrame(rho, maxrankSym));
-    c.supernodal_switch = asReal(findVarInFrame(rho, supernodal_switchSym));
-    c.supernodal = asLogical(findVarInFrame(rho, supernodalSym));
-    c.final_asis = asLogical(findVarInFrame(rho, final_asisSym));
-    c.final_super = asLogical(findVarInFrame(rho, final_superSym));
-    c.final_ll = asLogical(findVarInFrame(rho, final_llSym));
-    c.final_pack = asLogical(findVarInFrame(rho, final_packSym));
-    c.final_monotonic = asLogical(findVarInFrame(rho, final_monotonicSym));
-    c.final_resymbol = asLogical(findVarInFrame(rho, final_resymbolSym));
-    c.prefer_zomplex = asLogical(findVarInFrame(rho, prefer_zomplexSym));
-    c.prefer_upper = asLogical(findVarInFrame(rho, prefer_upperSym));
-    c.quick_return_if_not_posdef =
-	asLogical(findVarInFrame(rho, quick_return_if_not_posdefSym));
-    c.nmethods = asInteger(findVarInFrame(rho, nmethodsSym));
-    c.method[0].ordering = asInteger(findVarInFrame(rho, m0_ordSym));
-    c.postorder = asLogical(findVarInFrame(rho, postorderSym));
+    SEXP rho = chm_common_env, var;
+
+#define SET_AS_FROM_FRAME(_V_, _KIND_, _SYM_)	\
+    var = PROTECT(findVarInFrame(rho, _SYM_));	\
+    _V_ = _KIND_(var);				\
+    UNPROTECT(1)
+
+    SET_AS_FROM_FRAME(c.dbound, asReal,    dboundSym);
+    SET_AS_FROM_FRAME(c.grow0,  asReal,    grow0Sym);
+    SET_AS_FROM_FRAME(c.grow1,  asReal,    grow1Sym);
+    SET_AS_FROM_FRAME(c.grow2,  asInteger, grow2Sym);
+    SET_AS_FROM_FRAME(c.maxrank,asInteger, maxrankSym);
+    SET_AS_FROM_FRAME(c.supernodal_switch, asReal, supernodal_switchSym);
+    SET_AS_FROM_FRAME(c.supernodal,     asLogical, supernodalSym);
+    SET_AS_FROM_FRAME(c.final_asis,     asLogical, final_asisSym);
+    SET_AS_FROM_FRAME(c.final_super,    asLogical, final_superSym);
+    SET_AS_FROM_FRAME(c.final_ll,       asLogical, final_llSym);
+    SET_AS_FROM_FRAME(c.final_pack,     asLogical, final_packSym);
+    SET_AS_FROM_FRAME(c.final_monotonic,asLogical, final_monotonicSym);
+    SET_AS_FROM_FRAME(c.final_resymbol, asLogical, final_resymbolSym);
+    SET_AS_FROM_FRAME(c.prefer_zomplex, asLogical, prefer_zomplexSym);
+    SET_AS_FROM_FRAME(c.prefer_upper,   asLogical, prefer_upperSym);
+    SET_AS_FROM_FRAME(c.quick_return_if_not_posdef,
+					asLogical, quick_return_if_not_posdefSym);
+    SET_AS_FROM_FRAME(c.nmethods,           asInteger, nmethodsSym);
+    SET_AS_FROM_FRAME(c.method[0].ordering, asInteger, m0_ordSym);
+    SET_AS_FROM_FRAME(c.postorder,          asLogical, postorderSym);
 }
 
 SEXP CHM_set_common_env(SEXP rho) {
@@ -297,7 +303,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	}
     }
 
-    if (check_Udiag && ctype % 3 == 2 // triangular
+    if (check_Udiag && ctype % 3 == 2 /* triangular */ && ans->nrow // fails for Dim = (0,0)
 	&& (*diag_P(x) == 'U')) { /* diagU2N(.)  "in place" : */
 	double one[] = {1, 0};
 	CHM_SP eye = cholmod_speye(ans->nrow, ans->ncol, ans->xtype, &c);
@@ -337,9 +343,15 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 {
     SEXP ans;
     char *cls = "";/* -Wall */
-    int *dims, nnz, *ansp, *ansi, *aii = (int*)(a->i), *api = (int*)(a->p),
-	longi = (a->itype) == CHOLMOD_LONG;
-    SuiteSparse_long *ail = (SuiteSparse_long*)(a->i), *apl = (SuiteSparse_long*)(a->p);
+    Rboolean longi = (a->itype) == CHOLMOD_LONG;
+    int *dims, nnz, *ansp, *ansi;
+    // if (longi) :
+    SuiteSparse_long
+	*ail = (SuiteSparse_long*)(a->i),
+	*apl = (SuiteSparse_long*)(a->p);
+    // else  ((a->itype) == CHOLMOD_INT) :
+    int *aii = (int*)(a->i),
+	*api = (int*)(a->p);
 
     PROTECT(dn);  /* dn is usually UNPROTECTed before the call */
 
@@ -353,7 +365,8 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	longi ? cholmod_l_free_sparse(&a, &cl) : cholmod_free_sparse(&a, &c); \
     else if (dofree < 0) Free(a)
 
-    switch(a->xtype){
+
+    switch(a->xtype) {
     case CHOLMOD_PATTERN:
 	cls = uploT ? "ntCMatrix": ((a->stype) ? "nsCMatrix" : "ngCMatrix");
 	break;
@@ -377,7 +390,7 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	DOFREE_MAYBE;
 	error(_("unknown xtype in cholmod_sparse object"));
     }
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cls)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cls));
 				/* allocate and copy common slots */
     nnz = longi ? cholmod_l_nnz(a, &cl) : cholmod_nnz(a, &c);
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
@@ -385,7 +398,7 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
     ansp = INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, a->ncol + 1));
     ansi = INTEGER(ALLOC_SLOT(ans, Matrix_iSym, INTSXP, nnz));
     for (int j = 0; j <= a->ncol; j++) ansp[j] = longi ? (int)(apl[j]) : api[j];
-    for (int p = 0; p < nnz; p++) ansi[p] = longi ? (int)(ail[p]) : aii[p];
+    for (int p = 0; p < nnz; p++)      ansi[p] = longi ? (int)(ail[p]) : aii[p];
 				/* copy data slot if present */
     if (a->xtype == CHOLMOD_REAL) {
 	int i, *m_x;
@@ -505,10 +518,10 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 	/* diagU2N(.) "in place", similarly to Tsparse_diagU2N [./Tsparse.c]
 	   (but without new SEXP): */
 	int k = m + dims[0];
-	CHM_TR tmp = cholmod_l_copy_triplet(ans, &c);
+	CHM_TR tmp = cholmod_l_copy_triplet(ans, &cl);
 	int *a_i, *a_j;
 
-	if(!cholmod_reallocate_triplet((size_t) k, tmp, &c))
+	if(!cholmod_reallocate_triplet((size_t) k, tmp, &cl))
 	    error(_("as_cholmod_triplet(): could not reallocate for internal diagU2N()"
 		      ));
 
@@ -609,7 +622,7 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 	DOFREE_MAYBE;
 	error(_("unknown xtype in cholmod_triplet object"));
     }
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = a->nrow; dims[1] = a->ncol;
@@ -812,7 +825,10 @@ int R_cholmod_start(CHM_CM c)
     int res;
     if (!(res = cholmod_start(c)))
 	error(_("Unable to initialize cholmod: error code %d"), res);
-    c->print_function = R_cholmod_printf; /* Rprintf gives warning */
+    /*SuiteSparse <= 4.x.y :
+     * c->print_function = R_cholmod_printf; /. Rprintf gives warning */
+    SuiteSparse_config.printf_func = R_cholmod_printf;/* Rprintf gives warning */
+    //                                 ^^^^^^^^^ now is misnomer
     /* Since we provide an error handler, it may not be a good idea to allow CHOLMOD printing,
      * because that's not easily suppressed on the R level :
      * Hence consider, at least temporarily *  c->print_function = NULL; */
@@ -867,7 +883,7 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn, Rboolean transp
 	error(_("unknown xtype"));
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     if(transp) {
@@ -1149,7 +1165,7 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
 	error(_("f->xtype of %d not recognized"), f->xtype);
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(class)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(class));
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = dims[1] = f->n;
 				/* copy component of known length */
