@@ -22,24 +22,23 @@ static
 int left_cyclic(double x[], int ldx, int j, int k,
 		double cosines[], double sines[])
 {
-    double *lastcol;
-    int i, jj;
-
     if (j >= k)
 	error(_("incorrect left cyclic shift, j (%d) >= k (%d)"), j, k);
     if (j < 0)
 	error(_("incorrect left cyclic shift, j (%d) < 0"), j, k);
     if (ldx < k)
 	error(_("incorrect left cyclic shift, k (%d) > ldx (%d)"), k, ldx);
-    lastcol = (double*) R_alloc(k+1, sizeof(double));
+
+    double *lastcol = (double*) R_alloc(k+1, sizeof(double));
+    int i;
 				/* keep a copy of column j */
     for(i = 0; i <= j; i++) lastcol[i] = x[i + j*ldx];
 				/* For safety, zero the rest */
     for(i = j+1; i <= k; i++) lastcol[i] = 0.;
-    for(jj = j+1; jj <= k; jj++) { /* columns to be shifted */
-	int diagind = jj*(ldx+1), ind = (jj-j) - 1;
+    for(int jj = j+1, ind = 0; jj <= k; jj++, ind++) { /* columns to be shifted */
+	int diagind = jj*(ldx+1); //  ind == (jj-j) - 1
 	double tmp = x[diagind], cc, ss;
-				/* Calculate the Givens rotation. */
+	/* Calculate the Givens rotation. */
 				/* This modified the super-diagonal element */
 	F77_CALL(drotg)(x + diagind-1, &tmp, cosines + ind, sines + ind);
 	cc = cosines[ind]; ss = sines[ind];
@@ -68,7 +67,7 @@ SEXP getGivens(double x[], int ldx, int jmin, int rank)
     SET_VECTOR_ELT(ans, 0, ScalarInteger(jmin));
     SET_VECTOR_ELT(ans, 1, ScalarInteger(rank));
     SET_VECTOR_ELT(ans, 2, cosines = allocVector(REALSXP, shiftlen));
-    SET_VECTOR_ELT(ans, 3, sines = allocVector(REALSXP, shiftlen));
+    SET_VECTOR_ELT(ans, 3,   sines = allocVector(REALSXP, shiftlen));
     setAttrib(ans, R_NamesSymbol, nms = allocVector(STRSXP, 4));
     SET_STRING_ELT(nms, 0, mkChar("jmin"));
     SET_STRING_ELT(nms, 1, mkChar("rank"));
@@ -99,30 +98,30 @@ SEXP checkGivens(SEXP X, SEXP jmin, SEXP rank)
 SEXP lsq_dense_Chol(SEXP X, SEXP y)
 {
     SEXP ans;
-    int info, n, p, k, *Xdims, *ydims;
-    double *xpx, d_one = 1., d_zero = 0.;
+    double d_one = 1., d_zero = 0.;
 
     if (!(isReal(X) & isMatrix(X)))
 	error(_("X must be a numeric (double precision) matrix"));
-    Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP));
-    n = Xdims[0];
-    p = Xdims[1];
+    int *Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP)),
+	n = Xdims[0],
+	p = Xdims[1];
     if (!(isReal(y) & isMatrix(y)))
 	error(_("y must be a numeric (double precision) matrix"));
-    ydims = INTEGER(coerceVector(getAttrib(y, R_DimSymbol), INTSXP));
+    int *ydims = INTEGER(coerceVector(getAttrib(y, R_DimSymbol), INTSXP));
     if (ydims[0] != n)
 	error(_(
 	    "number of rows in y (%d) does not match number of rows in X (%d)"),
 	    ydims[0], n);
-    k = ydims[1];
+    int k = ydims[1];
     if (k < 1 || p < 1) return allocMatrix(REALSXP, p, k);
     ans = PROTECT(allocMatrix(REALSXP, p, k));
     F77_CALL(dgemm)("T", "N", &p, &k, &n, &d_one, REAL(X), &n, REAL(y), &n,
-		    &d_zero, REAL(ans), &p);
-    xpx = (double *) R_alloc(p * p, sizeof(double));
+		    &d_zero, REAL(ans), &p FCONE FCONE);
+    double *xpx = (double *) R_alloc(p * p, sizeof(double));
     F77_CALL(dsyrk)("U", "T", &p, &n, &d_one, REAL(X), &n, &d_zero,
-		    xpx, &p);
-    F77_CALL(dposv)("U", &p, &k, xpx, &p, REAL(ans), &p, &info);
+		    xpx, &p FCONE FCONE);
+    int info;
+    F77_CALL(dposv)("U", &p, &k, xpx, &p, REAL(ans), &p, &info FCONE);
     if (info) error(_("Lapack routine dposv returned error code %d"), info);
     UNPROTECT(1);
     return ans;
@@ -131,37 +130,32 @@ SEXP lsq_dense_Chol(SEXP X, SEXP y)
 
 SEXP lsq_dense_QR(SEXP X, SEXP y)
 {
-    SEXP ans;
-    int info, n, p, k, *Xdims, *ydims, lwork;
-    double *work, tmp, *xvals;
-
     if (!(isReal(X) & isMatrix(X)))
 	error(_("X must be a numeric (double precision) matrix"));
-    Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP));
-    n = Xdims[0];
-    p = Xdims[1];
+    int *Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP)),
+	n = Xdims[0],
+	p = Xdims[1];
     if (!(isReal(y) & isMatrix(y)))
 	error(_("y must be a numeric (double precision) matrix"));
-    ydims = INTEGER(coerceVector(getAttrib(y, R_DimSymbol), INTSXP));
+    int *ydims = INTEGER(coerceVector(getAttrib(y, R_DimSymbol), INTSXP));
     if (ydims[0] != n)
 	error(_(
 	    "number of rows in y (%d) does not match number of rows in X (%d)"),
 	    ydims[0], n);
-    k = ydims[1];
+    int k = ydims[1];
     if (k < 1 || p < 1) return allocMatrix(REALSXP, p, k);
-    xvals = (double *) R_alloc(n * p, sizeof(double));
-    Memcpy(xvals, REAL(X), n * p);
-    ans = PROTECT(duplicate(y));
-    lwork = -1;
+    double tmp, *xvals = (double *) Memcpy(R_alloc(n * p, sizeof(double)), REAL(X), n * p);
+    SEXP ans = PROTECT(duplicate(y));
+    int lwork = -1, info;
     F77_CALL(dgels)("N", &n, &p, &k, xvals, &n, REAL(ans), &n,
-		    &tmp, &lwork, &info);
+		    &tmp, &lwork, &info FCONE);
     if (info)
 	error(_("First call to Lapack routine dgels returned error code %d"),
 	      info);
     lwork = (int) tmp;
-    work = (double *) R_alloc(lwork, sizeof(double));
+    double *work = (double *) R_alloc(lwork, sizeof(double));
     F77_CALL(dgels)("N", &n, &p, &k, xvals, &n, REAL(ans), &n,
-		    work, &lwork, &info);
+		    work, &lwork, &info FCONE);
     if (info)
 	error(_("Second call to Lapack routine dgels returned error code %d"),
 	      info);
@@ -169,35 +163,47 @@ SEXP lsq_dense_QR(SEXP X, SEXP y)
     return ans;
 }
 
+/* Rank-Correcting/Adapting LAPACK  QR Decomposition
+ * From Doug Bates' initial import; __unused__
+
+ * Provides a qr() with 'rcond' and rank reduction while(rcond < tol),
+ * possibly via Givens rotations but WITHOUT PIVOTING
+
+ * .Call(Matrix:::lapack_qr, A, 1e-17) --> ~/R/MM/Pkg-ex/Matrix/qr-rank-deficient.R
+
+ * TODO: export as Matrix::qrNoPiv() or qr1()  or similar
+ */
 SEXP lapack_qr(SEXP Xin, SEXP tl)
 {
-    SEXP ans, Givens, Gcpy, nms, pivot, qraux, X;
-    int i, n, nGivens = 0, p, trsz, *Xdims, rank;
-    double rcond = 0., tol = asReal(tl), *work;
-
     if (!(isReal(Xin) & isMatrix(Xin)))
 	error(_("X must be a real (numeric) matrix"));
+    double tol = asReal(tl);
     if (tol < 0.) error(_("tol, given as %g, must be non-negative"), tol);
     if (tol > 1.) error(_("tol, given as %g, must be <= 1"), tol);
-    ans = PROTECT(allocVector(VECSXP,5));
+    SEXP ans = PROTECT(allocVector(VECSXP,5)), X, qraux, pivot;
     SET_VECTOR_ELT(ans, 0, X = duplicate(Xin));
-    Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP));
-    n = Xdims[0]; p = Xdims[1];
-    SET_VECTOR_ELT(ans, 2, qraux = allocVector(REALSXP, (n < p) ? n : p));
+    int *Xdims = INTEGER(coerceVector(getAttrib(X, R_DimSymbol), INTSXP)),
+	n = Xdims[0], i,
+	p = Xdims[1],
+	trsz = (n < p) ? n : p ; /* size of triangular part of decomposition */
+
+    SET_VECTOR_ELT(ans, 2, qraux = allocVector(REALSXP, trsz));
     SET_VECTOR_ELT(ans, 3, pivot = allocVector(INTSXP, p));
     for (i = 0; i < p; i++) INTEGER(pivot)[i] = i + 1;
-    trsz = (n < p) ? n : p;	/* size of triangular part of decomposition */
-    rank = trsz;
-    Givens = PROTECT(allocVector(VECSXP, rank - 1));
+    SEXP nms,
+	Givens = PROTECT(allocVector(VECSXP, trsz - 1));
     setAttrib(ans, R_NamesSymbol, nms = allocVector(STRSXP, 5));
     SET_STRING_ELT(nms, 0, mkChar("qr"));
     SET_STRING_ELT(nms, 1, mkChar("rank"));
     SET_STRING_ELT(nms, 2, mkChar("qraux"));
     SET_STRING_ELT(nms, 3, mkChar("pivot"));
     SET_STRING_ELT(nms, 4, mkChar("Givens"));
+    int rank = trsz,
+	nGivens = 0;
+    double rcond = 0.;
     if (n > 0 && p > 0) {
 	int  info, *iwork, lwork;
-	double *xpt = REAL(X), tmp;
+	double *xpt = REAL(X), *work, tmp;
 
 	lwork = -1;
 	F77_CALL(dgeqrf)(&n, &p, xpt, &n, REAL(qraux), &tmp, &lwork, &info);
@@ -211,15 +217,15 @@ SEXP lapack_qr(SEXP Xin, SEXP tl)
 	    error(_("Second call to dgeqrf returned error code %d"), info);
 	iwork = (int *) R_alloc(trsz, sizeof(int));
 	F77_CALL(dtrcon)("1", "U", "N", &rank, xpt, &n, &rcond,
-			 work, iwork, &info);
+			 work, iwork, &info FCONE FCONE FCONE);
 	if (info)
 	    error(_("Lapack routine dtrcon returned error code %d"), info);
 	while (rcond < tol) {	/* check diagonal elements */
 	    double minabs = (xpt[0] < 0.) ? -xpt[0]: xpt[0];
 	    int jmin = 0;
 	    for (i = 1; i < rank; i++) {
-		double el = xpt[i*(n+1)];
-		el = (el < 0.) ? -el: el;
+		double el = xpt[i*n]; // had  i*(n+1)  which looks wrong to MM
+		if(el < 0.) el = -el;
 		if (el < minabs) {
 		    jmin = i;
 		    minabs = el;
@@ -228,30 +234,38 @@ SEXP lapack_qr(SEXP Xin, SEXP tl)
 	    if (jmin < (rank - 1)) {
 		SET_VECTOR_ELT(Givens, nGivens, getGivens(xpt, n, jmin, rank));
 		nGivens++;
-	    }
+	    } // otherwise jmin == (rank - 1) , so just "drop that column"
 	    rank--;
+	    // new  rcond := ... for reduced rank
 	    F77_CALL(dtrcon)("1", "U", "N", &rank, xpt, &n, &rcond,
-			     work, iwork, &info);
+			     work, iwork, &info FCONE FCONE FCONE);
 	    if (info)
 		error(_("Lapack routine dtrcon returned error code %d"), info);
 	}
     }
+    SEXP Gcpy, sym;
     SET_VECTOR_ELT(ans, 4, Gcpy = allocVector(VECSXP, nGivens));
     for (i = 0; i < nGivens; i++)
 	SET_VECTOR_ELT(Gcpy, i, VECTOR_ELT(Givens, i));
     SET_VECTOR_ELT(ans, 1, ScalarInteger(rank));
-    setAttrib(ans, install("useLAPACK"), ScalarLogical(1));
-    setAttrib(ans, install("rcond"), ScalarReal(rcond));
+    sym = PROTECT(install("useLAPACK")); setAttrib(ans, sym, ScalarLogical(1)); UNPROTECT(1);
+    sym = PROTECT(install("rcond"));     setAttrib(ans, sym, ScalarReal(rcond));UNPROTECT(1);
     UNPROTECT(2);
     return ans;
 }
 
 SEXP dense_to_Csparse(SEXP x)
 {
-    CHM_DN chxd = AS_CHM_DN(PROTECT(mMatrix_as_geMatrix(x)));
+    SEXP ge_x = PROTECT(mMatrix_as_geMatrix(x)),
+	Dim = GET_SLOT(ge_x, Matrix_DimSym);
+    int *dims = INTEGER(Dim);
+    Rboolean longi = (dims[0] * (double)dims[1] > INT_MAX);
+    // int itype = longi ? CHOLMOD_LONG : CHOLMOD_INT;
+    CHM_DN chxd = AS_CHM_xDN(ge_x); // cholmod_dense (has no itype)
+    CHM_SP chxs;
     /* cholmod_dense_to_sparse() in CHOLMOD/Core/ below does only work for
        "REAL" 'xtypes', i.e. *not* for "nMatrix".
-       ===> need "_x" in above call.
+       ===> need "_x" in above AS_CHM_xDN() call.
 
        Also it cannot keep symmetric / triangular, hence the
        as_geMatrix() above.  Note that this is already a *waste* for
@@ -259,7 +273,31 @@ SEXP dense_to_Csparse(SEXP x)
        enhanced cholmod_dense_to_sparse(), with an extra boolean
        argument for symmetry.
     */
-    CHM_SP chxs = cholmod_dense_to_sparse(chxd, 1, &c);
+#define DLONG
+/* You can try defining DLONG -- then just get a seg.fault :
+ *  I think it is because of this in  ./CHOLMOD/Include/cholmod_core.h :
+ *
+ The itype of all parameters for all CHOLMOD routines must match.
+ --- ^^^^^ ------------------------------------------------------
+ but then as_cholmod_dense should *not* make a difference: cholmod_dense has *no* itype   (????)
+*/
+    if(longi) { // calling cholmod_dense_to_sparse() gives wrong matrix
+#ifdef DLONG
+	chxs = cholmod_l_dense_to_sparse(chxd, 1, &cl);
+	// in gdb, I found that 'chxs' seems "basically empty": all
+	// p chxs->foo   give ''Cannot access memory at address 0x....''
+	// for now rather give error:
+	if(cl.status)
+	    error(_("dense_to_Csparse(<LARGE>): cholmod_l_dense_to_sparse failure status=%d"),
+		  cl.status);
+#else
+        error(_("Matrix dimension %d x %d (= %g) too large [FIXME calling cholmod_l_dense_to_sparse]"),
+	      m,n, m * (double)n);
+#endif
+    } else { // fits, using integer (instead of long int) 'itype'
+	chxs = cholmod_dense_to_sparse(chxd, 1, &c);
+    }
+
     int Rkind = (chxd->xtype == CHOLMOD_REAL) ? Real_KIND2(x) : 0;
     /* Note: when 'x' was integer Matrix, Real_KIND(x) = -1, but *_KIND2(.) = 0 */
     R_CheckStack();
@@ -292,9 +330,8 @@ SEXP dense_band(SEXP x, SEXP k1P, SEXP k2P)
 	    sqr = (adims[0] == adims[1]),
 	    tru = (k1 >= 0), trl = (k2 <= 0);
 	const char *cl = class_P(ans);
-	enum dense_enum { ddense, ldense, ndense
-	} M_type = ( (cl[0] == 'd') ? ddense :
-		    ((cl[0] == 'l') ? ldense : ndense));
+	enum dense_enum M_type = ( (cl[0] == 'd') ? ddense :
+			      ((cl[0] == 'l') ? ldense : ndense));
 
 
 #define SET_ZERO_OUTSIDE				\
@@ -321,9 +358,8 @@ SEXP dense_band(SEXP x, SEXP k1P, SEXP k2P)
 	}
 	else {
 	    /* Copy ans to a *trMatrix object (must be square) */
-	    SEXP aa= PROTECT(NEW_OBJECT(MAKE_CLASS(M_type == ddense? "dtrMatrix":
-						   (M_type== ldense? "ltrMatrix"
-						    : "ntrMatrix"))));
+	    SEXP aa= PROTECT(NEW_OBJECT_OF_CLASS(M_type == ddense? "dtrMatrix":
+						 (M_type== ldense? "ltrMatrix": "ntrMatrix")));
 	    /* Because slots of ans are freshly allocated and ans will not be
 	     * used, we use the slots themselves and don't duplicate */
 	    SET_SLOT(aa, Matrix_xSym,       GET_SLOT(ans, Matrix_xSym));
@@ -340,14 +376,17 @@ SEXP dense_band(SEXP x, SEXP k1P, SEXP k2P)
 SEXP dense_to_symmetric(SEXP x, SEXP uplo, SEXP symm_test)
 /* Class of result will be [dln]syMatrix */
 {
+/*== FIXME: allow  uplo = NA   and then behave a bit like symmpart():
+ *== -----  would use the *dimnames* to determine U or L   (??)
+ */
+
     int symm_tst = asLogical(symm_test);
     SEXP dx = PROTECT(dup_mMatrix_as_geMatrix(x));
-    SEXP ans, dns;
+    SEXP ans, dns, nms_dns;
     const char *cl = class_P(dx);
     /* same as in ..._geMatrix() above:*/
-    enum dense_enum { ddense, ldense, ndense
-    } M_type = ( (cl[0] == 'd') ? ddense :
-		((cl[0] == 'l') ? ldense : ndense));
+    enum dense_enum M_type = ( (cl[0] == 'd') ? ddense :
+			  ((cl[0] == 'l') ? ldense : ndense));
     int *adims = INTEGER(GET_SLOT(dx, Matrix_DimSym)), n = adims[0];
     if(n != adims[1]) {
 	UNPROTECT(1);
@@ -357,7 +396,7 @@ SEXP dense_to_symmetric(SEXP x, SEXP uplo, SEXP symm_test)
 
     if(symm_tst) {
 	int i,j;
-#define CHECK_SYMMETRIC							\
+#       define CHECK_SYMMETRIC						\
 	for (j = 0; j < n; j++)						\
 	    for (i = 0; i < j; i++)					\
 		if(xx[j * n + i] != xx[i * n + j]) {			\
@@ -376,29 +415,42 @@ SEXP dense_to_symmetric(SEXP x, SEXP uplo, SEXP symm_test)
 	    CHECK_SYMMETRIC
 	}
     }
+#   undef CHECK_SYMMETRIC
 
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(M_type == ddense ? "dsyMatrix" :
+				      (M_type == ldense ? "lsyMatrix" : "nsyMatrix")));
+
+// --- FIXME: Use MK_SYMMETRIC_DIMNAMES_AND_RETURN  from below -- with "uplo" argument
+
+    /* need _symmetric_ dimnames */
     dns = GET_SLOT(dx, Matrix_DimNamesSym);
     if(!equal_string_vectors(VECTOR_ELT(dns,0),
 			     VECTOR_ELT(dns,1))) {
-	/* need _symmetric_ dimnames */
 	if(*CHAR(asChar(uplo)) == 'U')
 	    SET_VECTOR_ELT(dns,0, VECTOR_ELT(dns,1));
 	else
 	    SET_VECTOR_ELT(dns,1, VECTOR_ELT(dns,0));
     }
+    nms_dns = PROTECT(getAttrib(dns, R_NamesSymbol));
+    if(!isNull(nms_dns) &&
+       !R_compute_identical(STRING_ELT(nms_dns, 0),
+			    STRING_ELT(nms_dns, 1), 16)) { // names(dimnames(.)) :
+	if(*CHAR(asChar(uplo)) == 'U')
+	    SET_STRING_ELT(nms_dns, 0, STRING_ELT(nms_dns,1));
+	else
+	    SET_STRING_ELT(nms_dns, 1, STRING_ELT(nms_dns,0));
+	setAttrib(dns, R_NamesSymbol, nms_dns);
+    }
 
     /* Copy dx to ans;
      * Because slots of dx are freshly allocated and dx will not be
      * used, we use the slots themselves and don't duplicate */
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS( M_type == ddense ? "dsyMatrix" :
-					(M_type == ldense ? "lsyMatrix" :
-					 "nsyMatrix"))));
     SET_SLOT(ans, Matrix_xSym,	      GET_SLOT(dx, Matrix_xSym));
     SET_SLOT(ans, Matrix_DimSym,      GET_SLOT(dx, Matrix_DimSym));
     SET_SLOT(ans, Matrix_DimNamesSym, dns);
     SET_SLOT(ans, Matrix_uploSym,     ScalarString(asChar(uplo)));
 
-    UNPROTECT(2);
+    UNPROTECT(3);
     return ans;
 }
 
@@ -409,40 +461,54 @@ SEXP ddense_symmpart(SEXP x)
     int *adims = INTEGER(GET_SLOT(dx, Matrix_DimSym)), n = adims[0];
 
     if(n != adims[1]) {
-	UNPROTECT(1);
 	error(_("matrix is not square! (symmetric part)"));
 	return R_NilValue; /* -Wall */
     } else {
-	SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dsyMatrix"))), dns;
+	SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dsyMatrix")), dns, nms_dns;
 	double *xx = REAL(GET_SLOT(dx, Matrix_xSym));
-	int i,j;
 
 	/* only need to assign the *upper* triangle (uplo = "U");
 	 * noting that diagonal remains unchanged */
-	for (j = 0; j < n; j++) {
-	    for (i = 0; i < j; i++) {
+	for (int j = 0; j < n; j++) {
+	    for (int i = 0; i < j; i++) {
 		xx[j * n + i] = (xx[j * n + i] + xx[i * n + j]) / 2.;
 	    }
 	}
 
-	dns = GET_SLOT(dx, Matrix_DimNamesSym);
-	if(!equal_string_vectors(VECTOR_ELT(dns,0),
-				 VECTOR_ELT(dns,1))) {
-	    /* need _symmetric_ dimnames */
-	    SET_VECTOR_ELT(dns,0, VECTOR_ELT(dns,1));/* ==> uplo = "U" */
-	}
+// FIXME?: Compare and synchronize with symmetric_DimNames() in ./Mutils.c
+#       define MK_SYMMETRIC_DIMNAMES_AND_RETURN				\
+									\
+	dns = GET_SLOT(dx, Matrix_DimNamesSym);				\
+	int J = 1;							\
+	if(!equal_string_vectors(VECTOR_ELT(dns,0),			\
+				 VECTOR_ELT(dns,1))) {			\
+	    /* _symmetric_ dimnames: behave as symmDN(*, col=TRUE) */	\
+	    if(isNull(VECTOR_ELT(dns, J)))				\
+		J = !J;							\
+	    SET_VECTOR_ELT(dns, !J, VECTOR_ELT(dns, J));		\
+	}								\
+	/* names(dimnames(.)): */					\
+	nms_dns = PROTECT(getAttrib(dns, R_NamesSymbol));               \
+	if(!isNull(nms_dns) &&						\
+	   !R_compute_identical(STRING_ELT(nms_dns, 0),			\
+				STRING_ELT(nms_dns, 1), 16)) { 		\
+	    SET_STRING_ELT(nms_dns, !J, STRING_ELT(nms_dns, J));	\
+	    setAttrib(dns, R_NamesSymbol, nms_dns);			\
+	}								\
+									\
+	/* Copy dx to ans;						\
+	 * Because slots of dx are freshly allocated and dx will not be	\
+	 * used, we use the slots themselves and don't duplicate */	\
+									\
+	SET_SLOT(ans, Matrix_xSym,	  GET_SLOT(dx, Matrix_xSym));	\
+	SET_SLOT(ans, Matrix_DimSym,	  GET_SLOT(dx, Matrix_DimSym));	\
+	SET_SLOT(ans, Matrix_DimNamesSym, dns);				\
+	SET_SLOT(ans, Matrix_uploSym,	  mkString("U"));		\
+									\
+	UNPROTECT(3);							\
+	return ans
 
-	/* Copy dx to ans;
-	 * Because slots of dx are freshly allocated and dx will not be
-	 * used, we use the slots themselves and don't duplicate */
-
-	SET_SLOT(ans, Matrix_xSym,	  GET_SLOT(dx, Matrix_xSym));
-	SET_SLOT(ans, Matrix_DimSym,	  GET_SLOT(dx, Matrix_DimSym));
-	SET_SLOT(ans, Matrix_DimNamesSym, dns);
-	SET_SLOT(ans, Matrix_uploSym,	  mkString("U"));
-
-	UNPROTECT(2);
-	return ans;
+        MK_SYMMETRIC_DIMNAMES_AND_RETURN;
     }
 }
 
@@ -453,40 +519,21 @@ SEXP ddense_skewpart(SEXP x)
     int *adims = INTEGER(GET_SLOT(dx, Matrix_DimSym)), n = adims[0];
 
     if(n != adims[1]) {
-	UNPROTECT(1);
 	error(_("matrix is not square! (skew-symmetric part)"));
 	return R_NilValue; /* -Wall */
     } else {
-	SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))), dns;
+	SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")), dns, nms_dns;
 	double *xx = REAL(GET_SLOT(dx, Matrix_xSym));
-	int i,j;
 
-	for (j = 0; j < n; j++) {
+	for (int j = 0; j < n; j++) {
 	    xx[j * n + j] = 0.;
-	    for (i = 0; i < j; i++) {
+	    for (int i = 0; i < j; i++) {
 		double s = (xx[j * n + i] - xx[i * n + j]) / 2.;
 		xx[j * n + i] =  s;
 		xx[i * n + j] = -s;
 	    }
 	}
 
-	dns = GET_SLOT(dx, Matrix_DimNamesSym);
-	if(!equal_string_vectors(VECTOR_ELT(dns,0),
-				 VECTOR_ELT(dns,1))) {
-	    /* need _symmetric_ dimnames */
-	    SET_VECTOR_ELT(dns,0, VECTOR_ELT(dns,1));/* uplo = "U" */
-	}
-
-	/* Copy dx to ans;
-	 * Because slots of dx are freshly allocated and dx will not be
-	 * used, we use the slots themselves and don't duplicate */
-
-	SET_SLOT(ans, Matrix_xSym,	  GET_SLOT(dx, Matrix_xSym));
-	SET_SLOT(ans, Matrix_DimSym,	  GET_SLOT(dx, Matrix_DimSym));
-	SET_SLOT(ans, Matrix_DimNamesSym, dns);
-	SET_SLOT(ans, Matrix_uploSym,	  mkString("U"));
-
-	UNPROTECT(2);
-	return ans;
+        MK_SYMMETRIC_DIMNAMES_AND_RETURN;
     }
 }
