@@ -3,7 +3,7 @@
  *  file console.c
  *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *  Copyright (C) 2004-8      The R Foundation
- *  Copyright (C) 2004-2014   The R Core Team
+ *  Copyright (C) 2004-2020   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -261,6 +261,7 @@ static size_t enctowcs(wchar_t *wc, char *s, int n)
        *(pb+2) == UTF8in[2]) {
 	*pb = '\0';
 	nc += mbstowcs(wc, s, n);
+	*pb = UTF8in[0]; /* preserve input string, needed in consolewrites */
 	pb += 3; pe = pb;
 	while(*pe &&
 	      !((pe = strchr(pe, UTF8out[0])) && *(pe+1) == UTF8out[1] &&
@@ -269,6 +270,7 @@ static size_t enctowcs(wchar_t *wc, char *s, int n)
 	*pe = '\0';
 	/* convert string starting at pb from UTF-8 */
 	nc += Rf_utf8towcs(wc+nc, pb, (pe-pb));
+	*pe = UTF8out[0]; /* preserve input string, needed in consolewrites */
 	pe += 3;
 	nc += enctowcs(wc+nc, pe, n-nc);
     } else nc = mbstowcs(wc, s, n);
@@ -342,7 +344,7 @@ newconsoledata(font f, int rows, int cols, int bufbytes, int buflines,
 	p->lbuf = NULL;
 	p->kbuf = NULL;
     }
-    p->bm = NULL;
+    BM = NULL;
     p->rows = rows;
     p->cols = cols;
     for (int i=0; i<numGuiColors; i++)
@@ -353,10 +355,10 @@ newconsoledata(font f, int rows, int cols, int bufbytes, int buflines,
     WIDTH = (COLS + 1) * FW;
     HEIGHT = (ROWS + 1) * FH + 1; /* +1 avoids size problems in MDI */
     FV = FC = 0;
-    p->newfv = p->newfc = 0;
+    NEWFV = NEWFC = 0;
     p->firstkey = p->numkeys = 0;
     p->clp = NULL;
-    p->r = -1;
+    CURROW = -1;
     p->overwrite = 0;
     p->needredraw = 0;
     p->wipe_completion = 0;
@@ -427,7 +429,7 @@ static void writelineHelper(ConsoleData p, int fch, int lch,
 
     /* This is right, since columns are of fixed size */
     r = rect(BORDERX + fch * FW, BORDERY + j * FH, (lch - fch + 1) * FW, FH);
-    gfillrect(p->bm, bgr, r);
+    gfillrect(BM, bgr, r);
 
     if (len > FC+fch) {
 	/* Some of the string is visible: */
@@ -461,7 +463,7 @@ static void writelineHelper(ConsoleData p, int fch, int lch,
 	    if((len > FC+COLS) && (lch == COLS - 1)) *q++ = L'$';
 	    else *q++ = *P++;
 	    *q = L'\0';
-	    gdrawwcs(p->bm, p->f, fgr, pt(r.x, r.y), buff);
+	    gdrawwcs(BM, p->f, fgr, pt(r.x, r.y), buff);
 	} else {
 	    int last;
 	    wchar_t ch, chf, chl;
@@ -472,7 +474,7 @@ static void writelineHelper(ConsoleData p, int fch, int lch,
 	    } else chl = L'\0';
 	    last = FC + lch + 1;
 	    if (len > last) {ch = s[last]; s[last] = L'\0';} else ch = L'\0';
-	    gdrawwcs(p->bm, p->f, fgr, pt(r.x, r.y), &s[FC+fch]);
+	    gdrawwcs(BM, p->f, fgr, pt(r.x, r.y), &s[FC+fch]);
 	    /* restore the string */
 	    if (ch) s[last] = ch;
 	    if (chl) s[FC+lch] = chl;
@@ -549,7 +551,7 @@ static int writeline(control c, ConsoleData p, int i, int j)
     } else
 	WLHELPER(0, col1, fg, bg);
     /* This is the cursor, and it may need to be variable-width */
-    if ((p->r >= 0) && (CURCOL >= FC) && (CURCOL < FC + COLS) &&
+    if ((CURROW >= 0) && (CURCOL >= FC) && (CURCOL < FC + COLS) &&
 	(i == NUMLINES - 1) && (p->sel == 0 || !intersect_input(p, 0))) {
 	if (!p->overwrite) {
 	    if (p->cursor_blink) {
@@ -560,7 +562,7 @@ static int writeline(control c, ConsoleData p, int i, int j)
 	    
 	    if (p->cursor_blink < 2) {
 	    	r = rect(BORDERX + (CURCOL - FC) * FW, BORDERY + j * FH, FW/4, FH);
-	    	gfillrect(p->bm, highlight, r);
+	    	gfillrect(BM, highlight, r);
 	    }
 	} else if(mbcslocale) { /* determine the width of the current char */
 	    int w0;
@@ -581,8 +583,8 @@ static int writeline(control c, ConsoleData p, int i, int j)
 	    if (p->cursor_blink < 2) {
 	    	r = rect(BORDERX + (CURCOL - FC) * FW, BORDERY + j * FH,
 		         w0 * FW, FH);
-	    	gfillrect(p->bm, highlight, r);
-	    	gdrawwcs(p->bm, p->f, bg, pt(r.x, r.y), nn);
+	    	gfillrect(BM, highlight, r);
+	    	gdrawwcs(BM, p->f, bg, pt(r.x, r.y), nn);
 	    }
 	} else {
 	    if (p->cursor_blink) {
@@ -906,7 +908,7 @@ void set_completion_available(int x)
 static void performCompletion(control c)
 {
     ConsoleData p = getdata(c);
-    int i, alen, alen2, max_show = 10, cursor_position = p->c - prompt_wid;
+    int i, alen, alen2, max_show = 10, cursor_position = CURCOL - prompt_wid;
     wchar_t *partial_line = LINE(NUMLINES - 1) + prompt_wid;
     const char *additional_text;
     SEXP cmdSexp, cmdexpr, ans = R_NilValue;
@@ -945,14 +947,16 @@ static void performCompletion(control c)
 	}
     }
 
-    /* FIXME: need to escape quotes properly */
-    wchar_t *pline = (wchar_t*) R_alloc(wcslen(partial_line) + 1, sizeof(wchar_t));
-    wcscpy(pline, partial_line);
-    /* poor attempt at escaping quotes that sort of works */
-    alen = wcslen(pline);
-    for (i = 0; i < alen; i++)
-	if (pline[i] == '"') pline[i] = L'\'';
-
+    alen = wcslen(partial_line);
+    wchar_t orig[alen + 1], pline[2*alen + 1],
+            *pchar = pline, achar;
+    wcscpy(orig, partial_line);
+    for (i = 0; i < alen; i++) {
+        achar = orig[i];
+	if (achar == '"' || achar == '\\') *pchar++ = '\\';
+	*pchar++ = achar;
+    }
+    *pchar = 0;
     size_t len = wcslen(pline) + 100; 
     char cmd[len];
     snprintf(cmd, len, "utils:::.win32consoleCompletion(\"%ls\", %d)",
@@ -987,11 +991,12 @@ static void performCompletion(control c)
     alen2 = strlen(additional_text);
     if (alen) {
 	/* make a copy of the current string first */
-	wchar_t *p1 = LINE(NUMLINES - 1);
+	wchar_t p1[wcslen(LINE(NUMLINES - 1)) + 1];
+	wcscpy(p1, LINE(NUMLINES - 1));
 	checkpointpos(p->lbuf, 1);
 	size_t len = MB_CUR_MAX * wcslen(p1) + 1; 
-	char buf1[len];
-	snprintf(buf1, len, "%ls\n", p1);
+	char buf1[len+1];
+	snprintf(buf1, len+1, "%ls\n", p1);
 	consolewrites(c, buf1);
 
 	for (i = 0; i < min(alen, max_show); i++) {
@@ -1059,9 +1064,9 @@ void consolecmd(control c, const char *cmd)
     storekey(c, '\n');
 /* if we are editing we save the actual line
    FIXME: not right if Unicode */
-    if (p->r > -1) {
+    if (CURROW > -1) {
 	char buf[2000], *cp; /* maximum 2 bytes/char */
-	wchar_t *wc = &(p->lbuf->s[p->lbuf->ns - 1][prompt_len]);
+	wchar_t *wc = &(LINE(NUMLINES - 1)[prompt_len]);
 	memset(buf, 0, 2000);
 	wcstombs(buf, wc, 1000);
 	for (cp = buf; *cp; cp++) storekey(c, *cp);
@@ -1273,7 +1278,9 @@ static void consoletoclipboardHelper(control c, int x0, int y0, int x1, int y1)
 		w0 += Ri18n_wcwidth(*P);
 		*s++ = *P++;
 	    }
-	    if(w0 < x11) *s++ = L'\r'; *s++ = L'\n';
+	    if(w0 < x11) {
+		*s++ = L'\r'; *s++ = L'\n';
+	    }
 	    i++;
 	}
     } else {
@@ -1531,7 +1538,7 @@ int consolewrites(control c, const char *s)
 	/* now zap it */
 	for(i = 0; i < len; i++) xbufaddxc(p->lbuf, L'\b');
 	if (incomplete) {
-	    p->lbuf->ns--;
+	    NUMLINES--;
 	    p->lbuf->free--;
 	    p->lbuf->av++;
 	}
@@ -1548,11 +1555,11 @@ int consolewrites(control c, const char *s)
     if (!p->lazyupdate) {
 	setfirstvisible(c, NUMLINES - ROWS);
 	REDRAW;
-    } else if (p->r >= 0)
+    } else if (CURROW >= 0)
 	setfirstvisible(c, NUMLINES - ROWS);
     else {
-	p->newfv = NUMLINES - ROWS;
-	if (p->newfv < 0) p->newfv = 0;
+	NEWFV = NUMLINES - ROWS;
+	if (NEWFV < 0) NEWFV = 0;
     }
     if(p->input) REDRAW;
     return 0;
@@ -1561,7 +1568,7 @@ int consolewrites(control c, const char *s)
 void freeConsoleData(ConsoleData p)
 {
     if (!p) return;
-    if (p->bm) del(p->bm);
+    if (BM) del(BM);
     if (p->kind == CONSOLE) {
 	if (p->lbuf) xbufdel(p->lbuf);
 	if (p->kbuf) free(p->kbuf);
@@ -1666,8 +1673,8 @@ static void draweditline(control c)
     if (p->needredraw) {
 	REDRAW;
     } else {
-	WRITELINE(NUMLINES - 1, p->r);
-	RSHOW(RLINE(p->r));
+	WRITELINE(NUMLINES - 1, CURROW);
+	RSHOW(RLINE(CURROW));
     }
 }
 
@@ -1684,10 +1691,20 @@ static void wcstobuf(char *buf, int len, const wchar_t *in)
     int used, tot = 0;
     char *p = buf, tmp[7];
     const wchar_t *wc = in;
+    wchar_t wc_check;
+    mbstate_t mb_st;
 
     for(; wc; wc++, p+=used, tot+=used) {
 	if(tot >= len - 2) break;
 	used = wctomb(p, *wc);
+	if (used >= 0) {
+	    /* conversion was successful, but check that converting back gets
+	       the original result (it does not with best-fit transliteration)
+	       NOTE: WideCharToMultiByte may be faster */
+	    memset(&mb_st, 0, sizeof(mbstate_t));
+	    if (mbrtowc(&wc_check, p, used, &mb_st) < 0 || wc_check != *wc) 
+		used = -1;
+	}
 	if (used < 0) {
 	    snprintf(tmp, 7, "\\u%x", *wc);
 	    used = strlen(tmp);
@@ -1718,14 +1735,14 @@ int consolereads(control c, const char *prompt, char *buf, int len,
     USER(NUMLINES - 1) = w0;
     prompt_wid = wcswidth(aLine);
     if (NUMLINES > ROWS) {
-	p->r = ROWS - 1;
-	p->newfv = NUMLINES - ROWS;
+	CURROW = ROWS - 1;
+	NEWFV = NUMLINES - ROWS;
     } else {
-	p->r = NUMLINES - 1;
-	p->newfv = 0;
+	CURROW = NUMLINES - 1;
+	NEWFV = 0;
     }
     CURCOL = prompt_wid;
-    p->fc = 0;
+    FC = 0;
     cur_pos = 0;
     max_pos = 0;
     cur_line = &aLine[prompt_len];
@@ -1743,11 +1760,11 @@ int consolereads(control c, const char *prompt, char *buf, int len,
 	    cur_line = LINE(NUMLINES - 1) + prompt_len;
 	    ns0 = NUMLINES;
 	    if (NUMLINES > ROWS) {
-		p->r = ROWS - 1;
-		p->newfv = NUMLINES - ROWS;
+		CURROW = ROWS - 1;
+		NEWFV = NUMLINES - ROWS;
 	    } else {
-		p->r = NUMLINES - 1;
-		p->newfv = 0;
+		CURROW = NUMLINES - 1;
+		NEWFV = 0;
 	    }
 	    USER(NUMLINES - 1) = prompt_wid;
 	    p->needredraw = 1;
@@ -1842,7 +1859,7 @@ int consolereads(control c, const char *prompt, char *buf, int len,
 		    wcstobuf(buf, len, cur_line);
 		    //sprintf(buf, "%ls", cur_line);
 		    //if(strlen(buf) == 0) strcpy(buf, "invalid input\n");
-		    p->r = -1;
+		    CURROW = -1;
 		    cur_line[max_pos] = L'\0';
 		    if (max_pos && addtohistory) wgl_histadd(cur_line);
 		    xbuffixl(p->lbuf);
@@ -1884,7 +1901,7 @@ int consolecols(console c)
 {
     ConsoleData p = getdata(c);
 
-    return p->cols;
+    return COLS;
 }
 
 void consoleresize(console c, rect r)
@@ -1911,6 +1928,8 @@ void consoleresize(console c, rect r)
     HEIGHT = r.height;
     BORDERX = (WIDTH - COLS*FW) / 2;
     BORDERY = (HEIGHT - ROWS*FH) / 2;
+    NEWFV = NUMLINES - ROWS;
+    if (NEWFV < 0) NEWFV = 0;
     del(BM);
     BM = newbitmap(r.width, r.height, 2);
     if (!BM) {
@@ -1919,11 +1938,11 @@ void consoleresize(console c, rect r)
     }
     if(!p->lbuf) return;;    /* don't implement resize if no content
 				   yet in pager */
-    if (p->r >= 0) {
+    if (CURROW >= 0) {
 	if (NUMLINES > ROWS) {
-	    p->r = ROWS - 1;
+	    CURROW = ROWS - 1;
 	} else
-	    p->r = NUMLINES - 1;
+	    CURROW = NUMLINES - 1;
     }
     clear(c);
     p->needredraw = 1;
@@ -2273,6 +2292,6 @@ void consoleclear(control c)
     xbufshift(l);
     l->shift = oldshift;
     NEWFV = 0;
-    p->r = 0;
+    CURROW = 0;
     REDRAW;
 }
