@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1996 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2001 The R Core Team
+ *  Copyright (C) 1997-2014 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /*  Dynamic Loading Support: See ../main/Rdynload.c and ../include/Rdynpriv.h
@@ -53,6 +53,7 @@ int setDLLSearchPath(const char *path)
     PSDD p = (PSDD) -1;
     static char wd[MAX_PATH] = "";  /* stored real current directory */
 
+    // XP SP1 and later.
     if(p == (PSDD) -1)
 	p = (PSDD) GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")),
 				  "SetDllDirectoryA");
@@ -88,7 +89,9 @@ static void fixPath(char *path)
 static HINSTANCE R_loadLibrary(const char *path, int asLocal, int now,
 			       const char *search);
 static DL_FUNC getRoutine(DllInfo *info, char const *name);
+#ifdef CACHE_DLL_SYM
 static void R_deleteCachedSymbols(DllInfo *dll);
+#endif
 
 static void R_getDLLError(char *buf, int len);
 static void GetFullDLLPath(SEXP call, char *buf, const char *path);
@@ -105,13 +108,16 @@ void InitFunctionHashing()
     R_osDynSymbol->closeLibrary = closeLibrary;
     R_osDynSymbol->getError = R_getDLLError;
 
+#ifdef CACHE_DLL_SYM
     R_osDynSymbol->deleteCachedSymbols = R_deleteCachedSymbols;
     R_osDynSymbol->lookupCachedSymbol = Rf_lookupCachedSymbol;
+#endif
 
     R_osDynSymbol->fixPath = fixPath;
     R_osDynSymbol->getFullDLLPath = GetFullDLLPath;
 }
 
+#ifdef CACHE_DLL_SYM
 static void R_deleteCachedSymbols(DllInfo *dll)
 {
     int i;
@@ -124,6 +130,7 @@ static void R_deleteCachedSymbols(DllInfo *dll)
 	    } else nCPFun--;
 	}
 }
+#endif
 
 #ifndef _MCW_EM
 _CRTIMP unsigned int __cdecl
