@@ -16,6 +16,13 @@
    language is requested. */
 #define ENABLE_NLS 1
 
+/* C type used for Fortran character lengths */
+/* will be size_t for gfortran >= 8, int before */
+/* #undef FC_LEN_T */
+#if defined __GNUC__ && __GNUC__ >= 8
+#define FC_LEN_T size_t
+#endif
+
 /* Define to dummy `main' function (if any) required to link to the Fortran
    libraries. */
 /* #undef FC_DUMMY_MAIN */
@@ -596,6 +603,8 @@
 
 /* Define if your system has pcre2. */
 /* #undef HAVE_PCRE2 */
+#define HAVE_PCRE2 1
+#define PCRE2_CODE_UNIT_WIDTH 8
 
 /* Define to 1 if you have the <pcre.h> header file. */
 /* #undef HAVE_PCRE_H */
@@ -936,6 +945,9 @@
 /* Define to 1 if you have the `wcstombs' function. */
 #define HAVE_WCSTOMBS 1
 
+/* Define to 1 if you have the `wcswidth' function. */
+/* #undef HAVE_WCSWIDTH */
+
 /* Define to 1 if you have the `wctrans' function. */
 #define HAVE_WCTRANS 1
 
@@ -948,6 +960,9 @@
 /* Define to 1 if you have the <wctype.h> header file. */
 #define HAVE_WCTYPE_H 1
 
+/* Define to 1 if you have the `wcwidth' function. */
+/* #undef HAVE_WCWIDTH */
+
 /* Define if you have the 'wint_t' type. (For intl) */
 #define HAVE_WINT_T 1
 
@@ -1181,6 +1196,15 @@
    weak. (For intl) */
 /* #undef USE_PTH_THREADS_WEAK */
 
+/* Define to 1 to use internal code for `towlower' and `towupper'. */
+/* #undef USE_RI18N_CASE */
+
+/* Define to 1 to use internal `iswprint' etc. */
+#define USE_RI18N_FNS 1
+
+/* Define to 1 to use internal `wcwidth' */
+#define USE_RI18N_WIDTH
+
 /* Define if the old Solaris multithreading library can be used. (For intl) */
 /* #undef USE_SOLARIS_THREADS */
 
