@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2018  R Core Team
+ *  Copyright (C) 2000-2020  R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -88,13 +88,19 @@ void rcmdusage (char *RCMD)
 	    "for usage information for each command.\n\n");
 }
 
-#define PROCESS_CMD(ARG)	for (i = cmdarg + 1; i < argc; i++) {\
-	    strcat(cmd, ARG);\
-	    if (strlen(cmd) + strlen(argv[i]) > 9900) {\
-		fprintf(stderr, "command line too long\n");\
-		return(27);\
+#define PROCESS_CMD(ARG)	if (cmdarg + 1 < argc) {\
+	    for (i = cmdarg + 1; i < argc; i++) {\
+		strcat(cmd, ARG);\
+		if (strlen(cmd) + strlen(argv[i]) > 9900) {\
+		    fprintf(stderr, "command line too long\n");\
+		    return(27);\
+		}\
+		strcat(cmd, "\"");\
+		strcat(cmd, argv[i]);\
+		strcat(cmd, "\"");\
 	    }\
-	    strcat(cmd, argv[i]);\
+	    /* the outermost double quotes are needed for cmd.exe */\
+	    strcat(cmd,"\"");\
 	}\
 	return(system(cmd))
 
@@ -167,11 +173,9 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 		fprintf(stderr, "command line too long\n");
 		return(27);
 	    }
-	    if(strchr(argv[i], ' ') || !strlen(argv[i])) {
-		strcat(cmd, "\"");
-		strcat(cmd, argv[i]);
-		strcat(cmd, "\"");
-	    } else strcat(cmd, argv[i]);
+	    strcat(cmd, "\"");
+	    strcat(cmd, argv[i]);
+	    strcat(cmd, "\"");
 	}
 	/* the outermost double quotes are needed for cmd.exe */ 
 	strcat(cmd, "\"");
@@ -386,12 +390,12 @@ int rcmdfn (int cmdarg, int argc, char **argv)
     if (!strcmp(argv[cmdarg], "INSTALL")) {
 	/* Unix has --no-restore except for MM's undocumented --use-vanilla */
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.install_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.install_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "REMOVE")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"\"%s/%s/Rterm.exe\" -f \"%s/share/R/REMOVE.R\" R_DEFAULT_PACKAGES=NULL --no-restore --slave --args",
+		 "\"\"%s/%s/Rterm.exe\" -f \"%s/share/R/REMOVE.R\" R_DEFAULT_PACKAGES=NULL --no-restore --no-echo --args",
 		 getRHOME(3), BINDIR, getRHOME(3));
 	for (i = cmdarg + 1; i < argc; i++){
 	    strcat(cmd, " ");
@@ -399,29 +403,27 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 		fprintf(stderr, "command line too long\n");
 		return(27);
 	    }
-	    /* Library names could contain spaces */
-	    if(strchr(argv[i], ' ')) {
-		strcat(cmd, "\"");
-		strcat(cmd, argv[i]);
-		strcat(cmd, "\"");
-	    } else strcat(cmd, argv[i]);
+	    /* Library names could contain spaces and other special characters */
+	    strcat(cmd, "\"");
+	    strcat(cmd, argv[i]);
+	    strcat(cmd, "\"");
 	}
 	/* the outermost double quotes are needed for cmd.exe */ 
 	strcat(cmd, "\"");
 	return(system(cmd));
     } else if (!strcmp(argv[cmdarg], "build")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.build_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.build_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "check")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.check_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.check_packages() R_DEFAULT_PACKAGES= LC_COLLATE=C --no-restore --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Rprof")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.Rprof() R_DEFAULT_PACKAGES=utils LC_COLLATE=C --vanilla --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.Rprof() R_DEFAULT_PACKAGES=utils LC_COLLATE=C --vanilla --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
    } else if (!strcmp(argv[cmdarg], "texify")) {
@@ -430,41 +432,41 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 	    return(1);
 	}
 	snprintf(cmd, CMD_LEN,
-		 "texify.exe -I \"%s/share/texmf/tex/latex\" -I \"%s/share/texmf/bibtex/bst\"", getRHOME(3), getRHOME(3));
+		 "\"texify.exe -I \"%s/share/texmf/tex/latex\" -I \"%s/share/texmf/bibtex/bst\"", getRHOME(3), getRHOME(3));
 	PROCESS_CMD(" ");
     } else if (!strcmp(argv[cmdarg], "SHLIB")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.SHLIB() R_DEFAULT_PACKAGES=NULL --no-restore --slave --no-site-file --no-init-file --args",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.SHLIB() R_DEFAULT_PACKAGES=NULL --no-restore --no-echo --no-site-file --no-init-file --args",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD(" ");
     } else if (!strcmp(argv[cmdarg], "Rdiff")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.Rdiff() R_DEFAULT_PACKAGES=NULL --vanilla --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.Rdiff() R_DEFAULT_PACKAGES=NULL --vanilla --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Rdconv")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.Rdconv() R_DEFAULT_PACKAGES= LC_COLLATE=C --vanilla --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.Rdconv() R_DEFAULT_PACKAGES= LC_COLLATE=C --vanilla --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Rd2txt")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::.Rdconv() R_DEFAULT_PACKAGES= LC_COLLATE=C --vanilla --slave --args nextArg-tnextArgtxt",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::.Rdconv() R_DEFAULT_PACKAGES= LC_COLLATE=C --vanilla --no-echo --args nextArg-tnextArgtxt",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Rd2pdf")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" -e tools:::..Rd2pdf() R_DEFAULT_PACKAGES= LC_ALL=C --vanilla --slave --args ",
+		 "\"\"%s/%s/Rterm.exe\" -e tools:::..Rd2pdf() R_DEFAULT_PACKAGES= LC_ALL=C --vanilla --no-echo --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Sweave")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" --no-restore --slave -e utils:::.Sweave() --args ",
+		 "\"\"%s/%s/Rterm.exe\" --no-restore --no-echo -e utils:::.Sweave() --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else if (!strcmp(argv[cmdarg], "Stangle")) {
 	snprintf(cmd, CMD_LEN,
-		 "\"%s/%s/Rterm.exe\" --vanilla --slave -e utils:::.Stangle() --args ",
+		 "\"\"%s/%s/Rterm.exe\" --vanilla --no-echo -e utils:::.Stangle() --args ",
 		 getRHOME(3), BINDIR);
 	PROCESS_CMD("nextArg");
     } else {
@@ -488,11 +490,9 @@ int rcmdfn (int cmdarg, int argc, char **argv)
 		fprintf(stderr, "command line too long\n");
 		return(27);
 	    }
-	    if(strchr(argv[i], ' ')) {
-		strcat(cmd, "\"");
-		strcat(cmd, argv[i]);
-		strcat(cmd, "\"");
-	    } else strcat(cmd, argv[i]);
+	    strcat(cmd, "\"");
+	    strcat(cmd, argv[i]);
+	    strcat(cmd, "\"");
 	}
 	/* the outermost double quotes are needed for cmd.exe */ 
 	strcat(cmd, "\"");
