@@ -1,17 +1,12 @@
 #ifndef GETLINE_H
 #define GETLINE_H
 
-/* unix systems can #define POSIX to use termios, otherwise 
- * the bsd or sysv interface will be used 
- */
-
-#if __STDC__ > 0
 #include <stddef.h>
 
 typedef size_t (*gl_strwidth_proc)(char *);
 
 /* read a line of input */
-int             getline(char *prompt, char *buf, int maxlen);  
+int             getline(const char *prompt, char *buf, int maxlen);  
 
 void            gl_setwidth(int);		/* specify width of screen */
 void            gl_histadd(const char *);	/* adds entries to hist */
@@ -30,23 +25,4 @@ extern int	(*gl_tab_hook)(char *, int, int *);
 extern void 	(*gl_events_hook)(void);
 #endif
 
-
-#else	/* not __STDC__ */
-
-char           *getline();	
-void            gl_setwidth();
-void            gl_histadd();
-void		gl_strwidth();
-void		gl_loadhistory();
-void		gl_savehistory();
-void            gl_hist_init();
-char    	*gl_hist_next();
-char    	*gl_hist_prev();
-
-extern int 	(*gl_in_hook)();
-extern int 	(*gl_out_hook)();
-extern int	(*gl_tab_hook)();
-
-#endif /* __STDC__ */
-
 #endif /* GETLINE_H */
