@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  file run.c: a simple 'reading' pipe (and a command executor)
  *  Copyright  (C) 1999-2001  Guido Masarotto and Brian Ripley
- *             (C) 2007-2019  The R Core Team
+ *             (C) 2007-2020  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -148,7 +148,7 @@ static void pcreate(const char* cmd, cetype_t enc,
     DWORD ret;
     STARTUPINFO si;
     STARTUPINFOW wsi;
-    HANDLE dupIN, dupOUT, dupERR, job, port;
+    HANDLE dupIN, dupOUT, dupERR, job, port = NULL;
     WORD showWindow = SW_SHOWDEFAULT;
     DWORD flags;
     BOOL inJob;
@@ -755,6 +755,7 @@ rpipeGetc(rpipe * r)
 	/* we want to look for user break here */
 	while (peekevent()) doevent();
 	if (UserBreak) {
+	    /* FIXME: close handles */
 	    rpipeTerminate(r);
 	    break;
 	}
@@ -797,6 +798,23 @@ int rpipeClose(rpipe *r, int *timedout)
     int   i;
 
     if (!r) return NOLAUNCH;
+    /* Close both pipe ends before forcibly terminating the child process to
+       let it read all data (if it is reading) and exit gracefully.
+
+       r->write and r->read are set to hNULL for the case that threadedwait
+       ends up flushing file buffers
+
+       FIXME: should we be forcing the termination at all? */
+    HANDLE hNULL = CreateFile("NUL:", GENERIC_READ | GENERIC_WRITE, 0,
+                              NULL, OPEN_EXISTING, 0, NULL);
+    HANDLE tmp;
+    tmp = r->read;
+    r->read = hNULL;
+    CloseHandle(tmp);
+    tmp = r->write;
+    r->write = hNULL;
+    CloseHandle(tmp);
+
     rpipeTerminate(r);
     /* threadedwait may have obtained the exit code of the pipe process,
        but also may have been terminated too early; retrieve the exit
@@ -804,9 +822,8 @@ int rpipeClose(rpipe *r, int *timedout)
     DWORD ret;
     GetExitCodeProcess(r->pi.pi.hProcess, &ret);
     r->exitcode = ret;
-    CloseHandle(r->read);
-    CloseHandle(r->write);
     CloseHandle(r->pi.pi.hProcess);
+    CloseHandle(hNULL);
     i = r->exitcode;
     if (timedout)
 	*timedout = r->timedout;
@@ -827,7 +844,7 @@ typedef struct Wpipeconn {
 static Rboolean Wpipe_open(Rconnection con)
 {
     rpipe *rp;
-    int visible = -1, io;
+    int visible = -1, io, mlen;
 
     io = con->mode[0] == 'w';
     if(io) visible = 1; /* Somewhere to put the output */
@@ -840,7 +857,8 @@ static Rboolean Wpipe_open(Rconnection con)
     con->isopen = TRUE;
     con->canwrite = io;
     con->canread = !con->canwrite;
-    if(strlen(con->mode) >= 2 && con->mode[1] == 'b') con->text = FALSE;
+    mlen = (int) strlen(con->mode);
+    if(mlen >= 2 && con->mode[mlen-1] == 'b') con->text = FALSE;
     else con->text = TRUE;
     con->save = -1000;
     return TRUE;
@@ -934,13 +952,12 @@ static int Wpipe_vfprintf(Rconnection con, const char *format, va_list ap)
     char buf[BUFSIZE], *b = buf;
     int res = 0;
 
-    res = vsnprintf(b, BUFSIZE, format, ap);
-    if(res < 0) { /* a failure indication, so try again */
-	b[BUFSIZE -1] = '\0';
-	warning("printing of extremely long output is truncated");
-	res = BUFSIZE;
+    res = Rvsnprintf_mbcs(b, BUFSIZE, format, ap);
+    if(res < 0 || res >= BUFSIZE) {
+	warning(_("printing of extremely long output is truncated"));
+	res = strlen(b);
     }
-    return Wpipe_write(buf, res, 1, con);
+    return Wpipe_write(buf, (size_t)1, (size_t)res, con);
 }
 
 
