@@ -1,12 +1,16 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2007  The R Core Team.
+ *  Copyright (C) 2007--2020  The R Core Team.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This header file is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2.1 of the License, or
  *  (at your option) any later version.
  *
+ *  This file is part of R. R is distributed under the terms of the
+ *  GNU General Public License, either Version 2, June 1991 or Version 3,
+ *  June 2007. See doc/COPYRIGHTS for details of the copyright status of R.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -56,6 +60,11 @@ S_nlminb_iterate(double b[], double d[], double fx, double g[],
 		 double h[], int iv[], int liv, int lv, int n,
 		 double v[], double x[]);
 
+void attribute_hidden
+S_rcont2(int nrow, int ncol, const int nrowt[], const int ncolt[],
+         int ntotal, const double fact[],
+	 int jwork[], int matrix[]);
+
 static R_INLINE int S_v_length(int alg, int n)
 {
     return (alg - 1) ? (105 + (n * (2 * n + 20))) :
@@ -68,4 +77,3 @@ static R_INLINE int S_iv_length(int alg, int n)
 }
 
 #endif /* R_STATS_PACKAGE_H */
-
