@@ -35,7 +35,9 @@
 #define C99_INLINE_SEMANTICS 1
 #endif
 
-/* Apple's gcc build >5400 (since Xcode 3.0) doesn't support GNU inline in C99 mode */
+/* Apple's gcc build >5400 (since Xcode 3.0) doesn't support GNU inline in C99 mode 
+   FIXME: can this possibly still be needed?
+*/
 #if __APPLE_CC__ > 5400 && !defined(C99_INLINE_SEMANTICS) && __STDC_VERSION__ >= 199901L
 #define C99_INLINE_SEMANTICS 1
 #endif
