@@ -31,7 +31,9 @@ extern Rboolean EmitEmbeddedUTF8;
 
 /* originally from Defn.h : */
 
-void R_CleanUp(SA_TYPE, int, int);
+#include <R_ext/Error.h>  // includes NORET macro
+
+void NORET R_CleanUp(SA_TYPE, int, int);
 void R_StartUp(void);
 
 FILE *R_OpenInitFile(void);
