@@ -1,7 +1,7 @@
 #  File src/library/base/R/conditions.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -140,7 +140,7 @@ as.character.condition <- function(x, ...) {
     call <- conditionCall(x)
     cl <- class(x)[1L]
     if (! is.null(call))
-        paste0(cl, " in ", deparse(call)[1L], ": ", msg, "\n")
+        paste0(cl, " in ", deparse(call, nlines=1L), ": ", msg, "\n")
     else
         paste0(cl, ": ", msg, "\n")
 }
@@ -149,7 +149,7 @@ as.character.error <- function(x, ...) {
     msg <- conditionMessage(x)
     call <- conditionCall(x)
     if (! is.null(call))
-        paste0("Error in ", deparse(call)[1L], ": ", msg, "\n")
+        paste0("Error in ", deparse(call, nlines=1L), ": ", msg, "\n")
     else
         paste0("Error: ", msg, "\n")
 }
@@ -404,22 +404,22 @@ globalCallingHandlers <-
                 ## first duplicate on the stack, so that registering a
                 ## handler again has the effect of pushing it on top of the
                 ## stack.
-                classes <- names(gh)
-                for (class in unique(classes)) {
-                    idx <- which(class == classes)
+                for (class in unique(names(gh))) {
+                    idx <- which(class == names(gh))
 
                     ## Ideally we'd just use `duplicated()` on the list
                     ## of handlers. Since that doesn't take into
                     ## account the closure environments, we first
-                    ## convert the functions to lists.
+                    ## convert the functions to lists and also remove
+                    ## source references.
                     funAsList <- function(x) {
+                        x <- utils::removeSource(x)
                         out <- list(formals(x), body(x), environment(x))
                         attributes(out) <- attributes(x)
                         out
-                    }
+		    }
                     classHandlers <- lapply(gh[idx], funAsList)
                     dups <- duplicated(classHandlers)
-
                     if (any(dups)) {
                         message(sprintf("pushing duplicate `%s` handler on top of the stack", class))
                         gh <<- gh[-idx[dups]]
