@@ -1,7 +1,7 @@
 #  File src/library/base/R/connections.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -125,9 +125,22 @@ xzfile <- function(description, open = "", encoding = getOption("encoding"),
 socketConnection <- function(host = "localhost", port, server = FALSE,
                              blocking = FALSE, open = "a+",
                              encoding = getOption("encoding"),
-                             timeout = getOption("timeout"))
+                             timeout = getOption("timeout"),
+                             options = getOption("socketOptions"))
     .Internal(socketConnection(host, port, server, blocking, open, encoding,
-                               timeout))
+                               timeout, options))
+
+socketAccept <- function(socket, blocking = FALSE, open = "a+",
+                         encoding = getOption("encoding"),
+                         timeout = getOption("timeout"),
+                         options = getOption("socketOptions"))
+    .Internal(socketAccept(socket, blocking, open, encoding, timeout, options))
+
+serverSocket <- function(port)
+    .Internal(serverSocket(port))
+
+socketTimeout <- function(socket, timeout = -1)
+    .Internal(socketTimeout(socket, timeout))
 
 rawConnection <- function(object, open = "r") {
     .Internal(rawConnection(deparse(substitute(object)), object, open))
@@ -136,14 +149,16 @@ rawConnection <- function(object, open = "r") {
 rawConnectionValue <- function(con) .Internal(rawConnectionValue(con))
 
 textConnection <- function(object, open = "r", local = FALSE,
+                           name = deparse(substitute(object)),
                            encoding = c("", "bytes", "UTF-8"))
 {
     env <- if (local) parent.frame() else .GlobalEnv
     type <- match(match.arg(encoding), c("", "bytes", "UTF-8"))
-    nm <- deparse(substitute(object))
-    if(length(nm) != 1)
-        stop("argument 'object' must deparse to a single character string")
-    .Internal(textConnection(nm, object, open, env, type))
+    if(!(is.character(name) && length(name) == 1))
+        stop(if(missing(name))
+                 "argument 'object' must deparse to a single character string"
+             else "'name' must be a single character string")
+    .Internal(textConnection(name, object, open, env, type))
 }
 
 textConnectionValue <- function(con) .Internal(textConnectionValue(con))
@@ -244,6 +259,8 @@ closeAllConnections <- function()
 readBin <- function(con, what, n = 1L, size = NA_integer_, signed = TRUE,
                     endian = .Platform$endian)
 {
+    if (!endian %in% c("big", "little", "swap"))
+        stop("invalid 'endian' argument")
     if(is.character(con)) {
         con <- file(con, "rb")
         on.exit(close(con))
@@ -261,6 +278,8 @@ writeBin <-
     function(object, con, size = NA_integer_, endian = .Platform$endian,
              useBytes = FALSE)
 {
+    if (!endian %in% c("big", "little", "swap"))
+        stop("invalid 'endian' argument")
     swap <- endian != .Platform$endian
     if(!is.vector(object) || mode(object) == "list")
         stop("can only write vector objects")
