@@ -41,7 +41,11 @@ Sys.timezone <- function(location = TRUE)
                          inherits = FALSE, ifnotfound = NA_character_)))
         return(tz)
 
-    cacheIt <- function(tz) assign(".sys.timezone", tz, baseenv())
+    cacheIt <- function(tz) {
+        unlockBinding(".sys.timezone", baseenv())
+        assign(".sys.timezone", tz, baseenv())
+        lockBinding(".sys.timezone", baseenv())
+    }
 
     ## Many Unix set TZ, e.g. Solaris and AIX.
     ## For Solaris the system setting is a line in /etc/TIMEZONE
@@ -357,7 +361,7 @@ as.POSIXct.default <- function(x, tz = "", ...)
     if(is.character(x) || is.factor(x))
 	return(as.POSIXct(as.POSIXlt(x, tz, ...), tz, ...))
     if(is.logical(x) && all(is.na(x)))
-        return(.POSIXct(as.numeric(x)))
+        return(.POSIXct(as.numeric(x), tz))
     stop(gettextf("do not know how to convert '%s' to class %s",
                   deparse1(substitute(x)),
                   dQuote("POSIXct")),
@@ -595,14 +599,20 @@ anyNA.POSIXlt <- function(x, recursive = FALSE)
     anyNA(as.POSIXct(x))
 
 ## <FIXME> check the argument validity
-## This is documented to remove the timezone
-c.POSIXct <- function(..., recursive = FALSE)
-    .POSIXct(c(unlist(lapply(list(...),
-                             function(e) unclass(as.POSIXct(e))))))
+## This is documented to remove the timezone (unless all are marked with
+## the same).
+c.POSIXct <- function(..., recursive = FALSE) {
+    x <- lapply(list(...), function(e) unclass(as.POSIXct(e)))
+    tzones <- lapply(x, attr, "tzone")
+    tz <- if(length(unique(tzones)) == 1L) tzones[[1L]] else NULL
+    .POSIXct(c(unlist(x)), tz)
+}
 
 ## we need conversion to POSIXct as POSIXlt objects can be in different tz.
-c.POSIXlt <- function(..., recursive = FALSE)
-    as.POSIXlt(do.call("c", lapply(list(...), as.POSIXct)))
+c.POSIXlt <- function(..., recursive = FALSE) {
+    as.POSIXlt(do.call("c",
+                       lapply(list(...), as.POSIXct)))
+}
 
 
 ISOdatetime <- function(year, month, day, hour, min, sec, tz = "")
@@ -1011,22 +1021,30 @@ cut.POSIXt <-
         if(valid == 8L) incr <- 25*3600 # DSTdays
         if(valid == 6L) {               # months
             start$mday <- 1L
-            end <- as.POSIXlt(max(x, na.rm = TRUE))
+            maxx <- max(x, na.rm = TRUE)
+            end <- as.POSIXlt(maxx)
             step <- if(length(by2) == 2L) as.integer(by2[1L]) else 1L
             end <- as.POSIXlt(end + (31 * step * 86400))
             end$mday <- 1L
             end$isdst <- -1L
             breaks <- seq(start, end, breaks)
+            ## 31 days ahead could give an empty level, so
+	    lb <- length(breaks)
+	    if(maxx < breaks[lb-1]) breaks <- breaks[-lb]
         } else if(valid == 7L) {        # years
             start$mon <- 0L
             start$mday <- 1L
-            end <- as.POSIXlt(max(x, na.rm = TRUE))
+            maxx <- max(x, na.rm = TRUE)
+            end <- as.POSIXlt(maxx)
             step <- if(length(by2) == 2L) as.integer(by2[1L]) else 1L
             end <- as.POSIXlt(end + (366 * step* 86400))
             end$mon <- 0L
             end$mday <- 1L
             end$isdst <- -1L
             breaks <- seq(start, end, breaks)
+            ## 366 days ahead could give an empty level, so
+	    lb <- length(breaks)
+	    if(maxx < breaks[lb-1]) breaks <- breaks[-lb]
         } else if(valid == 9L) {        # quarters
             qtr <- rep(c(0L, 3L, 6L, 9L), each = 3L)
             start$mon <- qtr[start$mon + 1L]
@@ -1327,7 +1345,7 @@ is.numeric.difftime <- function(x) FALSE
 ## Class generators added in 2.11.0, class order changed in 2.12.0.
 
 ## FIXME:
-## At least temporarily avoide structure() for performance reasons.
+## At least temporarily avoid structure() for performance reasons.
 ## .POSIXct <- function(xx, tz = NULL)
 ##     structure(xx, class = c("POSIXct", "POSIXt"), tzone = tz)
 .POSIXct <- function(xx, tz = NULL, cl = c("POSIXct", "POSIXt")) {
@@ -1469,5 +1487,13 @@ as.list.POSIXlt <- function(x, ...)
 as.list.difftime <- function(x, ...)
     lapply(unclass(x), .difftime, attr(x, "units"), oldClass(x))
 
+## Added in 4.1.0.
 
+rep.difftime <- function(x, ...)
+    .difftime(NextMethod("rep"), attr(x, "units"), oldClass(x))
 
+`[<-.difftime` <- function(x, i, value) {
+    if(inherits(value, "difftime") && !identical(units(x), units(value)))
+        units(value) <- units(x)
+    NextMethod("[<-")
+}
