@@ -1,7 +1,7 @@
 #  File src/library/base/R/library.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 ## If we want this it would be better to factor out the core of checkConflicts.
 ## searchConflicts <- function(pkg) {
 ##     vars <- getNamespaceExports(pkg)
-##     conflicts <- function(pos) intersect(vars, ls(pos, all = TRUE))
+##     conflicts <- function(pos) intersect(vars, ls(pos, all.names = TRUE))
 ##     val <- Filter(length, sapply(search()[-1], conflicts))
 ##     if (length(val)) val else NULL
 ## }
@@ -307,7 +307,7 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                 else
                     gettext("no library trees found in 'lib.loc'")
                 if(logical.return) {
-                    warning(txt, domain = NA)
+                    if(!quietly) warning(txt, domain = NA)
 		    return(FALSE)
 		} else stop(txt, domain = NA)
             }
@@ -387,7 +387,7 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
 			     paste(" in", deparse(cc)[1L]) else ""
 		    msg <- gettextf("package or namespace load failed for %s%s:\n %s",
 				    sQuote(package), P, conditionMessage(e))
-		    if(logical.return)
+		    if(logical.return && !quietly)
 			message(paste("Error:", msg), domain = NA) # returns NULL
 		    else stop(msg, call. = FALSE, domain = NA)
 		})
@@ -626,8 +626,9 @@ function(chname, package, lib.loc, verbose = getOption("verbose"),
     }
     if(verbose)
         message(gettextf("now dyn.load(\"%s\") ...", file), domain = NA)
-    dll <- if("DLLpath" %in% names(list(...))) dyn.load(file, ...)
-    else dyn.load(file, DLLpath = DLLpath, ...)
+    dll <- if("DLLpath" %in% ...names())
+                dyn.load(file, ...)
+           else dyn.load(file, DLLpath = DLLpath, ...)
     .dynLibs(c(dll_list, list(dll)))
     invisible(dll)
 }
@@ -805,17 +806,22 @@ function(package = NULL, lib.loc = NULL, quiet = FALSE,
                 ## Note that this is sometimes used for source
                 ## packages, e.g. by promptPackage from package.skeleton
                 pfile <- file.path(p, "Meta", "package.rds")
-                info <- if(file.exists(pfile))
+                info <- if(file.exists(pfile)) {
                     ## this must have these fields to get installed
-                    readRDS(pfile)$DESCRIPTION[c("Package", "Version")]
-                else {
+                    tryCatch(readRDS(pfile)$DESCRIPTION[c("Package",
+                                                          "Version")],
+                             error = function(e)
+                                 c(Package = NA_character_,
+                                   Version = NA_character_))
+                } else {
                     info <- tryCatch(read.dcf(file.path(p, "DESCRIPTION"),
                                               c("Package", "Version"))[1, ],
                                      error = identity)
                     if(inherits(info, "error")
                        || (length(info) != 2L)
                        || anyNA(info))
-                        c(Package = NA, Version = NA) # need dimnames below
+                        c(Package = NA_character_,
+                          Version = NA_character_) # need dimnames below
                     else
                         info
                 }
