@@ -1,7 +1,7 @@
 #  File src/library/base/R/mapply.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 mapply <- function(FUN,..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE)
 {
@@ -29,13 +29,13 @@ mapply <- function(FUN,..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE)
 	else if (!is.null(names1))
 	    names(answer) <- names1
     }
-    if(!identical(SIMPLIFY, FALSE) && length(answer))
+    if(!isFALSE(SIMPLIFY))
 	simplify2array(answer, higher = (SIMPLIFY == "array"))
     else answer
 }
 
 .mapply <- function(FUN, dots, MoreArgs)
-    .Internal(mapply(FUN, dots, MoreArgs))
+    .Internal(mapply(match.fun(FUN), dots, MoreArgs))
 
 Vectorize <- function(FUN, vectorize.args = arg.names, SIMPLIFY = TRUE,
                       USE.NAMES = TRUE)
@@ -51,6 +51,13 @@ Vectorize <- function(FUN, vectorize.args = arg.names, SIMPLIFY = TRUE,
     if (!all(vectorize.args %in% arg.names))
     	stop("must specify names of formal arguments for 'vectorize'")
 
+    collisions <- arg.names %in% c("FUN", "SIMPLIFY", "USE.NAMES",
+                                   "vectorize.args")
+    if (any(collisions))
+	stop(sQuote("FUN"), " may not have argument(s) named ",
+	     paste(sQuote(arg.names[collisions]), collapse = ", "))
+    rm(arg.names, collisions)
+    (function() {
     FUNV <- function() { ## will set the formals below
         args <- lapply(as.list(match.call())[-1L], eval, parent.frame())
         names <- if(is.null(names(args))) character(length(args))
@@ -63,5 +70,7 @@ Vectorize <- function(FUN, vectorize.args = arg.names, SIMPLIFY = TRUE,
                             USE.NAMES = USE.NAMES))
     }
     formals(FUNV) <- formals(FUN)
+    environment(FUNV) <- parent.env(environment())
     FUNV
+    })()
 }
