@@ -1,7 +1,7 @@
 #  File src/library/base/R/pretty.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -28,10 +28,11 @@ pretty.default <-
     if(!length(x)) return(x)
     z <- .Internal(pretty(min(x), max(x), n, min.n, shrink.sml,
                           c(high.u.bias, u5.bias), eps.correct))
-    s <- seq.int(z$l, z$u, length.out = z$n + 1L)
-    if(!eps.correct && z$n) { # maybe zap smalls from seq() rounding errors
+    n <- z$n
+    s <- seq.int(z$l, z$u, length.out = n + 1L)
+    if(!eps.correct && n) { # maybe zap smalls from seq() rounding errors
         ## better than zapsmall(s, digits = 14) :
-        delta <- diff(range(z$l, z$u)) / z$n  # or abs(z$u - z$l)
+        delta <- diff(range(z$l, z$u)/n) # NB: |l - u| may be Inf
         if(any(small <- abs(s) < 1e-14 * delta)) s[small] <- 0
     }
     s
