@@ -1,7 +1,7 @@
 #  File src/library/base/R/range.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,9 @@ range.default <- function(..., na.rm = FALSE, finite = FALSE)
     if(is.numeric(x)) {
         if(finite) x <- x[is.finite(x)]
         else if(na.rm) x <- x[!is.na(x)]
-	return (c(min(x), max(x)))
+	c(min(x), max(x))
+    } else {
+        if(finite) na.rm <- TRUE
+        c(min(x, na.rm=na.rm), max(x, na.rm=na.rm))
     }
-    c(min(x, na.rm=na.rm), max(x, na.rm=na.rm))
 }
