@@ -263,9 +263,16 @@ Have you called 'sort' on a list?")
 
 ## xtfrm is now primitive
 ## xtfrm <- function(x) UseMethod("xtfrm")
-xtfrm.default <- function(x)
-    if(is.numeric(x)) unclass(x) else as.vector(rank(x, ties.method = "min",
-                                                     na.last = "keep"))
+xtfrm.default <- function(x) {
+    y <- if(is.numeric(x))
+             unclass(x)
+         else as.vector(rank(x, ties.method = "min", na.last = "keep"))
+    if(!is.numeric(y) ||
+       ((length(y) != length(x)) && !inherits(x, "data.frame")))
+        stop("cannot xtfrm 'x'")
+    y
+}
+
 xtfrm.factor <- function(x) as.integer(x) # primitive, so needs a wrapper
 
 ## ## Moved to package survival
