@@ -19,7 +19,7 @@
 svd <- function(x, nu = min(n,p), nv = min(n,p), LINPACK = FALSE)
 {
     if (!missing(LINPACK))
-        warning("the LINPACK argument has been defunct since R 3.1.0")
+        stop("the LINPACK argument has been defunct since R 3.1.0")
     x <- as.matrix(x)
     if (any(!is.finite(x))) stop("infinite or missing values in 'x'")
     dx <- dim(x)
