@@ -1,7 +1,7 @@
 #  File src/library/base/R/zzz.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -28,12 +28,13 @@ is.name <- is.symbol
     baseGenerics <- c("Math", "Ops", "Summary", "Complex",
         "as.character", "as.data.frame", "as.environment", "as.matrix", "as.vector",
         "cbind", "labels", "print", "rbind", "rep", "seq", "seq.int",
-        "solve", "summary", "t")
+        "plot", "sequence", "solve", "summary", "t")
 
     utilsGenerics <- c("edit", "str")
 
     graphicsGenerics <- c("contour", "hist", "identify", "image",
-        "lines", "pairs", "plot", "points", "text")
+        ## "plot",
+        "lines", "pairs", "points", "text")
 
     statsGenerics <- c("add1", "AIC", "anova", "biplot", "coef",
         "confint", "deviance", "df.residual", "drop1", "extractAIC",
@@ -55,8 +56,11 @@ is.name <- is.symbol
 
 .ArgsEnv <- new.env(hash = TRUE, parent = emptyenv())
 
+assign("::", function(pkg, name) NULL, envir = .ArgsEnv)
+assign(":::", function(pkg, name) NULL, envir = .ArgsEnv)
 assign("%*%", function(x, y) NULL, envir = .ArgsEnv)
 assign("...length", function() NULL, envir = .ArgsEnv)
+assign("...names",  function() NULL, envir = .ArgsEnv)
 assign("...elt", function(n) NULL, envir = .ArgsEnv)
 assign(".C", function(.NAME, ..., NAOK = FALSE, DUP = TRUE, PACKAGE,
                       ENCODING) NULL,
@@ -72,6 +76,7 @@ assign(".External.graphics", function(.NAME, ..., PACKAGE) NULL,
        envir = .ArgsEnv)
 assign(".Internal", function(call) NULL, envir = .ArgsEnv)
 assign(".Primitive", function(name) NULL, envir = .ArgsEnv)
+assign(".class2", function(x) NULL, envir = .ArgsEnv)
 assign(".isMethodsDispatchOn", function(onOff = NULL) NULL, envir = .ArgsEnv)
 assign(".primTrace", function(obj) NULL, envir = .ArgsEnv)
 assign(".primUntrace", function(obj) NULL, envir = .ArgsEnv)
@@ -251,7 +256,7 @@ assign("trunc", function(x, ...) UseMethod("trunc"), envir = .GenericArgsEnv)
 assign("as.numeric", get("as.double", envir = .GenericArgsEnv),
        envir = .GenericArgsEnv)
 
-## Keep this in sync with
+## Keep this in sync with ../../tools/R/utils.R
 ##   tools:::.make_S3_methods_table_for_base()
 ## for computing the methods table and
 ##   tools:::.deparse_S3_methods_table_for_base()
@@ -292,6 +297,7 @@ matrix(c("!", "hexmode",
          "[<-", "POSIXct",
          "[<-", "POSIXlt",
          "[<-", "data.frame",
+         "[<-", "difftime",
          "[<-", "factor",
          "[<-", "numeric_version",
          "[[", "Date",
@@ -333,6 +339,7 @@ matrix(c("!", "hexmode",
          "all.equal", "environment",
          "all.equal", "factor",
          "all.equal", "formula",
+         "all.equal", "function",
          "all.equal", "language",
          "all.equal", "list",
          "all.equal", "numeric",
@@ -342,6 +349,7 @@ matrix(c("!", "hexmode",
          "anyDuplicated", "default",
          "anyDuplicated", "matrix",
          "anyNA", "POSIXlt",
+         "anyNA", "data.frame",
          "anyNA", "numeric_version",
          "aperm", "default",
          "aperm", "table",
@@ -405,6 +413,7 @@ matrix(c("!", "hexmode",
          "as.list", "POSIXlt",
          "as.list", "data.frame",
          "as.list", "default",
+         "as.list", "difftime",
          "as.list", "environment",
          "as.list", "factor",
          "as.list", "function",
@@ -424,6 +433,7 @@ matrix(c("!", "hexmode",
          "c", "POSIXct",
          "c", "POSIXlt",
          "c", "difftime",
+         "c", "factor",
          "c", "noquote",
          "c", "numeric_version",
          "c", "warnings",
@@ -555,6 +565,7 @@ matrix(c("!", "hexmode",
          "rep", "Date",
          "rep", "POSIXct",
          "rep", "POSIXlt",
+         "rep", "difftime",
          "rep", "factor",
          "rep", "numeric_version",
          "rev", "default",
@@ -571,6 +582,7 @@ matrix(c("!", "hexmode",
          "seq", "Date",
          "seq", "POSIXt",
          "seq", "default",
+         "sequence", "default",
          "solve", "default",
          "solve", "qr",
          "sort", "POSIXlt",
@@ -623,6 +635,7 @@ matrix(c("!", "hexmode",
          "xtfrm", "Date",
          "xtfrm", "POSIXct",
          "xtfrm", "POSIXlt",
+         "xtfrm", "data.frame",
          "xtfrm", "default",
          "xtfrm", "difftime",
          "xtfrm", "factor",
