@@ -71,7 +71,7 @@ function (x = seq(0, 1, length.out = nrow(z)),
 	stop("no proper 'z' matrix specified")
     if (1.0 * length(x) * length(y) != length(z))
         stop("dimensions of 'x', 'y' and 'z' do not match")
-    invisible(.External2(C_contourLines, x, y, z, levels))
+    .External2(C_contourLines, x, y, z, levels)
 }
 
 chull <- function(x, y = NULL)
