@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/device.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,34 +17,36 @@
 #  https://www.R-project.org/Licenses/
 
 
-.known_interactive.devices <-
-    c("X11", "X11cairo", "quartz", "windows", "JavaGD", "CairoWin", "CairoX11")
-
 dev.interactive <- function(orNone = FALSE)
 {
     if(!interactive()) return(FALSE)
-    if(.Device %in% .known_interactive.devices) return(TRUE)
+    .known_interactive_devices <- deviceIsInteractive()
+    if(.Device %in% .known_interactive_devices) return(TRUE)
     if(!(orNone && .Device == "null device")) return(FALSE)
     ## at this point we have mo active device.
     newdev <- getOption("device")
-    if(is.character(newdev)) newdev %in% .known_interactive.devices
+    if(is.character(newdev)) newdev %in% .known_interactive_devices
     else { # a function
         if(.Platform$OS.type == "windows") identical(newdev, windows)
         else identical(newdev, X11) || identical(newdev, quartz)
     }
 }
 
-deviceIsInteractive <- function(name = NULL)
-{
-    if(length(name)) {
-        if(!is.character(name)) stop("'name' must be a character vector")
-        unlockBinding(".known_interactive.devices", asNamespace("grDevices"))
-        .known_interactive.devices <<- c(.known_interactive.devices, name)
-        lockBinding(".known_interactive.devices", asNamespace("grDevices"))
-        invisible(.known_interactive.devices)
-    } else .known_interactive.devices
-}
-
+deviceIsInteractive <- local({
+    .known_interactive_devices <- 
+        c("X11", "X11cairo", "quartz", "windows", "JavaGD", "CairoWin",
+          "CairoX11")
+    function(name = NULL) {
+        if(length(name)) {
+            if(!is.character(name))
+                stop("'name' must be a character vector")
+            .known_interactive_devices <<-
+                unique(c(.known_interactive_devices, name))
+            invisible(.known_interactive_devices)
+        } else
+            .known_interactive_devices
+    }
+})
 
 dev.list <- function()
 {
@@ -104,7 +106,7 @@ dev.off <-
 
 dev.copy <- function(device, ..., which = dev.next())
 {
-    if(!missing(which) & !missing(device))
+    if(!missing(which) && !missing(device))
 	stop("cannot supply 'which' and 'device' at the same time")
     old.device <- dev.cur()
     if(old.device == 1)
@@ -327,15 +329,16 @@ dev.new <- function(..., noRStudioGD = FALSE)
     do.call(dev, a)
 }
 
-### Check for a single valid integer format
+## Check string to contain at most one valid integer format (starting with '%')
+## 's' is typically the file(name) or title argument of a graphical device.
 checkIntFormat <- function(s)
 {
     ## OK if no unescaped %, so first remove those
-    s <- gsub("%%", "", s)
+    s <- gsub("%%", "", s, fixed=TRUE)
     if(length(grep("%", s)) == 0L) return(TRUE)
     ## now remove at most one valid(ish) integer format
     s <- sub("%[#0 ,+-]*[0-9.]*[diouxX]", "", s)
-    length(grep("%", s)) == 0L
+    length(grep("%", s, fixed=TRUE)) == 0L
 }
 
 devAskNewPage <- function(ask=NULL) .External2(C_devAskNewPage, ask)
