@@ -42,8 +42,6 @@
 #define HAVE_CAIRO_SVG 1
 #define HAVE_CAIRO_PDF 1
 #define HAVE_CAIRO_PS 1
-/* and if not using pango, this selects fontconfig */
-//#define USE_FC 1
 
 # define raise our_raise
 # include <Defn.h>
@@ -194,6 +192,11 @@ BM_Open(pDevDesc dd, pX11Desc xd, int width, int height)
     else
 	error(_("unimplemented cairo-based device"));
 
+    CairoInitPatterns(xd);
+    CairoInitClipPaths(xd);
+    CairoInitMasks(xd);
+    xd->appending = 0;
+
     return TRUE;
 }
 
@@ -381,6 +384,9 @@ static void BM_Close(pDevDesc dd)
 	    xd->type == TIFF || xd->type == BMP || xd->type == PNGdirect)
 	    BM_Close_bitmap(xd);
     if (xd->fp) fclose(xd->fp);
+    CairoDestroyMasks(xd);
+    CairoDestroyClipPaths(xd);
+    CairoDestroyPatterns(xd);
     if (xd->cc) cairo_show_page(xd->cc);
     if (xd->cs) cairo_surface_destroy(xd->cs);
     if (xd->cc) cairo_destroy(xd->cc);
@@ -507,6 +513,13 @@ BMDeviceDriver(pDevDesc dd, int kind, SEXP filename,
     dd->newPage = BM_NewPage;
     dd->close = BM_Close;
 
+    dd->setPattern = Cairo_SetPattern;
+    dd->releasePattern = Cairo_ReleasePattern;
+    dd->setClipPath = Cairo_SetClipPath;
+    dd->releaseClipPath = Cairo_ReleaseClipPath;
+    dd->setMask = Cairo_SetMask;
+    dd->releaseMask = Cairo_ReleaseMask;
+
     dd->left = 0;
     dd->right = width;
     dd->top = 0;
@@ -530,6 +543,8 @@ BMDeviceDriver(pDevDesc dd, int kind, SEXP filename,
     dd->startfont = 1;
     dd->startgamma = 1;
     dd->displayListOn = FALSE;
+    dd->deviceVersion = R_GE_definitions;
+
     dd->deviceSpecific = (void *) xd;
 
     return TRUE;
