@@ -371,6 +371,12 @@ static void     RQuartz_Path(double*, double*, int, int*, Rboolean, const pGEcon
 static Rboolean RQuartz_Locator(double*, double*, pDevDesc);
 static void     RQuartz_Mode(int mode, pDevDesc);
 static void     RQuartz_MetricInfo(int, const pGEcontext , double*, double*, double*, pDevDesc);
+static SEXP     RQuartz_setPattern(SEXP pattern, pDevDesc dd);
+static void     RQuartz_releasePattern(SEXP ref, pDevDesc dd);
+static SEXP     RQuartz_setClipPath(SEXP path, SEXP ref, pDevDesc dd);
+static void     RQuartz_releaseClipPath(SEXP ref, pDevDesc dd);
+static SEXP     RQuartz_setMask(SEXP path, SEXP ref, pDevDesc dd);
+static void     RQuartz_releaseMask(SEXP ref, pDevDesc dd);
 
 #pragma mark Quartz device implementation
 
@@ -430,6 +436,14 @@ void* QuartzDevice_Create(void *_dev, QuartzBackend_t *def)
     dev->haveCapture = (def->cap) ? 2 : 1;
     dev->haveLocator = (def->locatePoint) ? 2 : 1;
 
+    dev->setPattern      = RQuartz_setPattern;
+    dev->releasePattern  = RQuartz_releasePattern;
+    dev->setClipPath     = RQuartz_setClipPath;
+    dev->releaseClipPath = RQuartz_releaseClipPath;
+    dev->setMask         = RQuartz_setMask;
+    dev->releaseMask     = RQuartz_releaseMask;
+    dev->deviceVersion = R_GE_definitions;
+
     QuartzDesc *qd = calloc(1, sizeof(QuartzDesc));
     qd->width      = def->width;
     qd->height     = def->height;
@@ -582,7 +596,7 @@ static void RQuartz_CacheAddFont(const char *family, int face, ATSFontRef font)
     {
         int i = font_cache_tail->fonts;
         font_cache_tail->e[i].font = font;
-        font_cache_tail->e[i].family = strdup(family);
+        font_cache_tail->e[i].family = Rstrdup(family);
         font_cache_tail->e[i].face = face;
         font_cache_tail->fonts++;
     }
@@ -1282,6 +1296,24 @@ static Rboolean RQuartz_Locator(double *x, double *y, DEVDESC)
     return res;
 }
 
+static SEXP RQuartz_setPattern(SEXP pattern, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void RQuartz_releasePattern(SEXP ref, pDevDesc dd) {} 
+
+static SEXP RQuartz_setClipPath(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void RQuartz_releaseClipPath(SEXP ref, pDevDesc dd) {}
+
+static SEXP RQuartz_setMask(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void RQuartz_releaseMask(SEXP ref, pDevDesc dd) {}
+
 #pragma mark -
 #pragma mark R Interface
 
