@@ -111,4 +111,10 @@ unsigned int inR_GE_str2col(const char *s);
 void initPalette(void);
 
 SEXP cairoVersion(void);
+SEXP pangoVersion(void);
+SEXP cairoFT(void);
 SEXP bmVersion(void);
+
+SEXP setPattern(SEXP args);
+SEXP setClipPath(SEXP args);
+SEXP setMask(SEXP args);
