@@ -1,7 +1,7 @@
 #  File src/library/graphics/R/barplot.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 barplot <- function(height, ...) UseMethod("barplot")
 
@@ -28,6 +28,7 @@ function(height, width = 1, space = NULL, names.arg = NULL,
 	 axes = TRUE, axisnames = TRUE,
 	 cex.axis = par("cex.axis"), cex.names = par("cex.axis"),
          inside = TRUE, plot = TRUE, axis.lty = 0, offset = 0, add = FALSE,
+	 ann = !add && par("ann"),
          args.legend = NULL, ...)
  {
     if (!missing(inside)) .NotYetUsed("inside", error = FALSE)# -> help(.)
@@ -40,9 +41,10 @@ function(height, width = 1, space = NULL, names.arg = NULL,
 	names.arg <-
 	    if(is.matrix(height)) colnames(height) else names(height)
 
-    if (is.vector(height)
-	|| (is.array(height) && (length(dim(height)) == 1))) {
+    vectorInput <- (is.vector(height)
+	|| (is.array(height) && (length(dim(height)) == 1)))
 	## Treat vectors and 1-d arrays the same.
+    if (vectorInput) {
 	height <- cbind(height)
 	beside <- TRUE
 	## The above may look strange, but in particular makes color
@@ -74,14 +76,14 @@ function(height, width = 1, space = NULL, names.arg = NULL,
     NC <- ncol(height)
 
     if (beside) {
-	if (length(space) == 2)
+	if (length(space) == 2 && !vectorInput)
 	    space <- rep.int(c(space[2L], rep.int(space[1L], NR - 1)), NC)
-	width <- rep(width, length.out = NR)
+	width <- rep_len(width, NR)
     } else {
-	width <- rep(width, length.out = NC)
+	width <- rep_len(width, NC)
     }
 
-    offset <- rep(as.vector(offset), length.out = length(width))
+    offset <- rep_len(as.vector(offset), length(width))
 
     delta <- width / 2
     w.r <- cumsum(space + width)
@@ -172,7 +174,7 @@ function(height, width = 1, space = NULL, names.arg = NULL,
 		 lty = axis.lty, cex.axis = cex.names, ...)
 	}
 	if(!is.null(legend.text)) {
-	    legend.col <- rep(col, length.out = length(legend.text))
+	    legend.col <- rep_len(col, length(legend.text))
 	    if((horiz & beside) || (!horiz & !beside)){
 		legend.text <- rev(legend.text)
 		legend.col <- rev(legend.col)
@@ -194,8 +196,44 @@ function(height, width = 1, space = NULL, names.arg = NULL,
                 do.call("legend", args.legend1)
             }
 	}
-	title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)
+	if(ann) title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)
 	if(axes) axis(if(horiz) 1 else 2, cex.axis = cex.axis, ...)
 	invisible(w.m)
     } else w.m
 }
+
+barplot.formula <- function(formula, data, subset, na.action,
+                            horiz = FALSE, xlab = NULL, ylab = NULL, ...)
+{
+    if (missing(formula) || length(formula) != 3L)
+        stop("'formula' missing or incorrect")
+    m <- match.call(expand.dots = FALSE)
+    if (is.matrix(eval(m$data, parent.frame())))
+        m$data <- as.data.frame(data)
+    m$... <- m$horiz <- m$xlab <- m$ylab <- NULL
+    m[[1L]] <- quote(stats::model.frame)
+
+    mf <- eval(m, parent.frame())
+    if (ncol(mf[-1L]) == 0L || ncol(mf[-1L]) >= 3L)
+        stop("formula must specify 1 or 2 categorical variables")
+    if (anyDuplicated(mf[-1L]))
+        stop("duplicated categorical values - try another formula or subset")
+    if (horiz) {
+        if(is.null(ylab)) ylab <- names(mf)[ncol(mf)]
+    } else
+        if(is.null(xlab)) xlab <- names(mf)[ncol(mf)]
+    if (is.matrix(mf[[1L]])) { ## LHS is cbind()
+        if (ncol(mf[-1L]) != 1L)
+            stop("formula with cbind() must specify 1 categorical variable")
+        lhs <- t(mf[[1L]])
+        colnames(lhs) <- mf[[ncol(mf)]]
+        barplot.default(lhs, horiz = horiz, xlab = xlab, ylab = ylab, ...)
+    } else {
+	if (horiz) {
+	    if(is.null(xlab)) xlab <- names(mf)[1L]
+	} else
+	    if(is.null(ylab)) ylab <- names(mf)[1L]
+	barplot.default(stats::xtabs(mf, addNA = TRUE),
+			horiz = horiz, xlab = xlab, ylab = ylab, ...)
+  }
+}
