@@ -1,7 +1,7 @@
 #  File src/library/graphics/R/boxplot.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -43,8 +43,17 @@ function(x, ..., range = 1.5, width = NULL, varwidth = FALSE,
 	    attr(groups, "names") <- 1L:n
 	names <- attr(groups, "names")
     }
-    cls <- vapply(groups, function(x) class(x)[1L], "")
-    cl <- if(all(cls == cls[1L])) cls[1L] # else NULL
+    cls <- lapply(groups, class)    
+    cl <- NULL
+    ## Check for simple numeric structures with only a common class attribute
+    if(all(vapply(groups,
+                  function(e) {
+                      is.numeric(unclass(e)) &&
+                          identical(names(attributes(e)), "class")
+                  },
+                  NA)) &&
+       (length(unique(cls)) == 1L))
+        cl <- cls[[1L]]
     for(i in 1L:n)
 	groups[i] <- list(boxplot.stats(unclass(groups[[i]]), range)) # do.conf=notch)
     stats <- matrix(0, nrow = 5L, ncol = n)
@@ -61,17 +70,19 @@ function(x, ..., range = 1.5, width = NULL, varwidth = FALSE,
 	}
 	ct <- ct+1
     }
-    if(length(cl) && cl != "numeric") oldClass(stats) <- cl
+    if(length(cl) && cl != "numeric")
+        oldClass(stats) <- oldClass(conf) <- oldClass(out) <- cl
     z <- list(stats = stats, n = ng, conf = conf, out = out, group = group,
 	      names = names)
     if(plot) {
         if(is.null(pars$boxfill) && is.null(args$boxfill)) pars$boxfill <- col
-        do.call("bxp",
+        do.call(bxp,
                 c(list(z, notch = notch, width = width, varwidth = varwidth,
                        log = log, border = border, pars = pars,
                        outline = outline, horizontal = horizontal, add = add,
                        ann = ann,
-                       at = at), args[namedargs]))
+                       at = at), args[namedargs]),
+                quote = TRUE)# *not* to eval() calls in labels etc
 	invisible(z)
     }
     else z
@@ -285,7 +296,7 @@ bxp <- function(z, notch = FALSE, width = NULL, varwidth = FALSE,
 
     width <-
 	if(!is.null(width)) {
-	    if(length(width) != n | anyNA(width) | any(width <= 0))
+	    if(length(width) != n || anyNA(width) || any(width <= 0))
 		stop("invalid boxplot widths")
 	    boxwex * width/max(width)
 	}
@@ -316,18 +327,22 @@ bxp <- function(z, notch = FALSE, width = NULL, varwidth = FALSE,
     if(!axes) { axes <- pars$axes; pars$axes <- NULL }
     if(axes) {
 	ax.pars <- pars[names(pars) %in% c("xaxt", "yaxt", "xaxp", "yaxp",
+					   "gap.axis",
                                            "las",
 					   "cex.axis", "col.axis", "format")]
 	if (is.null(show.names)) show.names <- n > 1
 	if (show.names)
 	    do.call("axis", c(list(side = 1 + horizontal,
-				   at = at, labels = z$names), ax.pars))
-	do.call("Axis", c(list(x = z$stats, side = 2 - horizontal), ax.pars))
+				   at = at, labels = z$names), ax.pars),
+		    quote = TRUE)
+	do.call("Axis", c(list(x = z$stats, side = 2 - horizontal), ax.pars),
+		quote = TRUE)
     }
     if(ann) do.call(title,
 	    pars[names(pars) %in% c("main", "cex.main", "col.main",
 				    "sub", "cex.sub", "col.sub",
-				    "xlab", "ylab", "cex.lab", "col.lab")])
+				    "xlab", "ylab", "cex.lab", "col.lab")],
+	    quote = TRUE)
     if(frame.plot)
 	box()
     invisible(at)
