@@ -1,7 +1,7 @@
 #  File src/library/graphics/R/datetime.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -183,9 +183,13 @@ hist.POSIXt <- function(x, breaks, ..., xlab = deparse1(substitute(x)),
         ## trick to swallow arguments for hist.default, separate out 'axes'
         myplot <- function(res, xlab, freq, format, breaks,
                            right, include.lowest, labels = FALSE,
+                           density = NULL, angle = 45, col = NULL,
+                           border = NULL, lty = NULL,
                            axes = TRUE, xaxt = par("xaxt"), ...)
         {
 	    plot(res, xlab = xlab, axes = FALSE, freq = freq,
+                 density = density, angle = angle, col = col,
+                 border = border, lty = lty,
 		 labels = labels, ...)
 	    if(axes) {
 		axis(2, ...)
@@ -338,9 +342,13 @@ hist.Date <- function(x, breaks, ..., xlab = deparse1(substitute(x)),
         ## trick to swallow arguments for hist.default, separate out 'axes'
         myplot <- function(res, xlab, freq, format, breaks,
                            right, include.lowest, labels = FALSE,
+                           density = NULL, angle = 45, col = NULL,
+                           border = NULL, lty = NULL,
                            axes = TRUE, xaxt = par("xaxt"), ...)
         {
             plot(res, xlab = xlab, axes = FALSE, freq = freq,
+                 density = density, angle = angle, col = col,
+                 border = border, lty = lty,
                  labels = labels, ...)
             if(axes && xaxt != "n") {
                 axis(2, ...)
