@@ -114,22 +114,32 @@ convertNative <- function(unit, dimension="x", type="location") {
   .Defunct("convertUnit")
 }
 
-deviceLoc <- function(x, y, valueOnly=FALSE) {
-    result <- grid.Call(C_devLoc, x, y)
+deviceLoc <- function(x, y, valueOnly=FALSE, device=FALSE) {
+    result <- grid.Call(C_devLoc, x, y, as.logical(device))
     names(result) <- c("x", "y")
-    if (!valueOnly)
-        list(x=unit(result$x, "in"), y=unit(result$y, "in"))
-    else
+    if (valueOnly) {
         result
+    } else {
+        if (device) {
+            list(x=unit(result$x, "native"), y=unit(result$y, "native"))
+        } else {
+            list(x=unit(result$x, "in"), y=unit(result$y, "in"))
+        }
+    }
 }
 
-deviceDim <- function(w, h, valueOnly=FALSE) {
-    result <- grid.Call(C_devDim, w, h)
+deviceDim <- function(w, h, valueOnly=FALSE, device=FALSE) {
+    result <- grid.Call(C_devDim, w, h, as.logical(device))
     names(result) <- c("w", "h")
-    if (!valueOnly)
-        list(w=unit(result$w, "in"), h=unit(result$h, "in"))
-    else
+    if (valueOnly) {
         result
+    } else {
+        if (device) {
+            list(w=unit(result$w, "native"), h=unit(result$h, "native"))
+        } else {
+            list(w=unit(result$w, "in"), h=unit(result$h, "in"))
+        }
+    }
 }
 
 # This is like the "convert" functions:  it evaluates units (immediately)
@@ -451,7 +461,7 @@ pSummary <- function(..., op) {
 }
 `[[.unit` <- function(x, index, ...) {
     if (length(index) != 1) {
-        stop("index must be of length 1", call = FALSE)
+        stop("index must be of length 1", call. = FALSE)
     }
     x[index]
 }
@@ -480,10 +490,10 @@ pSummary <- function(..., op) {
 }
 `[[<-.unit` <- function(x, i, value) {
     if (length(i) != 1) {
-        stop("index must be of length 1", call = FALSE)
+        stop("index must be of length 1", call. = FALSE)
     }
     if (length(value) != 1) {
-        stop("replacement must be of length 1", call = FALSE)
+        stop("replacement must be of length 1", call. = FALSE)
     }
     x[i] <- value
     x
