@@ -27,11 +27,54 @@ initvpAutoName <- function() {
 
 vpAutoName <- initvpAutoName()
 
+vpObject <- function(x, y, width, height, just,
+                     gp, clip, mask,
+                     xscale, yscale, angle,
+                     layout, layout.pos.row, layout.pos.col,
+                     name) {
+    
+    vp <- list(x = x, y = y, width = width, height = height,
+               justification = just,
+               gp = gp,
+               clip = clip,
+               xscale = xscale,
+               yscale = yscale,
+               angle = angle,
+               layout = layout,
+               layout.pos.row = layout.pos.row,
+               layout.pos.col = layout.pos.col,
+               valid.just = valid.just(just),
+               valid.pos.row = layout.pos.row,
+               valid.pos.col = layout.pos.col,
+               name = name,
+               ## A whole lot of blank slots that pushedvp() fills in
+               parentgpar = NULL,
+               gpar = NULL,
+               trans = NULL,
+               widths = NULL,
+               heights = NULL,
+               width.cm = NULL,
+               height.cm = NULL,
+               rotation = NULL,
+               cliprect = NULL,
+               parent = NULL,
+               children = NULL,
+               devwidth = NULL,
+               devheight = NULL,
+               clippath = NULL,
+               ## Some viewport slots that were added later on
+               ## (pairs of 'vp' and 'pushedvp' slots)
+               mask = mask,
+               resolvedmask = NULL)
+    class(vp) <- "viewport"
+    vp
+}
+
 # NOTE: The order of the elements in viewports and pushedvps are
 # VERY IMPORTANT because the C code accesses them using constant
 # indices (i.e., if you change the order here the world will end!
 valid.viewport <- function(x, y, width, height, just,
-                           gp, clip,
+                           gp, clip, mask,
                            xscale, yscale, angle,
                            layout, layout.pos.row, layout.pos.col,
                            name) {
@@ -40,12 +83,27 @@ valid.viewport <- function(x, y, width, height, just,
     stop("'x', 'y', 'width', and 'height' must all be units of length 1")
   if (!is.gpar(gp))
     stop("invalid 'gp' value")
-  if (!is.logical(clip))
-    clip <- switch(as.character(clip),
-                   on=TRUE,
-                   off=NA,
-                   inherit=FALSE,
-                   stop("invalid 'clip' value"))
+  if (!is.logical(clip)) {
+      if (is.grob(clip)) {
+          clip <- createClipPath(clip)
+      } else {
+          clip <- switch(as.character(clip),
+                         on=TRUE,
+                         off=NA,
+                         inherit=FALSE,
+                         stop("invalid 'clip' value"))
+      }
+  }
+  if (!is.logical(mask)) {
+      if (is.grob(mask)) {
+          mask <- createMask(mask)
+      } else {
+          mask <- switch(as.character(mask),
+                         inherit=TRUE,
+                         none=FALSE,
+                         stop("invalid 'mask' value"))
+      }
+  }
   # Ensure both 'xscale' and 'yscale' are numeric (brute force defense)
   xscale <- as.numeric(xscale)
   yscale <- as.numeric(yscale)
@@ -75,22 +133,11 @@ valid.viewport <- function(x, y, width, height, just,
   if (is.null(name))
     name <- vpAutoName()
   # Put all the valid things first so that are found quicker
-  vp <- list(x = x, y = y, width = width, height = height,
-             justification = just,
-             gp = gp,
-             clip = clip,
-             xscale = xscale,
-             yscale = yscale,
-             angle = angle,
-             layout = layout,
-             layout.pos.row = layout.pos.row,
-             layout.pos.col = layout.pos.col,
-             valid.just = valid.just(just),
-             valid.pos.row = layout.pos.row,
-             valid.pos.col = layout.pos.col,
-             name=name)
-  class(vp) <- "viewport"
-  vp
+  ## Order is VERY important
+  vpObject(x, y, width, height, valid.just(just),
+           gp, clip, mask, xscale, yscale,
+           angle, layout, layout.pos.row, layout.pos.col,
+           name)
 }
 
 # When a viewport is pushed, an internal copy is stored along
@@ -102,35 +149,37 @@ pushedvp <- function(vp) {
     # either directly from L_setviewport() or indirectly from initVP()
     # via grid.top.level.vp()
     # vp$gpar and vp$parentgpar are both set previously in push.vp.viewport()
-  pvp <- c(vp, list(trans = NULL,
-                    widths = NULL,
-                    heights = NULL,
-                    width.cm = NULL,
-                    height.cm = NULL,
-                    rotation = NULL,
-                    cliprect = NULL,
-                    parent = NULL,
-                    # Children of this pushedvp will be stored
-                    # in an environment
-                    children = new.env(hash=TRUE, parent=baseenv()),
-                    # Initial value of 0 means that the viewport will
-                    # be pushed "properly" the first time, calculating
-                    # transformations, etc ...
-                    devwidthcm = 0,
-                    devheightcm = 0))
-  class(pvp) <- c("pushedvp", class(vp))
-  pvp
+    pvp <- vp
+    ## Children of this pushedvp will be stored
+    ## in an environment
+    pvp$children = new.env(hash=TRUE, parent=baseenv())
+    ## Initial value of 0 means that the viewport will
+    ## be pushed "properly" the first time, calculating
+    ## transformations, etc ...
+    pvp$devwidthcm <- 0
+    pvp$devheightcm <- 0
+    class(pvp) <- c("pushedvp", class(vp))
+    pvp
 }
 
 vpFromPushedvp <- function(pvp) {
-  vp <- pvp[c("x", "y", "width", "height",
-              "justification", "gp", "clip",
-              "xscale", "yscale", "angle",
-              "layout", "layout.pos.row", "layout.pos.col",
-              "valid.just", "valid.pos.row", "valid.pos.col",
-              "name")]
-  class(vp) <- "viewport"
-  vp
+    ## Unresolve any resolved fills
+    if (!is.null(pvp$gp$fill)) {
+        pvp$gp$fill <- unresolveFill(pvp$gp$fill)
+    }
+    ## Unresolve any clip paths or masks
+    if (isClipPath(pvp$clip)) {
+        pvp$clip <- unresolveClipPath(pvp$clip)
+    }
+    if (isMask(pvp$mask)) {
+        pvp$mask <- unresolveMask(pvp$mask)
+    }
+    ## Only keep non-pushedvp content
+    with(unclass(pvp),
+         vpObject(x, y, width, height, justification,
+                  gp, clip, mask, xscale, yscale,
+                  angle, layout, layout.pos.row, layout.pos.col,
+                  name))
 }
 
 as.character.viewport <- function(x, ...) {
@@ -216,6 +265,7 @@ viewport <- function(x = unit(0.5, "npc"),
                      just = "centre",
                      gp = gpar(),
                      clip = "inherit",
+                     mask = "inherit", # or "none" or grob
                      # FIXME: scales are only linear at the moment
                      xscale = c(0, 1),
                      yscale = c(0, 1),
@@ -228,17 +278,17 @@ viewport <- function(x = unit(0.5, "npc"),
                      # This is down here to avoid breaking
                      # existing code
                      name=NULL) {
-  if (!is.unit(x))
-    x <- unit(x, default.units)
-  if (!is.unit(y))
-    y <- unit(y, default.units)
-  if (!is.unit(width))
-    width <- unit(width, default.units)
-  if (!is.unit(height))
-    height <- unit(height, default.units)
-  valid.viewport(x, y, width, height, just,
-                 gp, clip, xscale, yscale, angle,
-                 layout, layout.pos.row, layout.pos.col, name)
+    if (!is.unit(x))
+        x <- unit(x, default.units)
+    if (!is.unit(y))
+        y <- unit(y, default.units)
+    if (!is.unit(width))
+        width <- unit(width, default.units)
+    if (!is.unit(height))
+        height <- unit(height, default.units)
+    valid.viewport(x, y, width, height, just,
+                   gp, clip, mask, xscale, yscale, angle,
+                   layout, layout.pos.row, layout.pos.col, name)
 }
 
 is.viewport <- function(vp) {
@@ -410,3 +460,12 @@ dataViewport <- function(xData = NULL, yData = NULL,
     }
     viewport(xscale = xscale, yscale = yscale, ...)
 }
+
+editViewport <- function(vp=current.viewport(), ...) {
+    edits <- list(...)
+    vp <- vpFromPushedvp(vp)
+    vp[names(edits)] <- edits
+    valid.viewport(vp$x, vp$y, vp$width, vp$height, vp$just,
+                   vp$gp, vp$clip, vp$mask, vp$xscale, vp$yscale, vp$angle,
+                   vp$layout, vp$layout.pos.row, vp$layout.pos.col, vp$name)
+}
