@@ -1,7 +1,7 @@
 #  File src/library/methods/R/RMethodUtils.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -84,11 +84,11 @@
     args <- formalArgs(fdef)
     if(is.null(signature))
         signature <- args
-    else if(any(is.na(match(signature, args))))
-        stop(sprintf(ngettext(sum(is.na(match(signature, args))),
+    else if(any(not.s.in.a <- is.na(match(signature, args))))
+        stop(sprintf(ngettext(sum(not.s.in.a),
                               "non-argument found in the signature: %s",
                               "non-arguments found in the signature: %s"),
-                     paste(signature[is.na(match(signature, args))], collapse = ", ")),
+                     paste(signature[not.s.in.a], collapse = ", ")),
              domain = NA)
     dots <- match("...", signature)
     if(!is.na(dots)) { # remove "..." unless it is the only element of the signature
@@ -164,11 +164,11 @@ makeGeneric <-
     args <- formalArgs(fdef)
     if(is.null(signature))
         signature <- args
-    else if(any(is.na(match(signature, args))))
-        stop(sprintf(ngettext(sum(is.na(match(signature, args))),
+    else if(any(not.s.in.a <- is.na(match(signature, args))))
+        stop(sprintf(ngettext(sum(not.s.in.a),
                               "non-argument found in the signature: %s",
                               "non-arguments found in the signature: %s"),
-                     paste(signature[is.na(match(signature, args))], collapse = ", ")),
+                     paste(signature[not.s.in.a], collapse = ", ")),
              domain = NA)
     attr(signature, "simpleOnly") <- simpleInheritanceOnly # usually NULL
     dots <- match("...", signature)
@@ -334,20 +334,18 @@ conformMethod <- function(signature, mnames, fnames,
     ##                       label, paste(missingFnames[foundNames], collapse = ", ")),
     ##              domain = NA)
     if(!any(omittedSig))
-      return(signature)
-    if(any(is.na(match(signature[omittedSig], c("ANY", "missing"))))) {
-        bad <- omittedSig & is.na(match(signature[omittedSig], c("ANY", "missing")))
+        return(signature)
+    if(any(iiN <- is.na(match(signature[omittedSig], c("ANY", "missing"))))) {
+        bad <- omittedSig & iiN
         bad2 <- paste0(fnames[bad], " = \"", signature[bad], "\"", collapse = ", ")
         stop(.renderSignature(f, sig0),
              gettextf("formal arguments (%s) omitted in the method definition cannot be in the signature", bad2),
              call. = TRUE, domain = NA)
     }
-    else if(!all(signature[omittedSig] == "missing")) {
-        omittedSig <- omittedSig && (signature[omittedSig] != "missing")
+    else if(any(omittedSig <- omittedSig & signature != "missing")) {
         .message("Note: ", .renderSignature(f, sig0),
                  gettextf("expanding the signature to include omitted arguments in definition: %s",
                           paste(sigNames[omittedSig], "= \"missing\"",collapse = ", ")))
-        omittedSig <- seq_along(omittedSig)[omittedSig] # logical index will extend signature!
         signature[omittedSig] <- "missing"
     }
     ## remove trailing "ANY"'s
@@ -361,54 +359,64 @@ conformMethod <- function(signature, mnames, fnames,
 
 rematchDefinition <- function(definition, generic, mnames, fnames, signature)
 {
-    added <- any(is.na(match(mnames, fnames)))
+    added <- anyNA(match(mnames, fnames))
     keepsDots <- !is.na(match("...", mnames))
     if(!added && keepsDots) {
         ## the formal args of the method must be identical to generic
-        formals(definition, envir = environment(definition)) <- formals(generic)
+        formals(definition) <- formals(generic)
         return(definition)
     }
     dotsPos <- match("...", fnames)
     if(added && is.na(dotsPos))
-        stop(gettextf("methods can add arguments to the generic %s only if '...' is an argument to the generic", sQuote(generic@generic)),
+        stop(gettextf("methods can add arguments to the generic %s only if '...' is an argument to the generic",
+                      sQuote(generic@generic)),
              call. = TRUE)
     ## pass down all the names in common between method & generic,
     ## plus "..."  even if the method doesn't have it.  But NOT any
     ## arguments having class "missing" implicitly (see conformMethod),
     ## i.e., are not among 'mnames':
-    useNames <- !is.na(imf <- match(fnames, mnames)) | fnames == "..."
-    newCall <- lapply(c(".local", fnames[useNames]), as.name)
-
+    useNames <- (useNm <- !is.na(imf <- match(fnames, mnames))) | fnames == "..."
     ## Should not be needed, if conformMethod() has already been called:
-    if(is.unsorted(imf[!is.na(imf)]))
+    if(is.unsorted(imf[useNm]))
 	stop(.renderSignature(generic@generic, signature),
              "formal arguments in method and generic do not appear in the same order",
              call. = FALSE)
-
+    clArgs <- fnames[useNames]
     ## leave newCall as a list while checking the trailing args
     if(keepsDots && dotsPos < length(fnames)) {
-	## Trailing arguments are required to match.  This is a little
+	## Trailing arguments (those after "...") are required to match.  This is a little
 	## stronger than necessary, but this is a dicey case, because
 	## the argument-matching may not be consistent otherwise (in
 	## the generic, such arguments have to be supplied by name).
 	## The important special case is replacement methods, where
 	## value is the last argument.
-
 	ntrail <- length(fnames) - dotsPos
 	trailingArgs <- fnames[seq.int(to = length(fnames), length.out = ntrail)]
-	if(!identical(	mnames[seq.int(to = length(mnames), length.out = ntrail)],
-		      trailingArgs))
+	if (!identical (mnames[seq.int(to = length(mnames), length.out = ntrail)],
+                        trailingArgs))
 	    stop(gettextf("%s arguments (%s) after %s in the generic must appear in the method, in the same place at the end of the argument list",
                           .renderSignature(generic@generic, signature),
 			  paste(sQuote(trailingArgs), collapse = ", "),
                           sQuote("...")),
                  call. = FALSE, domain = NA)
-	newCallNames <- character(length(newCall))
-	newCallNames[seq.int(to = length(newCallNames), length.out = ntrail)] <-
-	    trailingArgs
-	names(newCall) <- newCallNames
+	clNames <- character(length(clArgs))
+	clNames[seq.int(to = length(clNames), length.out = ntrail)] <- trailingArgs
+    } else
+        clNames <- NULL
+    if((iMi <- match("missing", signature, nomatch=0L)) && length(iNm <- which(useNm)) &&
+       any(i <- (iMi <= iNm & iNm <=
+                 if(is.na(dotsPos)) length(fnames) else dotsPos-1L))) {
+        ## name args in .local(..) call because we have "missing" in method signature
+	if(is.null(clNames))
+	    clNames <- character(length(clArgs))
+        ## fnames[iNm] == fnames[useNm] is subset of clArgs := fnames[useNames]
+        im <- match(fnames[iNm][i], clArgs)
+	clNames[im] <- clArgs[im]
     }
-    newCall <- as.call(newCall)
+    if(!is.null(clNames))
+        names(clArgs) <- clNames
+    newCall <- as.call(lapply(c(".local", clArgs), as.name))
+    ##== newCall <- as.call(c(quote(.local), lapply(clArgs, as.name)))
     newBody <- substitute({.local <- DEF; NEWCALL},
 			  list(DEF = definition, NEWCALL = newCall))
     generic <- .copyMethodDefaults(generic, definition)
@@ -484,7 +492,7 @@ getGeneric <-
     }
 }
 
-## low-level version
+## low-level version (called from high level)
 .getGeneric <- function(f, where = .GlobalEnv, # default only for C search
                         package = "")
 {
@@ -497,19 +505,13 @@ getGeneric <-
             message("Empty function name in .getGeneric")
             dput(sys.calls())
         }
+        ## ../src/methods_list_dispatch.c -- R_getGeneric(SEXP name, SEXP mustFind, SEXP env, SEXP package) :
         value <- .Call(C_R_getGeneric, f, FALSE, as.environment(where), package)
         ## cache public generics (usually these will have been cached already
         ## and we get to this code for non-exported generics)
-        if(!is.null(value) && !is.null(vv <- .GlobalEnv[[f]]) &&
-           identical(vv, value))
+        if(!is.null(value) && !is.null(vv <- .GlobalEnv[[f]]) && identical(vv, value))
             .cacheGeneric(f, value)
     }
-    ##     if(is.null(value) && nzchar(package) && !identical(package, "base")) {
-    ##         env <- .requirePackage(package, FALSE)
-    ##         if(is.environment(env))
-    ##           value <- .Call("R_getGeneric", f, FALSE, env, package,
-    ##                      PACKAGE = "methods")
-    ##     }
     value
 }
 
@@ -790,12 +792,12 @@ getGenerics <- function(where, searchForm = FALSE)
     these <- these[startsWith(these, ".__T__")]
     if(length(these) == 0L)
         return(character())
-    funNames <- gsub("^.__T__(.*):([^:]+)", "\\1", these)
+    funNames <- gsub("^\\.__T__(.*):([^:]+)", "\\1", these)
     ## FIXME: length(funNames) == length(these) != 0   ==> this never triggers:
     ## if(length(funNames) == 0L && any(startsWith(these, ".__M__")))
     ##     warning(sprintf("package %s seems to have out-of-date methods; need to reinstall from source",
     ##                      sQuote(getPackageName(where[[1L]]))))
-    packageNames <- gsub(".__T__(.*):([^:]+(.*))", "\\2", these)
+    packageNames <- gsub("^\\.__T__(.*):([^:]+(.*))", "\\2", these)
     attr(funNames, "package") <- packageNames
     ## Would prefer following, but may be trouble bootstrapping methods
     ## funNames <- new("ObjectsWithPackage", funNames, package = packageNames)
@@ -804,9 +806,10 @@ getGenerics <- function(where, searchForm = FALSE)
     else if(isFALSE(trim))
         these
     else
-        gsub(".__T__", as.character(trim), these)
+        gsub(".__T__", as.character(trim), these, fixed=TRUE)
 }
 
+## also called from base::loadNamespace, unloadNamespace(), attach() & detach()
 cacheMetaData <-
     function(where, attach = TRUE, searchWhere = as.environment(where),
              doCheck = TRUE)
@@ -815,13 +818,27 @@ cacheMetaData <-
     ## to update class and method information.
     pkg <- getPackageName(where)
     classes <- getClasses(where)
+    lev <- 0L
     if (attach) {
+        msg <- Sys.getenv("_R_TRACE_LOADNAMESPACE_", "")
+        if (nzchar(msg)) {
+            if(pkg %in%
+               c("base", "tools", "utils", "grDevices", "graphics",
+                 "stats", "datasets", "methods", "grid", "splines", "stats4",
+                 "tcltk", "compiler", "parallel")) lev <- 0L
+            else {
+                lev <- as.integer(msg)
+                if(is.na(lev)) lev <- 0L
+            }
+        }
         for(cl in classes) {
+            if(lev > 2L) message("--- caching class ", sQuote(cl))
             ## NOT getClassDef, it will use cache
             cldef <- get(classMetaName(cl), where)
             if(is(cldef, "classRepresentation"))
                 .cacheClass(cl, cldef, is(cldef, "ClassUnionRepresentation"),
                             where)
+            if(lev > 2L) message("--- done caching class ", sQuote(cl))
         }
     } else {
         for(cl in classes) {
@@ -853,7 +870,9 @@ cacheMetaData <-
     for(i in seq_along(generics)) {
         f <- generics[[i]]
         fpkg <- packages[[i]]
+        .tr <- attach && (lev > 2L || lev == -5L)
         if(!identical(fpkg, pkg) && doCheck) {
+            if(.tr) message("--- getting generic ", sQuote(f), " (and methods)")
             if(attach) {
                 env <- as.environment(where)
                 ## All instances of this generic in different attached packages must
@@ -881,12 +900,13 @@ cacheMetaData <-
             fdef <- getGeneric(f, FALSE, searchWhere, fpkg)
         if(!is(fdef, "genericFunction"))
             next ## silently ignores all generics not visible from searchWhere
-        if(attach)
+        if(attach) {
             .cacheGeneric(f, fdef)
-        else
+        } else
             .uncacheGeneric(f, fdef)
         methods <- .updateMethodsInTable(fdef, where, attach)
         cacheGenericsMetaData(f, fdef, attach, where, fdef@package, methods)
+        if(.tr) message("--- done getting generic ", sQuote(f))
     }
     .doLoadActions(where, attach)
     invisible(NULL) ## as some people call this at the end of functions
