@@ -1,7 +1,7 @@
 #  File src/library/parallel/R/windows/mcdummies.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -48,4 +48,8 @@ mcmapply <-
            USE.NAMES = USE.NAMES)
 }
 
-mcMap <- function (f, ...) Map(f, ...)
+mcMap <- function (f, ...)
+{
+    f <- match.fun(f)
+    mcmapply(f, ..., SIMPLIFY = FALSE, mc.silent = TRUE)
+}
