@@ -80,7 +80,7 @@ static void matrix_prod(Array mat1, Array mat2, int trans1, int trans2,
 static void assert(int bool)
 {
     if(!bool)
-	error(("assert failed in src/library/ts/src/carray.c"));
+	error(_("assert failed in src/library/ts/src/carray.c"));
 }
 
 static Array init_array(void)
