@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 2003-2004  The R Foundation
- *  Copyright (C) 1998--2013  The R Core Team
+ *  Copyright (C) 1998--2014  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,19 +16,22 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#define NO_NLS
 #include <Defn.h>
 #include <float.h>		/* for DBL_MAX */
 #include <R_ext/Applic.h>	/* for optif9, fdhess */
 #include <R_ext/RS.h>	       	/* for Memcpy */
 
 #include "statsR.h"
+#include "stats.h" // R_zeroin2
+
 #undef _
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -189,11 +192,11 @@ double Brent_fmin(double ax, double bx, double (*f)(double, void *),
     /* end of main loop */
 
     return x;
-}
+} // Brent_fmin()
 
 
 /* One Dimensional Minimization --- just wrapper for
- * Brent's "fmin" --> ../appl/fmin.c */
+ * Brent's "fmin" above */
 
 struct callinfo {
   SEXP R_fcall;
@@ -289,12 +292,6 @@ SEXP do_fmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 // Brent's "zeroin"
 // ---------------
 
-extern double 
-R_zeroin2(double ax, double bx, double fa, double fb, 
-	  double (*f)(double x, void *info), void *info, 
-	  double *Tol, int *Maxit);
-
-
 static double fcn2(double x, struct callinfo *info)
 {
     SEXP s, sx;
@@ -517,7 +514,8 @@ static void fcn(int n, const double x[], double *f, function_info
 	return;
     }
 				/* calculate for a new value of x */
-    s = CADR(R_fcall);
+    s = allocVector(REALSXP, n);
+    SETCADR(R_fcall, s);
     for (i = 0; i < n; i++) {
 	if (!R_FINITE(x[i])) error(_("non-finite value supplied by 'nlm'"));
 	REAL(s)[i] = x[i];
@@ -632,7 +630,7 @@ static double *fixparam(SEXP p, int *n)
 
 	/* Fatal errors - we don't deliver an answer */
 
-static void opterror(int nerr)
+static void NORET opterror(int nerr)
 {
     switch(nerr) {
     case -1:
@@ -692,7 +690,7 @@ from above in some direction,\n"\
     Rprintf("\n");
 }
 
-/* NOTE: The actual Dennis-Schnabel algorithm `optif9' is in ../appl/uncmin.c */
+/* NOTE: The actual Dennis-Schnabel algorithm `optif9' is in ../../../appl/uncmin.c */
 
 SEXP nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
@@ -782,12 +780,10 @@ SEXP nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
     R_gradientSymbol = install("gradient");
     R_hessianSymbol = install("hessian");
 
-    /* This vector is shared with all subsequent calls */
     v = allocVector(REALSXP, n);
     for (i = 0; i < n; i++) REAL(v)[i] = x[i];
     SETCADR(state->R_fcall, v);
-    SET_NAMED(v, 2); // in case the functions try to alter it
-    value = eval(state->R_fcall, state->R_env);
+    PROTECT(value = eval(state->R_fcall, state->R_env));
 
     v = getAttrib(value, R_gradientSymbol);
     if (v != R_NilValue) {
@@ -808,6 +804,7 @@ SEXP nlm(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    warning(_("gradient supplied is of the wrong length or mode, so ignored"));
 	}
     }
+    UNPROTECT(1); /* value */
     if (((msg/4) % 2) && !iahflg) { /* skip check of analytic Hessian */
       msg -= 4;
     }
