@@ -57,7 +57,9 @@ nls(y ~ a+b*x+c*I(x^2), start = c(a=1, b=1, c=0.1), algorithm = "port")
 (fm <- nls(y ~ a+b*x+c*I(x^2), start = c(a=1, b=1, c=0.1),
            algorithm = "port", lower = c(0, 0, 0)))
 ## IGNORE_RDIFF_END
-if(have_MASS) print(confint(fm))
+if(have_MASS) {
+    print(confint(fm))
+} else message("skipping tests requiring the MASS package")
 
 ## weighted nls fit
 set.seed(123)
@@ -247,11 +249,12 @@ t1$with.start
 ## finally worked in 2.4.0
 ##__no.start: failed in 3.0.2
 ## 2018-09 fails on macOS with Accelerate framework.
-stopifnot(all.equal(.n(t1[[1]]), .n(t1[[2]])))
+stopifnot(all.equal(.n(t1[[1]]), .n(t1[[2]]), check.environment = FALSE))
 rm(a,b)
 t2 <- test(FALSE)
 stopifnot(all.equal(lapply(t1, .n),
-		    lapply(t2, .n), tolerance = 0.16))# different random error
+		    lapply(t2, .n), tolerance = 0.16, # different random error
+                    check.environment = FALSE))
 
 
 ## list 'start'
@@ -323,3 +326,40 @@ options(op) # warnings about missing 'start' ok:
 f.1 <- nls(y ~ a) # failed in R 3.4.x
 stopifnot(all.equal(noC(f.1), noC(fi1)),
 	  all.equal(coef(f.1), c(a = mean(y))))
+
+
+##--- New option 'central' for numericDeriv() :
+
+## Continuing the  pnorm()  example from  example(numericDeriv):
+
+mkEnv <- function(n, from = -3, to = 3) {
+    stopifnot(is.numeric(n), n >= 2)
+    E <- new.env()
+    E$mean <- 0.
+    E$sd   <- 1.
+    E$x    <- seq(from, to, length.out = n)
+    E
+}
+
+pnEnv <- mkEnv(65) # is used inside  errE() :
+
+## varying eps (very platform dependent?):
+errE <- Vectorize(function(eps, central=FALSE) {
+  grad <- attr(numericDeriv(quote(pnorm(x, mean, sd)), c("mean", "sd"),
+                            pnEnv, eps=eps, central=central), "gradient")
+  target <- with(pnEnv, -dnorm(x) * cbind(1, x, deparse.level=0L))
+  ## return relative error {in the same sense as in all.equal()} :
+  sum(abs(target - grad)) / sum(abs(target))
+})
+
+curve(errE(x), 1e-9, 1e-4, log="xy", n=512, ylim = c(1.5e-11, 5e-7),
+      xlab = quote(epsilon), ylab=quote(errE(epsilon))) -> rex
+axis(1, at = 2^-(52/2), label = quote(sqrt(epsilon[c])), col=4, col.axis=4, line=-1/2)
+axis(1, at = 2^-(52/3), label = quote(epsilon[c]^{1/3}), col=4, col.axis=4, line=-1/2)
+curve(errE(x, central=TRUE), n=512, col=2, add = TRUE) -> rexC
+## IGNORE_RDIFF_BEGIN
+str(xy1 <- approx(rex , xout= sqrt(2^-52)) )
+str(xy2 <- approx(rexC, xout=(2^-52)^(1/3)))
+## IGNORE_RDIFF_END
+lines(xy1, type="h", col=4)
+lines(xy2, type="h", col=4)
