@@ -1,6 +1,6 @@
 #  File src/library/tools/R/Rd2HTML.R
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #  Part of the R package, https://www.R-project.org
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@ get_link <- function(arg, tag, Rdfile) {
     ## \link[pkg]{bar} means show bar and link to *file* bar in package pkg
     ## \link{pkg:bar]{foo} means show foo and link to file bar in package pkg.
     ## As from 2.10.0, look for topic 'bar' if file not found.
+    ## As from 4.1.0, prefer topic 'bar' over file 'bar' (in which case 'targetfile' is a misnomer)
 
     if (!all(RdTags(arg) == "TEXT"))
     	stopRd(arg, Rdfile, "Bad \\link text")
@@ -101,7 +102,7 @@ vhtmlify <- function(x, inEqn = FALSE) { # code version
     ## http://htmlhelp.com/reference/html40/entities/symbols.html
     if(inEqn) {
         x <- psub("\\\\(Alpha|Beta|Gamma|Delta|Epsilon|Zeta|Eta|Theta|Iota|Kappa|Lambda|Mu|Nu|Xi|Omicron|Pi|Rho|Sigma|Tau|Upsilon|Phi|Chi|Psi|Omega|alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega|le|ge|sum|prod)", "&\\1;", x)
-        x <- psub("\\\\(dots|ldots)", "&\\hellip;", x)
+        x <- psub("\\\\(dots|ldots)", "&hellip;", x)
         x <- fsub("\\infty", "&infin;", x)
         x <- fsub("\\sqrt", "&radic;", x)
     }
@@ -110,8 +111,8 @@ vhtmlify <- function(x, inEqn = FALSE) { # code version
 
 shtmlify <- function(s) {
     s <- gsub("&", "&amp;", s, fixed = TRUE)
-    s <- gsub("<", "&lt;", s, fixed = TRUE)
-    s <- gsub(">", "&gt;", s, fixed = TRUE)
+    s <- gsub("<", "&lt;" , s, fixed = TRUE)
+    s <- gsub(">", "&gt;" , s, fixed = TRUE)
     s
 }
 
@@ -152,6 +153,60 @@ escapeAmpersand <- function(x) gsub("&", "&amp;", x, fixed = TRUE)
 invalid_HTML_chars_re <-
     "[\u0001-\u0008\u000b\u000c\u000e-\u001f\u007f-\u009f]"
 
+
+topic2filename <- function(x) gsub("%", "+", utils::URLencode(x, reserved=TRUE))
+
+
+## Create HTTP redirect files for aliases; called only during package
+## installation if static help files are enabled. Files are named
+## after aliases, which may contain 'undesirable' characters. These
+## are escaped using topic2filename(). Analogous escaping needs to be
+## done when creating links in HTML output as well, but ONLY for
+## static HTML (dynamic help is already capable of handling such
+## links)
+createRedirects <- function(file, Rdobj)
+{
+    linksToTopics <-
+        config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
+    if (!linksToTopics) return(invisible()) # do nothing
+    ## create a HTTP redirect for each 'alias' in .../pkg/help/
+    redirHTML <- sprintf("<html><head><meta http-equiv='refresh' content='0; url=../html/%s'></head></html>\n", urlify(basename(file)))
+    toProcess <- which(RdTags(Rdobj) == "\\alias")
+    helpdir <- paste0(dirname(dirname(file)), "/help") # .../pkg/help/
+    aliasName <- function(i) trimws(Rdobj[[i]][[1]])
+    aliasFile <- function(i) file.path(helpdir, sprintf("%s.html", topic2filename(aliasName(i))))
+    redirMsg <- function(type, src, dest, status) {
+        ## change sprintf to gettextf to make translatable, but seems unnecessary
+        msg <- sprintf("\nREDIRECT:%s\t %s -> %s [ %s ]", type, src, dest, status)
+        message(msg, appendLF = FALSE)
+    }
+    ## remove duplicate aliases, if any
+    aliasesToProcess <- sapply(toProcess, aliasName)
+    toProcess <- toProcess[!duplicated(aliasesToProcess)]
+    for (i in toProcess) {
+        aname <- aliasName(i)
+        afile <- aliasFile(i)
+        if (file.exists(afile)) {
+            ## warning("Previous alias or file overwritten by alias: ", aname)
+            msg <- sprintf("\nREDIRECT:topic\t Previous alias or file overwritten by alias: %s",
+                           afile)
+            message(msg, appendLF = FALSE)
+        }
+        try(suppressWarnings(cat(redirHTML, file = afile)), silent = TRUE) # Fails for \alias{%/%}
+        ## redirMsg("topic", aname, basename(file), if (file.exists(afile)) "SUCCESS" else "FAIL")
+        if (!file.exists(afile)) redirMsg("topic", aname, basename(file), "FAIL")
+    }
+    ## Also add .../pkg/help/file.html -> ../pkg/html/file.html as fallback
+    ## when topic is not found (but do not overwrite)
+    file.fallback <- file.path(helpdir, basename(file))
+    if (!file.exists(file.fallback)) {
+        try(cat(redirHTML, file = file.fallback), silent = TRUE)
+        ## redirMsg("file", basename(file), basename(file), if (file.exists(file.fallback)) "SUCCESS" else "FAIL")
+        if (!file.exists(file.fallback)) redirMsg("file", basename(file), basename(file),  "FAIL")
+    }
+}
+
+
 ## This gets used two ways:
 
 ## 1) With dynamic = TRUE from tools:::httpd()
@@ -179,7 +234,13 @@ Rd2HTML <-
              dynamic = FALSE, no_links = FALSE, fragment=FALSE,
              stylesheet = "R.css", ...)
 {
+    ## Is this package help, as opposed to from Rdconv or similar?
+    ## Used to decide whether redirect files should be created when
+    ## generating static HTML
+    package_help <- inherits(Rd, "Rd") && (length(package) == 2L)
     if (missing(no_links) && is.null(Links) && !dynamic) no_links <- TRUE
+    linksToTopics <-
+        config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
     version <- ""
     if(!identical(package, "")) {
         if(length(package) > 1L) {
@@ -188,8 +249,7 @@ Rd2HTML <-
         } else {
             dir <- dirname(package)
             if(nzchar(dir) &&
-               file_test("-f", dfile <- file.path(package,
-                                                  "DESCRIPTION"))) {
+               file_test("-f", dfile <- file.path(package, "DESCRIPTION"))) {
                 version <- .read_description(dfile)["Version"]
                 package <- basename(package)
             } else {
@@ -331,7 +391,19 @@ Rd2HTML <-
                 htmlfile <- paste0("../../", urlify(package), "/help/", urlify(topic))
                 writeHref()
                 return()
-            } else {
+            }
+            else if (linksToTopics && !is.null(Links) && !is.na(Links[topic]) &&
+                     startsWith(Links[topic], paste0("../../", urlify(package)))) {
+                ## only if the topic exists in the package (else look
+                ## harder below). 'Links' contains all topics in the
+                ## package, but also those in base+recommended
+                ## packages. We do this branch only if this is a
+                ## within-package link
+                htmlfile <- paste0("../../", urlify(package), "/help/", topic2filename(topic), ".html")
+                writeHref()
+                return()
+
+            } else { # identify actual file containing topic
             	htmlfile  <- NA_character_
             	if (!is.null(Links)) {
             	    tmp <- Links[topic]
@@ -358,9 +430,12 @@ Rd2HTML <-
             }
     	} else {
             ## ----------------- \link[pkg]{file} and \link[pkg:file]{bar}
-            htmlfile <- paste0(urlify(parts$targetfile), ".html")
-            if (!dynamic && !no_links &&
-               nzchar(pkgpath <- system.file(package = parts$pkg))) {
+            if (!dynamic && !linksToTopics && !no_links &&
+                nzchar(pkgpath <- system.file(package = parts$pkg))) {
+                ## old-style static HTML: prefer filename over topic,
+                ## so treat as filename and urlify() instead of
+                ## topic2filename()
+                htmlfile <- paste0(urlify(parts$targetfile), ".html")
                 ## check the link, only if the package is found
                 OK <- FALSE
                 if (!file.exists(file.path(pkgpath, "html", htmlfile))) {
@@ -375,10 +450,10 @@ Rd2HTML <-
                     ## so how about as a topic?
                     file <- utils:::index.search(parts$targetfile, pkgpath)
                     if (length(file)) {
-                        warnRd(block, Rdfile,
-                               "file link ", sQuote(parts$targetfile),
-                               " in package ", sQuote(parts$pkg),
-                               " does not exist and so has been treated as a topic")
+                        ## warnRd(block, Rdfile,
+                        ##        "file link ", sQuote(parts$targetfile),
+                        ##        " in package ", sQuote(parts$pkg),
+                        ##        " does not exist and so has been treated as a topic")
                         parts$targetfile <- basename(file)
                     } else {
                         warnRd(block, Rdfile, "missing file link ",
@@ -386,12 +461,26 @@ Rd2HTML <-
                     }
                 }
             }
-            if (parts$pkg == package) {
-                ## use href = "file.html"
+            if (parts$pkg == package) { # within same package
+                if (linksToTopics)
+                    htmlfile <-
+                        if (dynamic) paste0("../help/", urlify(parts$targetfile))
+                        else paste0("../help/", topic2filename(parts$targetfile), ".html")
+                else # use href = "file.html"
+                    htmlfile <- paste0(urlify(parts$targetfile), ".html")
                 writeHref()
-            } else {
+            } else {  # link to different package
                 ## href = "../../pkg/html/file.html"
-                htmlfile <- paste0("../../", urlify(parts$pkg), "/html/", htmlfile)
+                if (linksToTopics)
+                    htmlfile <-
+                        if (dynamic) paste0("../../", urlify(parts$pkg), "/help/",
+                                            urlify(parts$targetfile))
+                        else paste0("../../", urlify(parts$pkg), "/help/",
+                                    topic2filename(parts$targetfile), ".html")
+
+                else
+                    htmlfile <- paste0("../../", urlify(parts$pkg), "/html/",
+                                       urlify(parts$targetfile), ".html")
                 writeHref()
             }
         }
@@ -448,29 +537,25 @@ Rd2HTML <-
                "\\link" = writeLink(tag, block, doParas),
                ## cwhmisc has an empty \\email
                "\\email" = if (length(block)) {
-                   url <- paste(as.character(block), collapse="")
-                   url <- gsub("\n", "", url)
+                   url <- lines2str(as.character(block))
                    enterPara(doParas)
                    of0('<a href="mailto:', urlify(url), '">',
                        htmlify(url), '</a>')},
                ## watch out for empty URLs (TeachingDemos had one)
                "\\url" = if(length(block)) {
-                   url <- paste(as.character(block), collapse = "")
-                   url <- trimws(gsub("\n", "", url,
-                                      fixed = TRUE, useBytes = TRUE))
+                   url <- lines2str(as.character(block))
                    enterPara(doParas)
-                   of0('<a href="', urlify(url), '">',
-                       htmlify(url), '</a>')
+                   of0('<a href="', urlify(url), '">', htmlify(url), '</a>')
                },
                "\\href" = {
-               	   if(length(block[[1L]])) {
-               	   	url <- paste(as.character(block[[1L]]), collapse = "")
-               	   	url <- trimws(gsub("\n", "", url,
-                                           fixed = TRUE, useBytes = TRUE))
-		        enterPara(doParas)
-               	   	of0('<a href="', urlify(url), '">')
-               	   	closing <- "</a>"
-               	   } else closing <- ""
+                   closing <-
+                       if(length(block[[1L]])) {
+                           url <- lines2str(as.character(block[[1L]]))
+                           enterPara(doParas)
+                           of0('<a href="', urlify(url), '">')
+                           "</a>"
+                       }
+                       else ""
                	   savePara <- inPara
                	   inPara <<- NA
                	   writeContent(block[[2L]], tag)
@@ -627,10 +712,9 @@ Rd2HTML <-
 		    block <- sub(paste0(arg1, "[[:space:]]*,[[:space:]]*"),
 				 "", block)
             	    of0(arg1, pendingOpen)
-            	    if (pendingOpen == "$")
-            	    	pendingClose <<- ""
-            	    else
-            	    	pendingClose <<- chartr("[", "]", pendingOpen)
+                    pendingClose <<-
+                        if(pendingOpen == "$") ""
+                        else chartr("[", "]", pendingOpen)
             	} else of0("`", pendingOpen, "`")
             	pendingOpen <<- character()
             }
@@ -710,7 +794,8 @@ Rd2HTML <-
                         txt <- addParaBreaks(htmlify(block))
                         of1(txt)
                     } else writeBlock(block, tag, blocktag) # should not happen
-                } else writeBlock(block, tag, blocktag)
+                }
+                else writeBlock(block, tag, blocktag) # "typical default"
     	    })
 	}
 	if (inlist) {
@@ -718,10 +803,10 @@ Rd2HTML <-
 	    switch(blocktag,
 		"\\value"=,
 		"\\arguments" = of1("</table>\n"),
-		"\\itemize" = of1("</li></ul>\n"),
+		"\\itemize"   = of1("</li></ul>\n"),
 		"\\enumerate" = of1("</li></ol>\n"),
 		# "\\value"=,
-		"\\describe" = of1("</dl>\n"))
+		"\\describe"  = of1("</dl>\n"))
 	}
     }
 
@@ -761,11 +846,14 @@ Rd2HTML <-
     	sectionLevel <<- save
     }
 
+    ## ----------------------- Continue in main function -----------------------
+    create_redirects <- FALSE
     if (is.character(out)) {
         if (out == "") {
             con <- stdout()
         } else {
 	    con <- file(out, "wt")
+            create_redirects <- !dynamic && package_help
 	    on.exit(close(con))
 	}
     } else {
@@ -785,6 +873,7 @@ Rd2HTML <-
     	    for (i in seq_along(sections))
     	    	writeBlock(Rd[[i]], sections[i], "")
     } else {
+        if (create_redirects) createRedirects(out, Rd)
 	name <- htmlify(Rd[[2L]][[1L]])
 
         of0('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
@@ -798,11 +887,12 @@ Rd2HTML <-
 	    '<meta http-equiv="Content-Type" content="text/html; charset=',
 	    mime_canonical_encoding(outputEncoding),
 	    '" />\n')
+        of1('<meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=yes" />\n')
 
 	of0('<link rel="stylesheet" type="text/css" href="',
 	    urlify(stylesheet),
 	    '" />\n',
-	    '</head><body>\n\n',
+	    '</head><body><div class="container">\n\n',
 	    '<table width="100%" summary="page for ', htmlify(name))
 	if (nchar(package))
 	    of0(' {', package, '}"><tr><td>',name,' {', package,'}')
@@ -828,10 +918,10 @@ Rd2HTML <-
 		if (!no_links) '<a href="00Index.html">Index</a>',
 		']</div>')
 	of0('\n',
-	    '</body></html>\n')
+	    '</div></body></html>\n')
     }
     invisible(out)
-}
+} ## Rd2HTML()
 
 findHTMLlinks <- function(pkgDir = "", lib.loc = NULL, level = 0:2)
 {
