@@ -1,7 +1,7 @@
 #  File src/library/tools/R/RdConv2.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -254,7 +254,7 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 	    tmpcon <- file()
 	    sink(file = tmpcon)
 	    if(options$eval) err <- evalWithOpt(ce, options, env)
-	    res <- c(res, "\n") # attampt to  make sure final line is complete
+	    res <- c(res, "\n") # attempt to  make sure final line is complete
 	    sink()
 	    output <- readLines(tmpcon, warn = FALSE) # sometimes attempt fails.
 	    close(tmpcon)
@@ -266,8 +266,7 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 	    	stopRd(code, Rdfile, err$message)
 	    }
 
-	    if(length(output) & (options$results != "hide")){
-
+	    if(length(output) && (options$results != "hide")) {
 		output <- paste(output, collapse="\n")
 		if(options$strip.white %in% c("all", "true")) {
 		    output <- sub("^[[:space:]]*\n", "", output)
@@ -280,14 +279,14 @@ processRdChunk <- function(code, stage, options, env, Rdfile, macros)
 	    }
 	}
 	if (options$results == "rd") {
-	    res <- as.character(err)   # The last value of the chunk
+	    res <- enc2utf8(as.character(err))   # The last value of the chunk
 	    tmpcon <- file()
 	    writeLines(res, tmpcon, useBytes = TRUE)
 	    parseFragment <- function(cond) {
 	    	               seek(tmpcon, 0)
-	    	               parse_Rd(tmpcon, fragment=TRUE, macros = macros)
+	    	               parse_Rd(tmpcon, encoding="UTF-8", fragment=TRUE, macros = macros)
 	    	            }
-	    res <- tryCatch(parse_Rd(tmpcon, fragment=FALSE, macros = macros),
+	    res <- tryCatch(parse_Rd(tmpcon, encoding="UTF-8", fragment=FALSE, macros = macros),
 	    	            warning = parseFragment, error = parseFragment,
 	    	            finally = close(tmpcon))
 	    # Now remove that extra newline added by the writeLines
@@ -390,7 +389,7 @@ processRdSexprs <-
              env = new.env(hash = TRUE, parent = globalenv()), macros)
 {
     recurse <- function(block) {
-    	if (!any(getDynamicFlags(block)[stage])) return(block)
+    	if (!any(getDynamicFlags(block)[c("\\Sexpr",stage)])) return(block)
 
         if (is.list(block)) {
             if (!is.null(tag <- attr(block, "Rd_tag"))) {
@@ -408,7 +407,7 @@ processRdSexprs <-
 	block
     }
 
-    if (!any(getDynamicFlags(block)[stage])) return(block)
+    if (!any(getDynamicFlags(block)[c("\\Sexpr",stage)])) return(block)
     expandDynamicFlags(recurse(block), options)
 }
 
@@ -1023,7 +1022,7 @@ toRd <- function(obj, ...)
     UseMethod("toRd")
 
 toRd.default <- function(obj, ...) {
-    fsub <- function(from, to, x) gsub(from, to, x, fixed=TRUE)
+    fsub <- function(from, to, x) gsub(from, to, x, fixed=TRUE)# useBytes=FALSE  {really?}
     fsub("%", "\\%",
      fsub("}", "\\}",
       fsub("{", "\\{",
