@@ -35,8 +35,7 @@ function(dir, outDir, builtStamp=character())
     ok <- .check_package_description(file.path(dir, "DESCRIPTION"))
     if(any(as.integer(lengths(ok)) > 0L)) {
         stop(paste(gettext("Invalid DESCRIPTION file") ,
-                   paste(.eval_with_capture(print(ok))$output,
-                         collapse = "\n"),
+                   paste(format(ok), collapse = "\n\n"),
                    sep = "\n\n"),
              domain = NA,
              call. = FALSE)
@@ -478,7 +477,7 @@ function(dir, outDir)
 
     }
     if(dir.exists(dataDir))
-        saveRDS(.build_data_index(dataDir, contents),
+        saveRDS(.build_data_index(outDir, contents),
                  file.path(outDir, "Meta", "data.rds"))
     invisible()
 }
@@ -1133,8 +1132,7 @@ add_datalist <- function(pkgpath, force = FALSE, small.size = 1024^2)
     if (!force && file.exists(dlist)) return()
     size <- sum(file.size(Sys.glob(file.path(pkgpath, "data", "*"))))
     if(size <= small.size) return()
-    z <- suppressPackageStartupMessages(
-        list_data_in_pkg(dataDir = file.path(pkgpath, "data"))) # for BARD
+    z <- list_data_in_pkg(dir = pkgpath, use_datalist = FALSE)
     if(!length(z)) return()
     con <- file(dlist, "w")
     for (nm in names(z)) {
