@@ -27,6 +27,8 @@
 ##             or by file, /library/<pkg>/html/<file>.html
 httpd <- function(path, query, ...)
 {
+    linksToTopics <-
+        config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
     .HTMLdirListing <- function(dir, base, up) {
         files <- list.files(dir)    # note, no hidden files are listed
         out <- HTMLheader(paste0("Listing of directory<br/>", dir),
@@ -40,7 +42,7 @@ httpd <- function(path, query, ...)
                      paste0("<dd>", mono(iconv(urls, "", "UTF-8")), "</dd>"),
                      "</dl>")
         }
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
@@ -54,7 +56,7 @@ httpd <- function(path, query, ...)
          	out <- c(out, paste0('<h2>Manuals in package', sQuote(pkg),'</h2>'),
          		 makeVignetteTable(cbind(Package=pkg, vinfo[,c("File", "Title", "PDF", "R"), drop = FALSE])))
      	}
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
@@ -155,7 +157,7 @@ httpd <- function(path, query, ...)
                 }
 	    }
         }
-        out <- c(out, "<hr/>\n</body></html>")
+        out <- c(out, "<hr/>\n</div></body></html>")
         list(payload = paste(out, collapse="\n"))
     }
 
@@ -169,7 +171,7 @@ httpd <- function(path, query, ...)
                 "<tr><th style=\"text-align: left\">Concept</th><th>Frequency</th><th>Packages</th><tr>",
                 paste0("<tr><td>",
                        "<a href=\"/doc/html/Search?pattern=",
-                       vapply(reQuote(s), utils::URLencode, "", reserved = TRUE),
+                       utils::URLencode(reQuote(s), reserved = TRUE),
                        "&fields.concept=1&agrep=0\">",
                        shtmlify(substr(s, 1L, 80L)),
                        "</a>",
@@ -179,7 +181,7 @@ httpd <- function(path, query, ...)
                        concepts$Packages,
                        "</td></tr>"),
                 "</table>",
-                "</body>",
+                "</div></body>",
                 "</html>"))
         list(payload = paste(out, collapse = "\n"))
     }
@@ -205,7 +207,7 @@ httpd <- function(path, query, ...)
                        keywords$Packages,
                        "</td></tr>"),
                 "</table>",
-                "</body>",
+                "</div></body>",
                 "</html>"))
         list(payload = paste(out, collapse = "\n"))
     }
@@ -260,7 +262,7 @@ httpd <- function(path, query, ...)
 
     error_page <- function(msg)
         list(payload =
-             paste0(HTMLheader("httpd error"), msg, "\n</body></html>"))
+             paste(c(HTMLheader("httpd error"), msg, "\n</div></body></html>"), collapse = "\n"))
 
     cssRegexp <- "^/library/([^/]*)/html/R.css$"
     if (grepl("R\\.css$", path) && !grepl(cssRegexp, path))
@@ -294,7 +296,7 @@ httpd <- function(path, query, ...)
     ## ----------------------- per-package documentation ---------------------
     ## seems we got ../..//<pkg> in the past
     fileRegexp <- "^/library/+([^/]*)/html/([^/]*)\\.html$"
-    topicRegexp <- "^/library/+([^/]*)/help/([^/]*)$"
+    topicRegexp <- "^/library/+([^/]*)/help/(.*)$"
     docRegexp <- "^/library/([^/]*)/doc(.*)"
     demoRegexp <- "^/library/([^/]*)/demo$"
     demosRegexp <- "^/library/([^/]*)/demo/([^/]*)$"
@@ -310,8 +312,20 @@ httpd <- function(path, query, ...)
     	if (pkg == "NULL") pkg <- NULL  # There were multiple hits in the console
     	topic <- sub(topicRegexp, "\\2", path)
         ## if a package is specified, look there first, then everywhere
-    	if (!is.null(pkg)) # () avoids deparse here
+    	if (!is.null(pkg)) { # () avoids deparse here
     	    file <- utils::help(topic, package = (pkg), help_type = "text")
+            ## Before searching other packages, check if topic.Rd is
+            ## available as a file in the package.
+            if (!length(file) && linksToTopics) {
+                helppath <- system.file("help", package = pkg)
+                if (nzchar(helppath)) {
+                    contents <- readRDS(sub("/help$", "/Meta/Rd.rds", helppath, fixed = FALSE))
+                    helpfiles <- sub("\\.[Rr]d$", "", contents$File)
+                    if (topic %in% helpfiles) file <- file.path(helppath, topic)
+                }
+            }
+        }
+        ## Next, search for topic in all installed packages
     	if (!length(file))
             file <- utils::help(topic, help_type = "text", try.all.packages = TRUE)
 	if (!length(file)) {
@@ -394,7 +408,7 @@ httpd <- function(path, query, ...)
             return(error_page(msg))
         }
         ## if 'topic' is not a help doc, try it as an alias in the package
-        contents <- readRDS(sub("/help", "/Meta/Rd.rds", path, fixed = TRUE))
+        contents <- readRDS(sub("/help$", "/Meta/Rd.rds", path, fixed = FALSE))
         files <- sub("\\.[Rr]d$", "", contents$File)
         if(helpdoc %notin% files) {
             ## or call help()
