@@ -1,7 +1,7 @@
 #  File src/library/tools/R/packages.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,12 +20,13 @@ write_PACKAGES <-
 function(dir = ".", fields = NULL,
          type = c("source", "mac.binary", "win.binary"),
          verbose = FALSE, unpacked = FALSE, subdirs = FALSE,
-         latestOnly = TRUE, addFiles = FALSE, rds_compress = "xz")
+         latestOnly = TRUE, addFiles = FALSE, rds_compress = "xz",
+         validate = FALSE)
 {
     if(missing(type) && .Platform$OS.type == "windows")
         type <- "win.binary"
     type <- match.arg(type)
-
+ 
     paths <- ""
     if(is.logical(subdirs) && subdirs) {
         owd <- setwd(dir)
@@ -48,87 +49,133 @@ function(dir = ".", fields = NULL,
     for(path in paths) {
         this <- if(nzchar(path)) file.path(dir, path) else dir
         desc <- .build_repository_package_db(this, fields, type, verbose,
-                                             unpacked)
-        desc <- Filter(length, desc)
-
-        if(length(desc)) {
-            Files <- names(desc)
-            fields <- names(desc[[1L]])
-            desc <- matrix(unlist(desc), ncol = length(fields), byrow = TRUE)
-            colnames(desc) <- fields
-            if(addFiles) desc <- cbind(desc, File = Files)
-            if(addPaths) desc <- cbind(desc, Path = path)
-            if(latestOnly) desc <- .remove_stale_dups(desc)
-
-            ## Standardize licenses or replace by NA.
-            license_info <- analyze_licenses(desc[, "License"])
-            desc[, "License"] <-
-                ifelse(license_info$is_standardizable,
-                       license_info$standardization,
-                       NA)
-
+                                             unpacked, validate)
+        desc <- .process_repository_package_db_to_matrix(desc,
+                                                         path,
+                                                         addFiles,
+                                                         addPaths,
+                                                         latestOnly)
+        if(NROW(desc))
             db <- rbind(db, desc)
-        }
+    
     }
 
-    np <- NROW(db)
-    if(np > 0L) {
-        ## To save space, empty entries are not written to the DCF, so
-        ## that read.dcf() on these will have the entries as missing.
-        ## Hence, change empty to missing in the db.
-        db[!is.na(db) & (db == "")] <- NA_character_
-        con <- file(file.path(dir, "PACKAGES"), "wt")
-        write.dcf(db, con)
-        close(con)
-        con <- gzfile(file.path(dir, "PACKAGES.gz"), "wt")
-        write.dcf(db, con)
-        close(con)
-        rownames(db) <- db[, "Package"]
-        saveRDS(db, file.path(dir, "PACKAGES.rds"), compress = rds_compress)
-    }
+    np <- .write_repository_package_db(db, dir, rds_compress)
 
     invisible(np)
 }
 
+.write_repository_package_db <-
+function(db, dir, rds_compress)
+{
+   np <- NROW(db)
+   if(np > 0L) {
+       ## To save space, empty entries are not written to the DCF, so
+       ## that read.dcf() on these will have the entries as missing.
+       ## Hence, change empty to missing in the db.
+       db[!is.na(db) & (db == "")] <- NA_character_
+       con <- file(file.path(dir, "PACKAGES"), "wt")
+       write.dcf(db, con)
+       close(con)
+       con <- gzfile(file.path(dir, "PACKAGES.gz"), "wt")
+       write.dcf(db, con)
+       close(con)
+       rownames(db) <- db[, "Package"]
+       saveRDS(db, file.path(dir, "PACKAGES.rds"), compress = rds_compress)
+   }
+   
+   invisible(np)
+}
+
+.process_repository_package_db_to_matrix <-
+function(desc, path, addFiles, addPaths, latestOnly)
+{
+    desc <- Filter(length, desc)
+    
+    if(length(desc)) {
+        Files <- names(desc)
+        fields <- names(desc[[1L]])
+        desc <- matrix(unlist(desc), ncol = length(fields), byrow = TRUE)
+        colnames(desc) <- fields
+        if(addFiles) desc <- cbind(desc, File = Files)
+        if(addPaths) desc <- cbind(desc, Path = path)
+        if(latestOnly) desc <- .remove_stale_dups(desc)
+        
+        ## Standardize licenses or replace by NA.
+        license_info <- analyze_licenses(desc[, "License"])
+            desc[, "License"] <-
+                ifelse(license_info$is_standardizable,
+                       license_info$standardization,
+                       NA)
+        }
+    desc
+}
+
+## factored out so it can be used in multiple
+## places without threat of divergence
+.get_pkg_file_pattern = function(type = c("source", "mac.binary", "win.binary"),
+                                 ext.only = FALSE)
+{
+
+    type <- match.arg(type)
+    ## FIXME: might the source pattern be more general?
+    ## was .tar.gz prior to 2.10.0
+ 
+    ret = switch(type,
+                 "source" = "_.*\\.tar\\.[^_]*$",
+                 "mac.binary" = "_.*\\.tgz$",
+                 "win.binary" = "_.*\\.zip$")
+    if(ext.only)
+        ret = gsub("_.*", "", fixed = TRUE, ret)
+    ret
+}
 ## this is OK provided all the 'fields' are ASCII -- so be careful
 ## what you add.
 .build_repository_package_db <-
 function(dir, fields = NULL,
          type = c("source", "mac.binary", "win.binary"),
          verbose = getOption("verbose"),
-         unpacked = FALSE)
+         unpacked = FALSE, validate = FALSE)
 {
     if(unpacked)
         return(.build_repository_package_db_from_source_dirs(dir,
                                                              fields,
-                                                             verbose))
+                                                             verbose,
+                                                             validate))
 
-    type <- match.arg(type)
-
-    ## FIXME: might the source pattern be more general?
-    ## was .tar.gz prior to 2.10.0
-    package_pattern <- switch(type,
-                              "source" = "_.*\\.tar\\..*$",
-                              "mac.binary" = "_.*\\.tgz$",
-                              "win.binary" = "_.*\\.zip$")
-    files <- list.files(dir, pattern = package_pattern)
+       package_pattern <- .get_pkg_file_pattern(type)
+    files <- list.files(dir, pattern = package_pattern, full.names = TRUE)
 
     if(!length(files))
         return(list())
+    db <- .process_package_files_for_repository_db(files,
+                                                   type, 
+                                                   fields,
+                                                   verbose,
+                                                   validate)
+    db
+}
 
+.process_package_files_for_repository_db <-
+function(files, type, fields, verbose, validate = FALSE)
+{
+
+    files <- normalizePath(files, mustWork=TRUE) # files comes from list.files, mustWork ok
     ## Add the standard set of fields required to build a repository's
     ## PACKAGES file:
     fields <- unique(c(.get_standard_repository_db_fields(type), fields))
-    packages <- sapply(strsplit(files, "_", fixed = TRUE), "[", 1L)
+    ## files was without path at this point in original code,
+    ## use filetbs instead to compute pkg names and set db names 
+    filetbs <- basename(files)
+    packages <- sapply(strsplit(filetbs, "_", fixed = TRUE), "[", 1L)
     db <- vector(length(files), mode = "list")
-    names(db) <- files
+    names(db) <- filetbs #files was not full paths before
     ## Many (roughly length(files)) warnings are *expected*, hence
     ## suppressed.
     op <- options(warn = -1)
     on.exit(options(op))
     if(verbose) message("Processing packages:")
     if(type == "win.binary") {
-        files <- file.path(dir, files)
         for(i in seq_along(files)) {
             if(verbose) message(paste0("  ", files[i]))
             con <- unz(files[i], file.path(packages[i], "DESCRIPTION"))
@@ -142,8 +189,6 @@ function(dir, fields = NULL,
             close(con)
         }
     } else {
-        dir <- file_path_as_absolute(dir)
-        files <- file.path(dir, files)
         cwd <- getwd()
         if (is.null(cwd))
             stop("current working directory cannot be ascertained")
@@ -160,6 +205,21 @@ function(dir, fields = NULL,
                 temp <- tryCatch(read.dcf(p, fields = fields)[1L, ],
                                  error = identity)
                 if(!inherits(temp, "error")) {
+                    if(validate) {
+                        ## .check_package_description() by default goes via
+                        ## .read_description() which re-encodes and insists on a
+                        ## single entry unlike the above read.dcf() call.
+                        ok <- .check_package_description(db = temp[!is.na(temp)])
+                        ## FIXME: no format.check_package_description yet.
+                        if(any(as.integer(lengths(ok)) > 0L)) {
+                            message(paste(gettextf("Invalid DESCRIPTION file for package %s",
+                                                   sQuote(basename(dirname(p)))),
+                                          paste(format(ok), collapse = "\n\n"),
+                                          sep = "\n\n"),
+                                    domain = NA)
+                            next
+                        }
+                    }
                     if("NeedsCompilation" %in% fields &&
                        is.na(temp["NeedsCompilation"])) {
                         l <- utils::untar(files[i], list = TRUE)
@@ -185,7 +245,8 @@ function(dir, fields = NULL,
 }
 
 .build_repository_package_db_from_source_dirs <-
-function(dir, fields = NULL, verbose = getOption("verbose"))
+function(dir, fields = NULL, verbose = getOption("verbose"),
+         validate = FALSE)
 {
     dir <- file_path_as_absolute(dir)
     fields <- unique(c(.get_standard_repository_db_fields(), fields))
@@ -200,6 +261,22 @@ function(dir, fields = NULL, verbose = getOption("verbose"))
                                   fields = fields)[1L, ],
                          error = identity)
         if(!inherits(temp, "error")) {
+            if(validate) {
+                ## .check_package_description() by default goes via
+                ## .read_description() which re-encodes and insists on a
+                ## single entry unlike the above read.dcf() call.
+                ok <- .check_package_description(db = temp[!is.na(temp)])
+                ## FIXME: no format.check_package_description yet.
+                if(any(as.integer(lengths(ok)) > 0L)) {
+                    warning(paste(gettextf("Invalid DESCRIPTION file for package %s",
+                                           sQuote(basename(paths[i]))),
+                                  paste(format(ok), collapse = "\n\n"),
+                                  sep = "\n\n"),
+                            domain = NA,
+                            call. = FALSE)
+                    next
+                }
+            }
             if(is.na(temp["NeedsCompilation"])) {
                 temp["NeedsCompilation"] <-
                     if(dir.exists(file.path(paths[i], "src"))) "yes" else "no"
@@ -220,16 +297,11 @@ function(dir, fields = NULL, verbose = getOption("verbose"))
 }
 
 dependsOnPkgs <-
-function(pkgs, dependencies = c("Depends", "Imports", "LinkingTo"),
+function(pkgs, dependencies = "strong",
          recursive = TRUE, lib.loc = NULL,
          installed = utils::installed.packages(lib.loc, fields = "Enhances"))
 {
-    if(identical(dependencies, "all"))
-        dependencies <-
-            c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")
-    else if(identical(dependencies, "most"))
-        dependencies <-
-            c("Depends", "Imports", "LinkingTo", "Suggests")
+    dependencies <- .expand_dependency_type_spec(dependencies)
 
     av <- installed[, dependencies, drop = FALSE]
     rn <- as.character(installed[, "Package"])
@@ -275,9 +347,9 @@ function(ap)
 }
 
 package_dependencies <-
-function(packages = NULL, db = NULL,
-         which = c("Depends", "Imports", "LinkingTo"),
-         recursive = FALSE, reverse = FALSE, verbose = getOption("verbose"))
+function(packages = NULL, db = NULL, which = "strong",
+         recursive = FALSE, reverse = FALSE,
+         verbose = getOption("verbose"))
 {
     ## <FIXME>
     ## What about duplicated entries?
@@ -285,15 +357,23 @@ function(packages = NULL, db = NULL,
 
     if(is.null(db)) db <- utils::available.packages()
 
+    fields <- which <- .expand_dependency_type_spec(which)
+    if(is.character(recursive)) {
+        recursive <- .expand_dependency_type_spec(recursive)
+        if(identical(which, recursive))
+            recursive <- TRUE
+        else
+            fields <- unique(c(fields, recursive))
+    }
+
     ## For given packages which are not found in the db, return "list
     ## NAs" (i.e., NULL entries), as opposed to character() entries
     ## which indicate no dependencies.
+    out_of_db_packages <- character()
 
     ## For forward non-recursive depends, we can simplify matters by
     ## subscripting the db right away---modulo boundary cases.
-
-    out_of_db_packages <- character()
-    if(!recursive && !reverse) {
+    if(!is.character(recursive) && !recursive && !reverse) {
         if(!is.null(packages)) {
             ind <- match(packages, db[, "Package"], nomatch = 0L)
             db <- db[ind, , drop = FALSE]
@@ -301,31 +381,39 @@ function(packages = NULL, db = NULL,
         }
     }
 
-    if(identical(which, "all"))
-        which <-
-            c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")
-    else if(identical(which, "most"))
-        which <-
-            c("Depends", "Imports", "LinkingTo", "Suggests")
+    db <- as.data.frame(db[, c("Package", fields), drop = FALSE])
+    ## Avoid recomputing package dependency names in recursive
+    ## invocations.
+    for(f in fields) {
+        if(!is.list(d <- db[[f]]))
+            db[[f]] <- lapply(d, .extract_dependency_package_names)
+    }
+
+    if(is.character(recursive)) {
+        ## Direct dependencies:
+        d_d <- Recall(packages, db, which, FALSE,
+                      reverse, verbose)
+        ## Recursive dependencies of all these:
+        d_r <- Recall(unique(unlist(d_d)), db, recursive, TRUE,
+                      reverse, verbose)
+        ## Now glue together:
+        return(lapply(d_d,
+                      function(p) {
+                          sort(unique(c(p, unlist(d_r[p],
+                                                  use.names = FALSE))))
+                      }))
+    }
 
     depends <-
         do.call(Map,
                 c(list("c"),
-                  ## Try to make this work for dbs which are character
-                  ## matrices as from available.packages(), or data
-                  ## frame variants thereof.
-                  lapply(which,
-                         function(f) {
-                             if(is.list(d <- db[, f])) d
-                             else lapply(d,
-                                         .extract_dependency_package_names)
-                         }),
+                  db[which],
                   list(USE.NAMES = FALSE)))
 
     depends <- lapply(depends, unique)
 
     if(!recursive && !reverse) {
-        names(depends) <- db[, "Package"]
+        names(depends) <- db$Package
         if(length(out_of_db_packages)) {
             depends <-
                 c(depends,
@@ -335,12 +423,12 @@ function(packages = NULL, db = NULL,
         return(depends)
     }
 
-    all_packages <- sort(unique(c(db[, "Package"], unlist(depends))))
+    all_packages <- sort(unique(c(db$Package, unlist(depends))))
 
     if(!recursive) {
         ## Need to invert.
         depends <-
-            split(rep.int(db[, "Package"], lengths(depends)),
+            split(rep.int(db$Package, lengths(depends)),
                   factor(unlist(depends), levels = all_packages))
         if(!is.null(packages)) {
             depends <- depends[match(packages, names(depends))]
@@ -362,7 +450,7 @@ function(packages = NULL, db = NULL,
     ## with i R j and j R k, respectively, and combine these.
     ## This works reasonably well, but of course more efficient
     ## implementations should be possible.
-    matchP <- match(rep.int(db[, "Package"], lengths(depends)),
+    matchP <- match(rep.int(db$Package, lengths(depends)),
 		    all_packages)
     matchD <- match(unlist(depends), all_packages)
     tab <- if(reverse)
@@ -376,7 +464,7 @@ function(packages = NULL, db = NULL,
             packages <- all_packages
             p_L <- seq_along(all_packages)
         } else {
-            packages <- db[, "Package"]
+            packages <- db$Package
             p_L <- match(packages, all_packages)
         }
     } else {
@@ -417,12 +505,33 @@ function(packages = NULL, db = NULL,
     depends
 }
 
+.expand_dependency_type_spec <-
+function(x)
+{
+    if(identical(x, "strong"))
+        c("Depends", "Imports", "LinkingTo")
+    else if(identical(x, "most"))
+        c("Depends", "Imports", "LinkingTo", "Suggests")
+    else if(identical(x, "all"))
+        c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")
+    else
+        x
+    ## (Could also intersect x with the possible types.)
+}
+
+## .extract_dependency_package_names <-
+## function(x)
+## {
+##     ## Assume a character *string*.
+##     if(is.na(x)) return(character())
+##     x <- strsplit(x, ",", fixed = TRUE)[[1L]]
+##     ## FIXME: The following is much faster on Linux but apparently not
+##     ## on Windows:
+##     ## x <- sub("(?s)[[:space:]]*([[:alnum:].]+).*", "\\1", x, perl = TRUE)
+##     x <- sub("[[:space:]]*([[:alnum:].]+).*", "\\1", x)
+##     x[nzchar(x) & (x != "R")]
+## }
 
 .extract_dependency_package_names <-
-function(x) {
-    ## Assume a character *string*.
-    if(is.na(x)) return(character())
-    x <- unlist(strsplit(x, ",[[:space:]]*"))
-    x <- sub("[[:space:]]*([[:alnum:].]+).*", "\\1", x)
-    x[nzchar(x) & (x != "R")]
-}
+function(x)
+    .Call(C_package_dependencies_scan, x)
