@@ -1,7 +1,7 @@
 #  File src/library/utils/R/aspell.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2019 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -268,7 +268,7 @@ function(x, sort = TRUE, verbose = FALSE, indent = 2L, ...)
     } else {
         sep <- sprintf("\n%s", strrep(" ", indent))
         paste(names(from),
-              sapply(from, paste, collapse = sep),
+              vapply(from, paste, "", collapse = sep),
               sep = sep)
     }
 }
@@ -383,8 +383,7 @@ function(x, ...)
         writeLines(c(sprintf("File '%s':", nms[i]),
                      sprintf("  Line %s: \"%s\", \"%s\", \"%s\"",
                              format(e$Line),
-                             gsub("\"", "\\\"", e$Left),
-                             e$Original,
+                             gsub("\"", "\\\"", e$Left ), e$Original,
                              gsub("\"", "\\\"", e$Right)),
                      ""))
     }
@@ -535,7 +534,7 @@ function(dir, drop = c("\\author", "\\references"),
     }
 
     macros <- tools::loadPkgRdMacros(dir,
-                                     macros = file.path(R.home("share"), 
+                                     macros = file.path(R.home("share"),
                                                         "Rd", "macros",
                                                         "system.Rd"))
 
@@ -726,7 +725,7 @@ function(ifile, encoding = "unknown", ignore = character())
                 col1 <- which(ptab == col1) + 1L
             }
             substring(lines[line1], col1) <- texts[1L]
-            pos <- seq(from = 2L, length.out = n - 2L)
+            pos <- seq.int(from = 2L, length.out = n - 2L)
             if(length(pos))
                 lines[line1 + pos - 1] <- texts[pos]
             if(length(ptab <- tab[[as.character(line2)]])) {
@@ -801,7 +800,7 @@ function(file, encoding = "unknown")
         ##   ... used in a situation where it doesn't exist
         ## so eliminate these.
         ## (Note that we also drop "..." strings.)
-        call <- parse(text = call)[[1L]]
+        call <- str2lang(call)
         call <- call[ as.character(call) != "..." ]
         mc <- as.list(match.call(get(fun, envir = .BaseNamespaceEnv),
                                  call))
@@ -817,7 +816,7 @@ function(file, encoding = "unknown")
         strings <- as.character(args[vapply(args, is.character, TRUE)])
         ## Need to canonicalize to match string constants before and
         ## after parsing ...
-        texts <- vapply(parse(text = table$text), as.character, "")
+        texts <- vapply(str2expression(table$text), as.character, "")
         pos <- which(!is.na(match(texts, strings)))
         cbind(table[pos, ], caller = rep.int(fun, length(pos)))
     }
@@ -1089,7 +1088,7 @@ function(dir, ignore = character(),
                 aspell_find_dictionaries(d, file.path(dir, ".aspell"))
         }
     }
-    
+
     program <- aspell_find_program(program)
 
     aspell(files,
@@ -1130,10 +1129,11 @@ function(ifile, encoding = "UTF-8")
         ## Legibility ...
         l1 <- p[1L]; c1 <- p[2L]; l2 <- p[3L]; c2 <- p[4L]
         if(l1 < l2) {
-            w <- seq(l1 + 1L, l2 - 1L)
-            if(length(w))
-                y[w] <- x[w]
             substring(y[l1], c1, n[l1]) <- substring(x[l1], c1, n[l1])
+            if(l1 + 1L < l2) {
+                w <- seq.int(from = l1 + 1L, to = l2 - 1L)
+                y[w] <- x[w]
+            }
             substring(y[l2], 1L, c2) <- substring(x[l2], 1L, c2)
         } else {
             substring(y[l1], c1, c2) <- substring(x[l1], c1, c2)
@@ -1141,7 +1141,7 @@ function(ifile, encoding = "UTF-8")
     }
     y
 }
-    
+
 ## For spell checking packages.
 
 aspell_package <-
