@@ -1,7 +1,7 @@
 #  File src/library/utils/R/packages.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,11 @@ available.packages <-
 function(contriburl = contrib.url(repos, type), method,
          fields = NULL, type = getOption("pkgType"),
          filters = NULL, repos = getOption("repos"),
-         ignore_repo_cache =  FALSE, max_repo_cache_age,
-         ...)
+         ignore_repo_cache = FALSE, max_repo_cache_age,
+         quiet = TRUE, ...)
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     requiredFields <-
         c(tools:::.get_standard_repository_db_fields(), "File")
     if (is.null(fields))
@@ -71,7 +73,7 @@ function(contriburl = contrib.url(repos, type), method,
                     if(isTRUE(age < max_repo_cache_age)) {
                         res0 <- readRDS(dest)
                         used_dest <- TRUE
-                        ## Be defensive ...
+                        ## Be defensive :
                         if(length(res0))
                             rownames(res0) <- res0[, "Package"]
                     }
@@ -88,13 +90,13 @@ function(contriburl = contrib.url(repos, type), method,
                 z <- tryCatch({
                     download.file(url = paste0(repos, "/PACKAGES.rds"),
                                   destfile = dest, method = method,
-                                  cacheOK = FALSE, quiet = TRUE, mode = "wb")
+                                  cacheOK = FALSE, quiet = quiet, mode = "wb", ...)
                 }, error = identity)
                 options(op)
                 if(!inherits(z, "error")) {
                     z <- res0 <- tryCatch(readRDS(dest), error = identity)
-                     if(ignore_repo_cache) unlink(dest)
-               }
+                    if(ignore_repo_cache) unlink(dest)
+                }
 
                 if(inherits(z, "error")) {
                     ## Downloading or reading .rds failed, so try the
@@ -108,7 +110,7 @@ function(contriburl = contrib.url(repos, type), method,
                         ## This is a binary file
                         download.file(url = paste0(repos, "/PACKAGES.gz"),
                                       destfile = tmpf, method = method,
-                                      cacheOK = FALSE, quiet = TRUE, mode = "wb")
+                                      cacheOK = FALSE, quiet = quiet, mode = "wb", ...)
                     }, error = identity)
                     if(inherits(z, "error"))
                         z <- tryCatch({
@@ -116,7 +118,7 @@ function(contriburl = contrib.url(repos, type), method,
                             ## LF, so use binary mode to avoid CRLF.
                             download.file(url = paste0(repos, "/PACKAGES"),
                                           destfile = tmpf, method = method,
-                                          cacheOK = FALSE, quiet = TRUE, mode = "wb")
+                                          cacheOK = FALSE, quiet = quiet, mode = "wb", ...)
                         }, error = identity)
                     options(op)
 
@@ -133,6 +135,9 @@ function(contriburl = contrib.url(repos, type), method,
                                      repos),
                             ":\n  ", conditionMessage(z),
                             call. = FALSE, immediate. = TRUE, domain = NA)
+                    ## Do not cache incorrect results. It can be a page
+                    ## from a web proxy about inaccessible network.
+                    unlink(dest)
                     next
                 }
 
@@ -355,6 +360,8 @@ update.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                             available = NULL, oldPkgs = NULL, ...,
                             checkBuilt = FALSE, type = getOption("pkgType"))
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     force(ask)  # just a check that it is valid before we start work
     text.select <- function(old)
     {
@@ -462,6 +469,8 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                          method, available = NULL, checkBuilt = FALSE,
                          ..., type = getOption("pkgType"))
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     if(is.null(lib.loc))
         lib.loc <- .libPaths()
     if(!missing(instPkgs)) {
@@ -491,12 +500,8 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
         deps <- onRepos["Depends"]
         if(!is.na(deps)) {
             Rdeps <- tools:::.split_dependencies(deps)[["R", exact=TRUE]]
-            if(length(Rdeps) > 1L) {
-                target <- Rdeps$version
-                res <- do.call(Rdeps$op, list(currentR, target))
- ##               res <- eval(parse(text=paste("currentR", Rdeps$op, "target")))
-                if(!res) next
-            }
+            if(length(Rdeps) > 1L && !do.call(Rdeps$op, list(currentR, Rdeps$version)))
+                next
         }
         update <- rbind(update,
                         c(instPkgs[k, c("Package", "LibPath", "Version", "Built")],
@@ -516,6 +521,8 @@ new.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                          method, available = NULL, ask = FALSE,
                          ..., type = getOption("pkgType"))
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     ask  # just a check that it is valid before we start work
     if(type == "both" && (!missing(contriburl) || !is.null(available))) {
         stop("specifying 'contriburl' or 'available' requires a single type, not type = \"both\"")
@@ -725,6 +732,8 @@ download.packages <- function(pkgs, destdir, available = NULL,
                               contriburl = contrib.url(repos, type),
                               method, type = getOption("pkgType"), ...)
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     nonlocalcran <- !all(startsWith(contriburl, "file:"))
     if(nonlocalcran && !dir.exists(destdir))
         stop("'destdir' is not a directory")
@@ -757,7 +766,8 @@ download.packages <- function(pkgs, destdir, available = NULL,
                          switch(type,
                                 "source" = ".tar.gz",
                                 "mac.binary" = ".tgz",
-                                "win.binary" = ".zip"))
+                                "win.binary" = ".zip",
+                                stop("invalid 'type'")))
             have_fn <- !is.na(File)
             fn[have_fn] <- File[have_fn]
             repos <- available[ok, "Repository"]
@@ -808,6 +818,8 @@ resolvePkgType <- function(type) {
 
 contrib.url <- function(repos, type = getOption("pkgType"))
 {
+    if (!is.character(type))
+        stop("invalid 'type'; must be a character string")
     type <- resolvePkgType(type)
     if(is.null(repos)) return(NULL)
     if("@CRAN@" %in% repos && interactive()) {
@@ -834,7 +846,8 @@ contrib.url <- function(repos, type = getOption("pkgType"))
     res <- switch(type,
 		"source" = paste(gsub("/$", "", repos), "src", "contrib", sep = "/"),
                 "mac.binary" = paste(gsub("/$", "", repos), "bin", mac.path, "contrib", ver, sep = "/"),
-                "win.binary" = paste(gsub("/$", "", repos), "bin", "windows", "contrib", ver, sep = "/")
+                "win.binary" = paste(gsub("/$", "", repos), "bin", "windows", "contrib", ver, sep = "/"),
+                stop("invalid 'type'")
                )
     res
 }
@@ -933,6 +946,8 @@ setRepositories <-
         stop("cannot set repositories non-interactively")
     a <- tools:::.get_repositories()
     pkgType <- getOption("pkgType")
+    if (!is.character(pkgType))
+        stop("invalid options(\"pkgType\"); must be a character string")
     if (pkgType == "both") pkgType <- "source" #.Platform$pkgType
     if (pkgType == "binary") pkgType <- .Platform$pkgType
     if(startsWith(pkgType, "mac.binary")) pkgType <- "mac.binary"
@@ -987,15 +1002,19 @@ compareVersion <- function(a, b)
 }
 
 ## ------------- private functions --------------------
-.clean_up_dependencies <- function(x, available = NULL)
-{
-    ## x is a character vector of Depends / Suggests / Imports entries
-    ## returns a character vector of all the package dependencies mentioned
-    x <- x[!is.na(x)]
-    if(!length(x)) return(x)
-    x <- unlist(strsplit(x, ","))
-    unique(sub("^[[:space:]]*([[:alnum:].]+).*$", "\\1" , x))
-}
+
+## .clean_up_dependencies <- function(x, available = NULL)
+## {
+##     ## x is a character vector of Depends / Suggests / Imports entries
+##     ## returns a character vector of all the package dependencies mentioned
+##     x <- x[!is.na(x)]
+##     if(!length(x)) return(x)
+##     x <- unlist(strsplit(x, ",", fixed = TRUE), use.names = FALSE)
+##     unique(sub("^[[:space:]]*([[:alnum:].]+).*$", "\\1" , x))
+## }
+
+.clean_up_dependencies <- function(x)
+    unique.default(tools:::.extract_dependency_package_names(x))
 
 .clean_up_dependencies2 <- function(x, installed, available)
 {
@@ -1040,7 +1059,6 @@ compareVersion <- function(a, b)
             current <- as.package_version(installed[pkgs == x[[1L]], "Version"])
             target <- as.package_version(x[[3L]])
             any(do.call(x$op, list(current, target)))
-##            eval(parse(text = paste("any(current", x$op, "target)")))
         } else x[[1L]] %in% pkgs
     })
     xx <- xx[!have]
@@ -1058,7 +1076,6 @@ compareVersion <- function(a, b)
             current <- as.package_version(available[pkgs == x[[1L]], "Version"])
             target <- as.package_version(x[[3L]])
             res <- any(do.call(x$op, list(current, target)))
-##            res <- eval(parse(text = paste("any(current", x$op, "target)")))
             if(res) canget <- c(canget, x[[1L]])
             else  miss <- c(miss, paste0(x[[1L]], " (>= ", x[[3L]], ")"))
         } else if(x[[1L]] %in% pkgs) canget <- c(canget, x[[1L]])
