@@ -1,7 +1,7 @@
 #  File src/library/utils/R/windows/download.file.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2020 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -28,15 +28,15 @@ download.file <-
         match.arg(method, c("auto", "internal", "wininet", "libcurl",
                             "wget", "curl", "lynx"))
 
-    if(missing(mode) && length(grep("\\.(gz|bz2|xz|tgz|zip|rd[as]|RData)$",
+    if(missing(mode) && length(grep("\\.(gz|bz2|xz|tgz|zip|jar|rd[as]|RData)$",
 				    URLdecode(url))))
         mode <- "wb"
     if(method == "auto") {
         if(length(url) != 1L || typeof(url) != "character")
             stop("'url' must be a length-one character vector");
 	method <-
-            if(startsWith(url, "ftps:") && capabilities("libcurl")) "libcurl"
-            else "wininet"
+            if((startsWith(url, "ftp:") || startsWith(url, "ftps:"))
+               && capabilities("libcurl")) "libcurl" else "wininet"
     }
 
     nh <- names(headers)
