@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1998--2020 The R Core Team.
- *  Copyright (C) 2003--2019 The R Foundation
+ *  Copyright (C) 2003--2020 The R Foundation
  *  Copyright (C) 1995--1997 Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -122,24 +122,22 @@ static double R_ValueOfNA(void)
     return x.value;
 }
 
+/* is a value known to be a NaN also an R NA? */
+int attribute_hidden R_NaN_is_R_NA(double x)
+{
+    ieee_double y;
+    y.value = x;
+    return (y.word[lw] == 1954);
+}
+
 int R_IsNA(double x)
 {
-    if (isnan(x)) {
-	ieee_double y;
-	y.value = x;
-	return (y.word[lw] == 1954);
-    }
-    return 0;
+    return isnan(x) && R_NaN_is_R_NA(x);
 }
 
 int R_IsNaN(double x)
 {
-    if (isnan(x)) {
-	ieee_double y;
-	y.value = x;
-	return (y.word[lw] != 1954);
-    }
-    return 0;
+    return isnan(x) && ! R_NaN_is_R_NA(x);
 }
 
 /* ISNAN uses isnan, which is undefined by C++ headers
@@ -183,22 +181,23 @@ void attribute_hidden InitArithmetic()
 
 
 #if HAVE_LONG_DOUBLE && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
+/*
 # ifdef __powerpc__
  // PowerPC 64 (when gcc has -mlong-double-128) fails constant folding with LDOUBLE
  // Debian Bug#946836 shows it is needed also for 32-bit ppc, not just __PPC64__
-#  define q_1_eps (1 / LDBL_EPSILON)
-# else
-static LDOUBLE q_1_eps = 1 / LDBL_EPSILON;
-# endif
+ // NB: 1 / LDBL_EPSILON has been seen to overflow on 'ppc64el ...
+ ==> use  eps instead of  1 / eps  (and one multiplication more)
+*/
+# define c_eps LDBL_EPSILON
 #else
-static double  q_1_eps = 1 / DBL_EPSILON;
+# define c_eps DBL_EPSILON
 #endif
 
 /* Keep myfmod() and myfloor() in step */
 static double myfmod(double x1, double x2)
 {
     if (x2 == 0.0) return R_NaN;
-    if(fabs(x2) > q_1_eps && R_FINITE(x1) && fabs(x1) <= fabs(x2)) {
+    if(fabs(x2) * c_eps > 1 && R_FINITE(x1) && fabs(x1) <= fabs(x2)) {
 	return
 	    (fabs(x1) == fabs(x2)) ? 0 :
 	    ((x1 < 0 && x2 > 0) ||
@@ -207,7 +206,7 @@ static double myfmod(double x1, double x2)
 	     : x1   ; // "same" signs (incl. 0)
     }
     double q = x1 / x2;
-    if(R_FINITE(q) && (fabs(q) > q_1_eps))
+    if(R_FINITE(q) && (fabs(q) * c_eps > 1))
 	warning(_("probable complete loss of accuracy in modulus"));
     LDOUBLE tmp = (LDOUBLE)x1 - floor(q) * (LDOUBLE)x2;
     return (double) (tmp - floorl(tmp/x2) * x2);
@@ -216,7 +215,7 @@ static double myfmod(double x1, double x2)
 static double myfloor(double x1, double x2)
 {
     double q = x1 / x2;
-    if (x2 == 0.0 || fabs(q) > q_1_eps || !R_FINITE(q))
+    if (x2 == 0.0 || fabs(q) * c_eps > 1 || !R_FINITE(q))
 	return q;
     if(fabs(q) < 1)
 	return (q < 0) ? -1
@@ -1640,11 +1639,8 @@ SEXP attribute_hidden do_Math2(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (length(args) >= 2 &&
 	isSymbol(CADR(args)) && R_isMissing(CADR(args), env)) {
-#define SET_DEFAULT_digits_			\
-	double digits = 0.; /* round() */	\
-	if(PRIMVAL(op) == 10004) /* signif() */	\
-	    digits = 6.
-	SET_DEFAULT_digits_;
+	double digits = 0;
+	if(PRIMVAL(op) == 10004) digits = 6.0; // for signif()
 	PROTECT(args = list2(CAR(args), ScalarReal(digits))); nprotect++;
     }
 
@@ -1667,7 +1663,9 @@ SEXP attribute_hidden do_Math2(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if(CAR(args) == R_MissingArg ||
 	       (TAG(args) != R_NilValue && TAG(args) != R_x_Symbol))
 		error(_("argument \"%s\" is missing, with no default"), "x");
-	    SET_DEFAULT_digits_;
+	    double digits = 0.0; // round()
+	    if(PRIMVAL(op) == 10004) // signif()
+		digits = 6.0;
 	    SETCDR(args, CONS(ScalarReal(digits), R_NilValue));
 	} else {
 	    /* If named, do argument matching by name */
@@ -2276,7 +2274,7 @@ static SEXP math5(SEXP sa, SEXP sb, SEXP sc, SEXP sd, SEXP se, double (*f)())
 #define FINISH_Math5					\
     if(naflag) warning(R_MSG_NA);			\
 							\
-    if (n == na) SHALLOW_DUPLICATE_ATTRIB(sy, sa);	\
+    if      (n == na) SHALLOW_DUPLICATE_ATTRIB(sy, sa);	\
     else if (n == nb) SHALLOW_DUPLICATE_ATTRIB(sy, sb);	\
     else if (n == nc) SHALLOW_DUPLICATE_ATTRIB(sy, sc);	\
     else if (n == nd) SHALLOW_DUPLICATE_ATTRIB(sy, sd);	\
