@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2020  The R Core Team
+ *  Copyright (C) 1997--2021  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@ here prior to 2.10.0 are now in grep.c and agrep.c
 make.unique, duplicated, unique, match, pmatch, charmatch are in unique.c
 iconv is in sysutils.c
 
-Character strings in R are less than 2^31-1 bytes, so we use int not size_t.
+Character strings in R are at most 2^31-1 bytes, so we use int not size_t.
 
 Support for UTF-8-encoded strings in non-UTF-8 locales
 ======================================================
@@ -44,11 +44,13 @@ Comparison is done directly unless you happen to be comparing the same
 string in different encodings.
 
 nzchar and nchar(, "bytes") are independent of the encoding
-nchar(, "char") nchar(, "width") handle UTF-8 directly, translate Latin-1
+nchar(, "char") nchar(, "width") handle UTF-8 and Latin-1 directly 
 substr substr<-  handle UTF-8 and Latin-1 directly
-tolower toupper chartr  translate UTF-8 to wchar, rest to current charset
-  which needs Unicode wide characters
-abbreviate strtrim  translate
+tolower toupper chartr  translate UTF-8 and Latin-1 to wchar (which needs 
+  Unicode wide characters), rest to current charset
+abbreviate translates non-ASCII inputs to UTF-8 then wchar_t*.
+strtrim translates to the native encoding
+make.names translates to the native encoding, works in wchar_t in a MBCS.
 
 All the string matching functions handle UTF-8 directly, otherwise
 translate (latin1 to UTF-8, otherwise to native).
@@ -69,12 +71,23 @@ abbreviate chartr make.names strtrim tolower toupper give error.
 # include <config.h>
 #endif
 
+/* Used to indicate that we can safely converted marked UTF-8 strings
+   to wchar_t* -- not currently used.
+*/
+#if defined(Win32) || defined(__STDC_ISO_10646__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__sun)
+# define TO_WCS_OK 1
+#else
+/*
+  Maybe warn if utf8towcs is used, but no known platforms.
+ */
+#endif
+
 #include <Defn.h>
 #include <Internal.h>
 #include <errno.h>
-#include <R_ext/RS.h>  /* for Calloc/Free */
+#include <R_ext/RS.h>  // for Calloc/Free
 #include <R_ext/Itermacros.h>
-#include <rlocale.h>
+#include <rlocale.h>   // overrides iswxxxx on some platforms.
 
 /* We use a shared buffer here to avoid reallocing small buffers, and
    keep a standard-size (MAXELTSIZE = 8192) buffer allocated shared
@@ -91,13 +104,14 @@ static R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
 /* Functions to perform analogues of the standard C string library. */
 /* Most are vectorized */
 
-/* primitive */
+/* primitive, nzchar(x, keepNA = FALSE) where the second argument is optional.
+   Encoding of x is immaterial.
+*/
 SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP x, ans;
     int nargs = length(args);
 
-    // checkArity(op, args);  .Primitive()  &  may have 1 or 2 args now
+    // checkArity(op, args);  .Primitive() and may have 1 or 2 args
     if (nargs < 1 || nargs > 2)
 	errorcall(call,
 		  ngettext("%d argument passed to '%s' which requires %d to %d",
@@ -108,7 +122,7 @@ SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (isFactor(CAR(args)))
 	error(_("'%s' requires a character vector"), "nzchar()");
-    PROTECT(x = coerceVector(CAR(args), STRSXP));
+    SEXP x = PROTECT(coerceVector(CAR(args), STRSXP));
     if (!isString(x))
 	error(_("'%s' requires a character vector"), "nzchar()");
 
@@ -118,7 +132,7 @@ SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
 	if (keepNA == NA_LOGICAL) keepNA = FALSE;
     }
     R_xlen_t i, len = XLENGTH(x);
-    PROTECT(ans = allocVector(LGLSXP, len));
+    SEXP ans = PROTECT(allocVector(LGLSXP, len));
     if (keepNA)
 	for (i = 0; i < len; i++) {
 	    SEXP sxi = STRING_ELT(x, i);
@@ -132,6 +146,17 @@ SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 /* R strings are limited to 2^31 - 1 bytes on all platforms */
+
+/* when msg_name is not NULL,
+     error handling is via error() with a message including msg_name
+     semi-internal buffer cbuff is freed if over default size
+
+   when msg_name is NULL, (for use where performance matters)
+     error handling is via negative return value (other than NA_INTEGER):
+       -1 ... invalid multi-byte string
+       -2 ... the quantity is not computable (bytes encoding)
+     semi-internal buffer cbuff is never freed, should be freed by caller
+*/
 int R_nchar(SEXP string, nchar_type type_,
 	    Rboolean allowNA, Rboolean keepNA, const char* msg_name)
 {
@@ -146,23 +171,38 @@ int R_nchar(SEXP string, nchar_type type_,
 	if (IS_UTF8(string)) {
 	    const char *p = CHAR(string);
 	    if (!utf8Valid(p)) {
-		if (!allowNA)
-		    error(_("invalid multibyte string, %s"), msg_name);
+		if (!allowNA) {
+		    if (msg_name)
+			error(_("invalid multibyte string, %s"), msg_name);
+		    else
+			return -1;
+		}
 		return NA_INTEGER;
 	    } else {
 		int nc = 0;
 		for( ; *p; p += utf8clen(*p)) nc++;
 		return nc;
 	    }
+	} else if (IS_LATIN1(string)) {
+	    // just count bytes
+	    return (int) strlen(CHAR(string));
 	} else if (IS_BYTES(string)) {
-	    if (!allowNA) /* could do chars 0 */
-		error(_("number of characters is not computable in \"bytes\" encoding, %s"),
-		      msg_name);
+	    if (!allowNA) /* could do chars 0 */ {
+		if (msg_name)
+		    error(_("number of characters is not computable in \"bytes\" encoding, %s"),
+		          msg_name);
+		else
+		    return -2;
+	    }
 	    return NA_INTEGER;
 	} else if (mbcslocale) {
 	    int nc = (int) mbstowcs(NULL, translateChar(string), 0);
-	    if (!allowNA && nc < 0)
-		error(_("invalid multibyte string, %s"), msg_name);
+	    if (!allowNA && nc < 0) {
+		if (msg_name)
+		    error(_("invalid multibyte string, %s"), msg_name);
+		else
+		    return -1;
+	    }
 	    return (nc >= 0 ? nc : NA_INTEGER);
 	} else
 	    return ((int) strlen(translateChar(string)));
@@ -171,44 +211,81 @@ int R_nchar(SEXP string, nchar_type type_,
 	if (IS_UTF8(string)) {
 	    const char *p = CHAR(string);
 	    if (!utf8Valid(p)) {
-		if (!allowNA)
-		    error(_("invalid multibyte string, %s"), msg_name);
+		if (!allowNA) {
+		    if (msg_name)
+			error(_("invalid multibyte string, %s"), msg_name);
+		    else
+			return -1;
+		}
 		return NA_INTEGER;
 	    } else {
-		wchar_t wc1;
-		R_wchar_t ucs;
 		int nc = 0;
 		for( ; *p; p += utf8clen(*p)) {
+		    wchar_t wc1;
 		    utf8toucs(&wc1, p);
+		    R_wchar_t ucs;
 		    if (IS_HIGH_SURROGATE(wc1))
 		    	ucs = utf8toucs32(wc1, p);
 		    else
 		    	ucs = wc1;
+#ifdef USE_RI18N_WIDTH
 		    nc += Ri18n_wcwidth(ucs);
+#else
+		    {
+			int this = wcwidth(ucs);
+			if (this >= 0) nc += this;
+		    }
+#endif
 		}
 		return nc;
 	    }
 	} else if (IS_BYTES(string)) {
-	    if (!allowNA) /* could do width 0 */
-		error(_("width is not computable for %s in \"bytes\" encoding"),
-		      msg_name);
+	    if (!allowNA) { /* could do width 0 */
+		if (msg_name)
+		    error(_("width is not computable for %s in \"bytes\" encoding"),
+		          msg_name);
+		else
+		    return -2;
+	    }
 	    return NA_INTEGER;
+	} else if (IS_LATIN1(string)) {
+	    // just count bytes as they are all width-1 chars
+	    // FIXME, well not control chars but there is ambiguity for most of 0x80-9F
+	    return (int) strlen(CHAR(string));
 	} else if (mbcslocale) {
 	    const char *xi = translateChar(string);
 	    int nc = (int) mbstowcs(NULL, xi, 0);
 	    if (nc >= 0) {
 		const void *vmax = vmaxget();
+		/* working in wchar_t restricts this to the BMP on
+		   Windows, but maybe that is all current native
+		   charsets cover. */
 		wchar_t *wc = (wchar_t *)
 		    R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 		mbstowcs(wc, xi, nc + 1);
+		// FIXME: width could conceivably exceed MAX_INT.
+#ifdef USE_RI18N_WIDTH
 		int nci18n = Ri18n_wcswidth(wc, 2147483647);
+#else
+		// We do not use this unless R_wchar_t is wchar_t
+		// This could be -1 if there are non-printable chars,
+		// then this is ignored
+		int nci18n = wcswidth(wc, 2147483647);
+#endif
+		if (msg_name)
+		    R_FreeStringBufferL(&cbuff);
 		vmaxset(vmax);
 		return (nci18n < 1) ? nc : nci18n;
-	    } else if (allowNA)
-		error(_("invalid multibyte string, %s"), msg_name);
+	    } else if (allowNA) {
+		if (msg_name)
+		    error(_("invalid multibyte string, %s"), msg_name);
+		else
+		    return -1;
+	    }
 	    else
 		return NA_INTEGER;
 	} else
+	    // See Latin-1 comment.
 	    return (int) strlen(translateChar(string));
 
     } // switch
@@ -262,8 +339,21 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
     int *s_ = INTEGER(s);
     for (R_xlen_t i = 0; i < len; i++) {
 	SEXP sxi = STRING_ELT(x, i);
-	char msg_i[30]; sprintf(msg_i, "element %ld", (long)i+1);
-	s_[i] = R_nchar(sxi, type_, allowNA, keepNA, msg_i);
+	int res = R_nchar(sxi, type_, allowNA, keepNA, NULL);
+	switch(res) {
+	case -1:
+	    error(_("invalid multibyte string, element %ld"), (long)i+1);
+	case -2: 
+	    if (type_ == Chars)
+		error(_("number of characters is not computable in \"bytes\" encoding, element %ld"),
+		      (long)i+1);
+	    else /* type_ == Width */
+		error(_("width is not computable in \"bytes\" encoding, element %ld"),
+		      (long)i+1);
+	default:
+	    s_[i] = res;
+	    break;
+	}
     }
     R_FreeStringBufferL(&cbuff);
     if ((d = getAttrib(x, R_NamesSymbol)) != R_NilValue)
@@ -309,11 +399,11 @@ static void substr(const char *str, int len, int ienc, int sa, int so,
 	mbs_init(&mb_st);
 	for (i = 0; i < sa - 1 && str < end; i++)
 	    /* throws error on invalid multi-byte string */
-	    str += Mbrtowc(NULL, str, MB_CUR_MAX, &mb_st);
+	    str += Mbrtowc(NULL, str, R_MB_CUR_MAX, &mb_st);
 	*rfrom = str;
 	for (; i < so && str < end; i++)
 	    /* throws error on invalid multi-byte string */
-	    str += (int) Mbrtowc(NULL, str, MB_CUR_MAX, &mb_st);
+	    str += (int) Mbrtowc(NULL, str, R_MB_CUR_MAX, &mb_st);
 	*rlen = (int) (str - *rfrom);
     } else {
 	if (so - 1 < len) {
@@ -522,11 +612,11 @@ substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so,
     } else {
 	/* This cannot work for stateful encodings */
 	if (mbcslocale) {
-	    for (i = 1; i < sa; i++) buf += Mbrtowc(NULL, buf, MB_CUR_MAX, NULL);
+	    for (i = 1; i < sa; i++) buf += Mbrtowc(NULL, buf, R_MB_CUR_MAX, NULL);
 	    /* now work out how many bytes to replace by how many */
 	    for (i = sa; i <= so && in < strlen(str); i++) {
-		in += (int) Mbrtowc(NULL, str+in, MB_CUR_MAX, NULL);
-		out += (int) Mbrtowc(NULL, buf+out, MB_CUR_MAX, NULL);
+		in += (int) Mbrtowc(NULL, str+in, R_MB_CUR_MAX, NULL);
+		out += (int) Mbrtowc(NULL, buf+out, R_MB_CUR_MAX, NULL);
 		if (!str[in]) break;
 	    }
 	    if (in != out) memmove(buf+in, buf+out, strlen(buf+out)+1);
@@ -592,7 +682,7 @@ SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 		int ienc2 = ienc;
 		v_ss = CHAR(v_el);
 		/* is the value in the same encoding?
-		   FIXME: could prefer UTF-8 here
+		   FIXME: could re-encode to UTF-8 rather than to native.
 		 */
 		venc = getCharCE(v_el);
 		if (venc != ienc && !strIsASCII(v_ss)) {
@@ -855,7 +945,7 @@ SEXP attribute_hidden do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
 		int nc = (int) utf8towcs(NULL, s, 0);
 		if (nc > minlen) {
 		    warn = TRUE;
-		    const wchar_t *wc = wtransChar(el);
+		    const wchar_t *wc = wtransChar(el); // to WCS-2 on Windows
 		    nc = (int) wcslen(wc);
 		    R_AllocStringBuffer(sizeof(wchar_t)*(nc+1), &cbuff);
 		    SET_STRING_ELT(ans, i, wstripchars(wc, minlen, usecl));
@@ -904,11 +994,11 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 	    mbstate_t mb_st;
 	    const char *pp = This;
 	    mbs_init(&mb_st);
-	    used = (int) Mbrtowc(&wc, pp, MB_CUR_MAX, &mb_st);
+	    used = (int) Mbrtowc(&wc, pp, R_MB_CUR_MAX, &mb_st);
 	    pp += used; nc -= used;
 	    if (wc == L'.') {
 		if (nc > 0) {
-		    Mbrtowc(&wc, pp, MB_CUR_MAX, &mb_st);
+		    Mbrtowc(&wc, pp, R_MB_CUR_MAX, &mb_st);
 		    if (iswdigit(wc))  need_prefix = TRUE;
 		}
 	    } else if (!iswalpha(wc)) need_prefix = TRUE;
@@ -946,7 +1036,6 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 		/* else leave alone */
 	    }
 	}
-//	l = (int) strlen(tmp);        /* needed? */
 	SET_STRING_ELT(ans, i, mkChar(tmp));
 	/* do we have a reserved word?  If so the name is invalid */
 	if (!isValidName(tmp)) {
@@ -984,17 +1073,16 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
     if (!isString(x)) error(_("non-character argument"));
     n = XLENGTH(x);
     PROTECT(y = allocVector(STRSXP, n));
-#if defined(Win32) || defined(__STDC_ISO_10646__) || defined(__APPLE__) || defined(__FreeBSD__)
-    /* utf8towcs is really to UCS-4/2 */
-    for (i = 0; i < n; i++)
-	if (getCharCE(STRING_ELT(x, i)) == CE_UTF8) use_UTF8 = TRUE;
-    if (mbcslocale || use_UTF8 == TRUE)
-#else
-    if (mbcslocale)
-#endif
-    {
+    for (i = 0; i < n; i++) {
+	SEXP xi = STRING_ELT(x, i);
+	if (IS_UTF8(xi) ||
+	    (!latin1locale && IS_LATIN1(xi))) use_UTF8 = TRUE;
+    }
+    if (mbcslocale || use_UTF8 == TRUE) {
 	int nb, nc, j;
+#ifndef USE_RI18N_CASE
 	wctrans_t tr = wctrans(ul ? "toupper" : "tolower");
+#endif
 	wchar_t * wc;
 	char * cbuf;
 
@@ -1004,30 +1092,64 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 	    el = STRING_ELT(x, i);
 	    if (el == NA_STRING) SET_STRING_ELT(y, i, NA_STRING);
 	    else {
+		/* FIXME: in Windows UTF-8 locales, use UTF-8 branch */
 		const char *xi;
 		ienc = getCharCE(el);
 		if (use_UTF8 && ienc == CE_UTF8) {
 		    xi = CHAR(el);
+		    // could overcount if there are conjugate pairs
 		    nc = (int) utf8towcs(NULL, xi, 0);
+		} else if (use_UTF8 && ienc == CE_LATIN1) {
+		    xi = translateCharUTF8(el); // in case it is really in CP1252
+		    nc = (int) utf8towcs(NULL, xi, 0);
+		    ienc = CE_UTF8;
 		} else {
 		    xi = translateChar(el);
 		    nc = (int) mbstowcs(NULL, xi, 0);
 		    ienc = CE_NATIVE;
 		}
 		if (nc >= 0) {
-		    /* FIXME use this buffer for new string as well */
-		    wc = (wchar_t *)
-			R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 		    if (ienc == CE_UTF8) {
+#ifdef USE_RI18N_CASE
+			R_wchar_t *wcr = (R_wchar_t *)
+			    R_AllocStringBuffer((nc+1)*sizeof(R_wchar_t), &cbuff);
+			utf8towcs4(wcr, xi, nc + 1);
+			if (ul)
+			    for (j = 0; j < nc; j++)
+				wcr[j] = Ri18n_towupper(wcr[j]);
+			else
+			    for (j = 0; j < nc; j++)
+				wcr[j] = Ri18n_towlower(wcr[j]);
+			nb = (int) wcs4toutf8(NULL, wcr, INT_MAX);
+			cbuf = CallocCharBuf(nb);
+			wcs4toutf8(cbuf, wcr, nb);
+			SET_STRING_ELT(y, i, mkCharCE(cbuf, CE_UTF8));
+#else
+			wc = (wchar_t *)
+			    R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 			utf8towcs(wc, xi, nc + 1);
 			for (j = 0; j < nc; j++) wc[j] = towctrans(wc[j], tr);
 			nb = (int) wcstoutf8(NULL, wc, INT_MAX);
 			cbuf = CallocCharBuf(nb);
 			wcstoutf8(cbuf, wc, nb);
 			SET_STRING_ELT(y, i, mkCharCE(cbuf, CE_UTF8));
+#endif
 		    } else {
+			wc = (wchar_t *)
+			    R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 			mbstowcs(wc, xi, nc + 1);
+#ifdef USE_RI18N_CASE
+			if (ul)
+			    for (j = 0; j < nc; j++)
+				wc[j] = Ri18n_towupper(wc[j]);
+			else
+			    for (j = 0; j < nc; j++)
+				wc[j] = Ri18n_towlower(wc[j]);
+#else
+			/* This cannot cope with surrogate pairs,
+			   if mbstowcs can make them. */ 
 			for (j = 0; j < nc; j++) wc[j] = towctrans(wc[j], tr);
+#endif
 			nb = (int) wcstombs(NULL, wc, 0);
 			cbuf = CallocCharBuf(nb);
 			wcstombs(cbuf, wc, nb + 1);
@@ -1065,6 +1187,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 
+/* These assume one wchar_t per char so will not work with surrogate pairs */
 typedef enum { WTR_INIT, WTR_CHAR, WTR_RANGE } wtr_type;
 struct wtr_spec {
     wtr_type type;
@@ -1313,7 +1436,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
     char *cbuf;
     SEXP el;
     cetype_t ienc;
-    Rboolean use_UTF8 = FALSE;
+    Rboolean use_WC = FALSE;
     const void *vmax;
 
     checkArity(op, args);
@@ -1331,19 +1454,21 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	warning(_("argument '%s' has length > 1 and only the first element will be used"), "new");
     if (!isString(x)) error("invalid '%s' argument", "x");
 
-    /* utf8towcs is really to UCS-4/2 */
-#if defined(Win32) || defined(__STDC_ISO_10646__) || defined(__APPLE__)  || defined(__FreeBSD__)
-    for (i = 0; i < n; i++)
-	if (getCharCE(STRING_ELT(x, i)) == CE_UTF8) use_UTF8 = TRUE;
+    /* If we have marked strings we want to do this in Unicode as some
+     * of them might be mis-represented by translateChar.  But
+     * utf8towcs may not be reliable unless TO_WCS_OK is defined.
+     */
+    for (i = 0; i < n; i++) {
+	SEXP xi = STRING_ELT(x, i);
+	if (IS_UTF8(xi) || (!latin1locale && IS_LATIN1(xi))) use_WC = TRUE;
+    }
 
-    if (getCharCE(STRING_ELT(old, 0)) == CE_UTF8) use_UTF8 = TRUE;
-    if (getCharCE(STRING_ELT(_new, 0)) == CE_UTF8) use_UTF8 = TRUE;
+    if (IS_UTF8(STRING_ELT(old, 0)) ||
+	(!latin1locale && IS_LATIN1(STRING_ELT(old, 0)))) use_WC = TRUE;
+    if (IS_UTF8(STRING_ELT(_new, 0)) ||
+	(!latin1locale && IS_LATIN1(STRING_ELT(_new, 0)))) use_WC = TRUE;
 
-    if (mbcslocale || use_UTF8 == TRUE)
-#else
-    if (mbcslocale)
-#endif
-    {
+    if (mbcslocale || use_WC == TRUE) {
 	int j, nb, nc;
 	xtable_t *xtable, *tbl;
 	int xtable_cnt;
@@ -1361,12 +1486,18 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	trs_new->type = WTR_INIT;
 	trs_new->next = NULL;
 	/* Build the old and new wtr_spec lists. */
-	if (use_UTF8 && getCharCE(STRING_ELT(old, 0)) == CE_UTF8) {
+	if (use_WC && IS_UTF8(STRING_ELT(old, 0))) {
 	    s = CHAR(STRING_ELT(old, 0));
 	    nc = (int) utf8towcs(NULL, s, 0);
 	    if (nc < 0) error(_("invalid UTF-8 string 'old'"));
 	    wc = (wchar_t *) R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 	    utf8towcs(wc, s, nc + 1);
+	} else if (use_WC && IS_LATIN1(STRING_ELT(old, 0))) {
+	    s = translateCharUTF8(STRING_ELT(old, 0));
+	    nc = (int) utf8towcs(NULL, s, 0);
+	    if (nc < 0) error(_("invalid UTF-8 string 'old'")); // but must be valid
+	    wc = (wchar_t *) R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
+	    utf8towcs(wc, s, nc + 1);
 	} else {
 	    s = translateChar(STRING_ELT(old, 0));
 	    nc = (int) mbstowcs(NULL, s, 0);
@@ -1380,12 +1511,18 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	trs_cnt->next = NULL;
 	wtr_build_spec(wc, trs_cnt); /* use count only */
 
-	if (use_UTF8 && getCharCE(STRING_ELT(_new, 0)) == CE_UTF8) {
+	if (use_WC && IS_UTF8(STRING_ELT(_new, 0))) {
 	    s = CHAR(STRING_ELT(_new, 0));
 	    nc = (int) utf8towcs(NULL, s, 0);
 	    if (nc < 0) error(_("invalid UTF-8 string 'new'"));
 	    wc = (wchar_t *) R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
 	    utf8towcs(wc, s, nc + 1);
+	} else if (use_WC && IS_LATIN1(STRING_ELT(_new, 0))) {
+	    s = translateCharUTF8(STRING_ELT(_new, 0));
+	    nc = (int) utf8towcs(NULL, s, 0);
+	    if (nc < 0) error(_("invalid UTF-8 string 'new'"));
+	    wc = (wchar_t *) R_AllocStringBuffer((nc+1)*sizeof(wchar_t), &cbuff);
+	    utf8towcs(wc, s, nc + 1);
 	} else {
 	    s = translateChar(STRING_ELT(_new, 0));
 	    nc = (int) mbstowcs(NULL, s, 0);
@@ -1440,7 +1577,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 		SET_STRING_ELT(y, i, NA_STRING);
 	    else {
 		ienc = getCharCE(el);
-		if (use_UTF8 && ienc == CE_UTF8) {
+		if (use_WC && ienc == CE_UTF8) {
 		    xi = CHAR(el);
 		    nc = (int) utf8towcs(NULL, xi, 0);
 		} else {
@@ -1549,7 +1686,6 @@ SEXP attribute_hidden do_strtrim(SEXP call, SEXP op, SEXP args, SEXP env)
     char *buf;
     const char *p; char *q;
     int w0, wsum, k, nb;
-    wchar_t wc;
     mbstate_t mb_st;
     const void *vmax;
 
@@ -1575,14 +1711,20 @@ SEXP attribute_hidden do_strtrim(SEXP call, SEXP op, SEXP args, SEXP env)
 		continue;
 	    }
 	    w = INTEGER(width)[i % nw];
+	    // FIXME: this could do a better job with UTF-8 or Latin-1 input
 	    This = translateChar(STRING_ELT(x, i));
 	    nc = (int) strlen(This);
 	    buf = R_AllocStringBuffer(nc, &cbuff);
 	    wsum = 0;
 	    mbs_init(&mb_st);
 	    for (p = This, w0 = 0, q = buf; *p ;) {
-		nb =  (int) Mbrtowc(&wc, p, MB_CUR_MAX, &mb_st);
-		w0 = Ri18n_wcwidth((R_wchar_t)wc);
+		wchar_t wc;
+		nb =  (int) Mbrtowc(&wc, p, R_MB_CUR_MAX, &mb_st);
+#ifdef USE_RI18N_WIDTH
+		w0 = Ri18n_wcwidth((R_wchar_t) wc);
+#else
+		w0 = wcwidth(wc);
+#endif
 		if (w0 < 0) { p += nb; continue; } /* skip non-printable chars */
 		wsum += w0;
 		if (wsum <= w) {
