@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995--2020  The R Core Team.
+ *  Copyright (C) 1995--2021  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,13 +48,19 @@ static SEXP evalKeepVis(SEXP e, SEXP rho)
 #ifndef min
 #define min(a, b) (a<b?a:b)
 #endif
+#ifndef max
+#define max(a, b) (a>b?a:b)
+#endif
 
 /* Total line length, in chars, before splitting in warnings/errors */
 #define LONGWARN 75
 
 /*
 Different values of inError are used to indicate different places
-in the error handling.
+in the error handling:
+inError = 1: In internal error handling, e.g. `verrorcall_dflt`, others.
+inError = 2: Writing traceback
+inError = 3: In user error handler (i.e. options(error=handler))
 */
 static int inError = 0;
 static int inWarning = 0;
@@ -195,7 +201,7 @@ void onintrNoResume() { onintrEx(FALSE); }
    These do far more processing than is allowed in a signal handler ....
 */
 
-RETSIGTYPE attribute_hidden onsigusr1(int dummy)
+void attribute_hidden onsigusr1(int dummy)
 {
     if (R_interrupts_suspended) {
 	/**** ought to save signal and handle after suspend */
@@ -230,7 +236,7 @@ RETSIGTYPE attribute_hidden onsigusr1(int dummy)
 }
 
 
-RETSIGTYPE attribute_hidden onsigusr2(int dummy)
+void attribute_hidden onsigusr2(int dummy)
 {
     inError = 1;
 
@@ -259,42 +265,63 @@ static void setupwarnings(void)
     setAttrib(R_Warnings, R_NamesSymbol, allocVector(STRSXP, R_nwarnings));
 }
 
-/* Rvsnprintf: like vsnprintf, but guaranteed to null-terminate and not to
-   split multi-byte characters */
+/* Rvsnprintf_mbcs: like vsnprintf, but guaranteed to null-terminate and not to
+   split multi-byte characters, except if size is zero in which case the buffer
+   is untouched and thus may not be null-terminated.
+
+   This function may be invoked by the error handler via REvprintf.  Do not
+   change it unless you are SURE that your changes are compatible with the
+   error handling mechanism.
+
+   REvprintf is also used in R_Suicide on Unix.
+
+   Dangerous pattern: `Rvsnprintf_mbcs(buf, size - n, )` with n >= size */
 #ifdef Win32
 int trio_vsnprintf(char *buffer, size_t bufferSize, const char *format,
 		   va_list args);
 
-static int Rvsnprintf(char *buf, size_t size, const char  *format, va_list ap)
+attribute_hidden
+int Rvsnprintf_mbcs(char *buf, size_t size, const char *format, va_list ap)
 {
     int val;
     val = trio_vsnprintf(buf, size, format, ap);
-    buf[size-1] = '\0';
-    if (val >= size)
-	mbcsTruncateToValid(buf);
+    if (size) {
+	if (val < 0) buf[0] = '\0'; /* not all uses check val < 0 */
+	else buf[size-1] = '\0';
+	if (val >= size)
+	    mbcsTruncateToValid(buf);
+    }
     return val;
 }
 #else
-static int Rvsnprintf(char *buf, size_t size, const char  *format, va_list ap)
+attribute_hidden
+int Rvsnprintf_mbcs(char *buf, size_t size, const char *format, va_list ap)
 {
     int val;
     val = vsnprintf(buf, size, format, ap);
-    buf[size-1] = '\0';
-    if (val >= size)
-	mbcsTruncateToValid(buf);
+    if (size) {
+	if (val < 0) buf[0] = '\0'; /* not all uses check val < 0 */
+	else buf[size-1] = '\0';
+	if (val >= size)
+	    mbcsTruncateToValid(buf);
+    }
     return val;
 }
 #endif
 
-/* Rsnprintf: like snprintf, but guaranteed to null-terminate and not to
-   split multi-byte characters */
-static int Rsnprintf(char *str, size_t size, const char *format, ...)
+/* Rsnprintf_mbcs: like snprintf, but guaranteed to null-terminate and
+   not to split multi-byte characters, except if size is zero in which
+   case the buffer is untouched and thus may not be null-terminated.
+
+   Dangerous pattern: `Rsnprintf_mbcs(buf, size - n, )` with maybe n >= size*/
+attribute_hidden
+int Rsnprintf_mbcs(char *str, size_t size, const char *format, ...)
 {
     int val;
     va_list ap;
 
     va_start(ap, format);
-    val = Rvsnprintf(str, size, format, ap);
+    val = Rvsnprintf_mbcs(str, size, format, ap);
     va_end(ap);
 
     return val;
@@ -317,12 +344,12 @@ static char *Rstrncat(char *dest, const char *src, size_t n)
     return dest;
 }
 
-/* Rstrncat: like strncpy, but guaranteed to null-terminate and not to
+/* Rstrncpy: like strncpy, but guaranteed to null-terminate and not to
    split multi-byte characters */
 static char *Rstrncpy(char *dest, const char *src, size_t n)
 {
     strncpy(dest, src, n);
-    if (dest[n-1] != '\0') {
+    if (n) {
 	dest[n-1] = '\0';
 	mbcsTruncateToValid(dest);
     }
@@ -332,11 +359,12 @@ static char *Rstrncpy(char *dest, const char *src, size_t n)
 #define BUFSIZE 8192
 static R_INLINE void RprintTrunc(char *buf, int truncated)
 {
-    if(R_WarnLength < BUFSIZE - 20 &&
-      (truncated || strlen(buf) == R_WarnLength)) {
-
-	strcat(buf, " ");
-	strcat(buf, _("[... truncated]"));
+    if (truncated) {
+	char *msg = _("[... truncated]");
+	if (strlen(buf) + 1 + strlen(msg) < BUFSIZE) {
+	    strcat(buf, " ");
+	    strcat(buf, msg);
+	}
     }
 }
 
@@ -364,7 +392,7 @@ void warning(const char *format, ...)
     int pval;
 
     psize = min(BUFSIZE, R_WarnLength+1);
-    pval = Rvsnprintf(buf, psize, format, ap);
+    pval = Rvsnprintf_mbcs(buf, psize, format, ap);
     va_end(ap);
     p = buf + strlen(buf) - 1;
     if(strlen(buf) > 0 && *p == '\n') *p = '\0';
@@ -391,7 +419,12 @@ static int wd(const char * buf)
     if(nc > 0 && nc < 2000) {
 	wchar_t wc[2000];
 	mbstowcs(wc, buf, nc + 1);
+	// FIXME: width could conceivably exceed MAX_INT.
+#ifdef USE_RI18N_WIDTH
 	nw = Ri18n_wcswidth(wc, 2147483647);
+#else
+	nw = wcswidth(wc, 2147483647);
+#endif
 	return (nw < 1) ? nc : nw;
     }
     return nc;
@@ -441,7 +474,7 @@ static void vwarningcall_dflt(SEXP call, const char *format, va_list ap)
 
     if(w >= 2) { /* make it an error */
 	psize = min(BUFSIZE, R_WarnLength+1);
-	pval = Rvsnprintf(buf, psize, format, ap);
+	pval = Rvsnprintf_mbcs(buf, psize, format, ap);
 	RprintTrunc(buf, pval >= psize);
 	inWarning = 0; /* PR#1570 */
 	errorcall(call, _("(converted from warning) %s"), buf);
@@ -452,7 +485,7 @@ static void vwarningcall_dflt(SEXP call, const char *format, va_list ap)
 	    dcall = CHAR(STRING_ELT(deparse1s(call), 0));
 	} else dcall = "";
 	psize = min(BUFSIZE, R_WarnLength+1);
-	pval = Rvsnprintf(buf, psize, format, ap);
+	pval = Rvsnprintf_mbcs(buf, psize, format, ap);
 	RprintTrunc(buf, pval >= psize);
 
 	if(dcall[0] == '\0') REprintf(_("Warning:"));
@@ -474,7 +507,7 @@ static void vwarningcall_dflt(SEXP call, const char *format, va_list ap)
 	if(R_CollectWarnings < R_nwarnings) {
 	    SET_VECTOR_ELT(R_Warnings, R_CollectWarnings, call);
 	    psize = min(BUFSIZE, R_WarnLength+1);
-	    pval = Rvsnprintf(buf, psize, format, ap);
+	    pval = Rvsnprintf_mbcs(buf, psize, format, ap);
 	    RprintTrunc(buf, pval >= psize);
 	    if(R_ShowWarnCalls && call != R_NilValue) {
 		char *tr =  R_ConciseTraceback(call, 0);
@@ -698,6 +731,8 @@ static void restore_inError(void *data)
    checks in GC and session exit (or .Call) do not have such limit. */
 static int allowedConstsChecks = 1000;
 
+/* Construct newline terminated error message, write it to global errbuf, and
+   possibly display with REprintf. */
 static void NORET
 verrorcall_dflt(SEXP call, const char *format, va_list ap)
 {
@@ -716,7 +751,7 @@ verrorcall_dflt(SEXP call, const char *format, va_list ap)
 	    REprintf(_("Error during wrapup: "));
 	    /* this does NOT try to print the call since that could
 	       cause a cascade of error calls */
-	    Rvsnprintf(errbuf, sizeof(errbuf), format, ap);
+	    Rvsnprintf_mbcs(errbuf, sizeof(errbuf), format, ap);
 	    REprintf("%s\n", errbuf);
 	}
 	if (R_Warnings != R_NilValue) {
@@ -738,6 +773,9 @@ verrorcall_dflt(SEXP call, const char *format, va_list ap)
     oldInError = inError;
     inError = 1;
 
+    // For use with Rv?snprintf, which truncates at size - 1, hence the + 1
+    size_t msg_len = min(BUFSIZE, R_WarnLength) + 1;
+
     if(call != R_NilValue) {
 	char tmp[BUFSIZE], tmp2[BUFSIZE];
 	char *head = _("Error in "), *tail = "\n  ";
@@ -756,22 +794,22 @@ verrorcall_dflt(SEXP call, const char *format, va_list ap)
 	}
 
 	const char *dcall = CHAR(STRING_ELT(deparse1s(call), 0));
-	Rsnprintf(tmp2, BUFSIZE,  "%s", head);
+	Rsnprintf_mbcs(tmp2, BUFSIZE,  "%s", head);
 	if (skip != NA_INTEGER) {
 	    PROTECT(srcloc = GetSrcLoc(R_GetCurrentSrcref(skip)));
 	    protected++;
 	    len = strlen(CHAR(STRING_ELT(srcloc, 0)));
 	    if (len)
-		Rsnprintf(tmp2, BUFSIZE,  _("Error in %s (from %s) : "),
-			 dcall, CHAR(STRING_ELT(srcloc, 0)));
+		Rsnprintf_mbcs(tmp2, BUFSIZE,  _("Error in %s (from %s) : "),
+			       dcall, CHAR(STRING_ELT(srcloc, 0)));
 	}
 
-	Rvsnprintf(tmp, min(BUFSIZE, R_WarnLength) - strlen(head), format, ap);
+	Rvsnprintf_mbcs(tmp, max(msg_len - strlen(head), 0), format, ap);
 	if (strlen(tmp2) + strlen(tail) + strlen(tmp) < BUFSIZE) {
-	    if(len) Rsnprintf(errbuf, BUFSIZE,
-			     _("Error in %s (from %s) : "),
-			     dcall, CHAR(STRING_ELT(srcloc, 0)));
-	    else Rsnprintf(errbuf, BUFSIZE,  _("Error in %s : "), dcall);
+	    if(len) Rsnprintf_mbcs(errbuf, BUFSIZE,
+				   _("Error in %s (from %s) : "),
+				   dcall, CHAR(STRING_ELT(srcloc, 0)));
+	    else Rsnprintf_mbcs(errbuf, BUFSIZE,  _("Error in %s : "), dcall);
 	    if (mbcslocale) {
 		int msgline1;
 		char *p = strchr(tmp, '\n');
@@ -794,37 +832,43 @@ verrorcall_dflt(SEXP call, const char *format, va_list ap)
 	    }
 	    ERRBUFCAT(tmp);
 	} else {
-	    Rsnprintf(errbuf, BUFSIZE, _("Error: "));
-	    ERRBUFCAT(tmp); // FIXME
+	    Rsnprintf_mbcs(errbuf, BUFSIZE, _("Error: "));
+	    ERRBUFCAT(tmp);
 	}
 	UNPROTECT(protected);
     }
     else {
-	Rsnprintf(errbuf, BUFSIZE, _("Error: "));
+	Rsnprintf_mbcs(errbuf, BUFSIZE, _("Error: "));
 	p = errbuf + strlen(errbuf);
-	Rvsnprintf(p, min(BUFSIZE, R_WarnLength) - strlen(errbuf), format, ap);
+	Rvsnprintf_mbcs(p, max(msg_len - strlen(errbuf), 0), format, ap);
     }
-
-    size_t nc = strlen(errbuf);
-    if (nc == BUFSIZE - 1) {
-	errbuf[BUFSIZE - 4] = '.';
-	errbuf[BUFSIZE - 3] = '.';
-	errbuf[BUFSIZE - 2] = '.';
-	errbuf[BUFSIZE - 1] = '\n';
-    }
-    else {
+    /* Approximate truncation detection, may produce false positives.  Assumes
+       R_MB_CUR_MAX > 0. Note: approximation is fine, as the string may include
+       dots, anyway */
+    size_t nc = strlen(errbuf); // > 0, ignoring possibility of failure
+    if (nc > BUFSIZE - 1 - (R_MB_CUR_MAX - 1)) {
+	size_t end = min(nc + 1, (BUFSIZE + 1) - 4); // room for "...\n\0"
+	for(size_t i = end; i <= BUFSIZE + 1; ++i) errbuf[i - 1] = '\0';
+	mbcsTruncateToValid(errbuf);
+	ERRBUFCAT("...\n");
+    } else {
 	p = errbuf + nc - 1;
-	if(*p != '\n') ERRBUFCAT("\n");
-    }
-
-    if(R_ShowErrorCalls && call != R_NilValue) {  /* assume we want to avoid deparse */
-	tr = R_ConciseTraceback(call, 0);
-	size_t nc = strlen(tr);
-	if (nc && nc + strlen(errbuf) + 8 < BUFSIZE) {
-	    ERRBUFCAT(_("Calls:"));
-	    ERRBUFCAT(" ");
-	    ERRBUFCAT(tr);
-	    ERRBUFCAT("\n");
+	if(*p != '\n') {
+	    ERRBUFCAT("\n");  // guaranteed to have room for this
+	    ++nc;
+	}
+	if(R_ShowErrorCalls && call != R_NilValue) {  /* assume we want to avoid deparse */
+	    tr = R_ConciseTraceback(call, 0);
+	    size_t nc_tr = strlen(tr);
+	    if (nc_tr) {
+		char * call_trans = _("Calls:");
+		if (nc_tr + nc + strlen(call_trans) + 2 < BUFSIZE + 1) {
+		    ERRBUFCAT(call_trans);
+		    ERRBUFCAT(" ");
+		    ERRBUFCAT(tr);
+		    ERRBUFCAT("\n");
+		}
+	    }
 	}
     }
     if (R_ShowErrorMessages) REprintf("%s", errbuf);
@@ -867,7 +911,7 @@ void NORET errorcall(SEXP call, const char *format,...)
 	void (*hook)(SEXP, char *) = R_ErrorHook;
 	R_ErrorHook = NULL; /* to avoid recursion */
 	va_start(ap, format);
-	Rvsnprintf(buf, min(BUFSIZE, R_WarnLength), format, ap);
+	Rvsnprintf_mbcs(buf, min(BUFSIZE, R_WarnLength), format, ap);
 	va_end(ap);
 	hook(call, buf);
     }
@@ -886,7 +930,7 @@ void NORET errorcall_cpy(SEXP call, const char *format, ...)
 
     va_list(ap);
     va_start(ap, format);
-    Rvsnprintf(buf, BUFSIZE, format, ap);
+    Rvsnprintf_mbcs(buf, BUFSIZE, format, ap);
     va_end(ap);
 
     errorcall(call, "%s", buf);
@@ -908,7 +952,7 @@ void error(const char *format, ...)
 
     va_list(ap);
     va_start(ap, format);
-    Rvsnprintf(buf, min(BUFSIZE, R_WarnLength), format, ap);
+    Rvsnprintf_mbcs(buf, min(BUFSIZE, R_WarnLength), format, ap);
     va_end(ap);
     errorcall(getCurrentCall(), "%s", buf);
 }
@@ -1045,60 +1089,99 @@ void NORET jump_to_toplevel()
 
 /* #define DEBUG_GETTEXT 1 */
 
+
+/* Called from do_gettext() and do_ngettext() */
+static char * determine_domain_gettext(SEXP domain_, SEXP rho)
+{
+    const char *domain = "";
+    char *buf; // will be returned
+
+    /* The passed rho is ignored because (n)gettext is called through a
+     * helper function in stop.R. And the context is more useful anyhow */
+    if(isNull(domain_)) {
+	RCNTXT *cptr;
+	for (cptr = R_GlobalContext; cptr != NULL && cptr->callflag != CTXT_TOPLEVEL;
+	     cptr = cptr->nextcontext)
+	    if (cptr->callflag & CTXT_FUNCTION) {
+		/* stop() etc have internal call to .makeMessage */
+		const char *cfn = CHAR(STRING_ELT(deparse1s(CAR(cptr->call)), 0));
+
+		if(streql(cfn, "stop") || streql(cfn, "warning") || streql(cfn, "message"))
+		    continue;
+		else
+		    break;
+	    }
+
+	/* First we try to see if sysparent leads us to a namespace, because gettext
+	   might have a different environment due to being called from (in?) a closure.
+	   If that fails we try cloenv, as the original code did. */
+	/* FIXME: should we only do this search when cptr->callflag & CTXT_FUNCTION? */
+	SEXP ns = R_NilValue;
+	for(size_t attempt = 0; attempt < 2 && isNull(ns); attempt++) {
+	    rho = (cptr == NULL) ?
+		R_EmptyEnv :
+		attempt == 0 ? cptr->sysparent : cptr->cloenv;
+	    while(rho != R_EmptyEnv) {
+		if (rho == R_GlobalEnv) break;
+		else if (R_IsNamespaceEnv(rho)) {
+		    ns = R_NamespaceEnvSpec(rho);
+		    break;
+		}
+		rho = ENCLOS(rho);
+	    }
+	}
+	if (!isNull(ns)) {
+	    PROTECT(ns);
+	    domain = translateChar(STRING_ELT(ns, 0));
+	    if (strlen(domain)) {
+		size_t len = strlen(domain)+3;
+		buf = R_alloc(len, sizeof(char));
+		Rsnprintf_mbcs(buf, len, "R-%s", domain);
+		UNPROTECT(1); /* ns */
+#ifdef DEBUG_GETTEXT
+		REprintf("Managed to determine 'domain' from environment as: '%s'\n", buf);
+#endif
+		return buf;
+	    }
+	    UNPROTECT(1); /* ns */ 
+	}
+	return NULL;
+
+    } else if(isString(domain_)) {
+	domain = translateChar(STRING_ELT(domain_, 0));
+	if (!strlen(domain))
+	    return NULL;
+	buf = R_alloc(strlen(domain) + 1, sizeof(char));
+	strcpy(buf, domain);
+	return buf;
+	
+    } else if(isLogical(domain_) && LENGTH(domain_) == 1 && LOGICAL(domain_)[0] == NA_LOGICAL)
+	return NULL;
+    else error(_("invalid '%s' value"), "domain");
+}
+
+
 /* gettext(domain, string) */
 SEXP attribute_hidden do_gettext(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
 #ifdef ENABLE_NLS
-    const char *domain = "", *cfn;
-    char *buf;
-    SEXP ans, string = CADR(args);
-    int i, n = LENGTH(string);
+    SEXP string = CADR(args);
+    int n = LENGTH(string);
 
-    checkArity(op, args);
     if(isNull(string) || !n) return string;
 
     if(!isString(string)) error(_("invalid '%s' value"), "string");
 
-    if(isNull(CAR(args))) {
-	RCNTXT *cptr;
-	SEXP rho = R_BaseEnv;
-	for (cptr = R_GlobalContext->nextcontext;
-	     cptr != NULL && cptr->callflag != CTXT_TOPLEVEL;
-	     cptr = cptr->nextcontext)
-	    if (cptr->callflag & CTXT_FUNCTION) {
-		/* stop() etc have internal call to .makeMessage */
-		cfn = CHAR(STRING_ELT(deparse1s(CAR(cptr->call)), 0));
-		if(streql(cfn, "stop") || streql(cfn, "warning")
-		   || streql(cfn, "message")) continue;
-		rho = cptr->cloenv;
-	    }
-	while(rho != R_EmptyEnv) {
-	    if (rho == R_GlobalEnv) break;
-	    else if (R_IsNamespaceEnv(rho)) {
-		domain = translateChar(STRING_ELT(R_NamespaceEnvSpec(rho), 0));
-		break;
-	    }
-	    rho = CDR(rho);
-	}
-	if(strlen(domain)) {
-	    size_t len = strlen(domain)+3;
-	    R_CheckStack2(len);
-	    buf = (char *) alloca(len);
-	    Rsnprintf(buf, len, "R-%s", domain);
-	    domain = buf;
-	}
-    } else if(isString(CAR(args)))
-	domain = translateChar(STRING_ELT(CAR(args),0));
-    else if(isLogical(CAR(args)) && LENGTH(CAR(args)) == 1 && LOGICAL(CAR(args))[0] == NA_LOGICAL) ;
-    else error(_("invalid '%s' value"), "domain");
+    char * domain = determine_domain_gettext(CAR(args), rho);
 
-    if(strlen(domain)) {
-	PROTECT(ans = allocVector(STRSXP, n));
-	for(i = 0; i < n; i++) {
+    if(domain && strlen(domain)) {
+	SEXP ans = PROTECT(allocVector(STRSXP, n));
+	for(int i = 0; i < n; i++) {
 	    int ihead = 0, itail = 0;
 	    const char * This = translateChar(STRING_ELT(string, i));
 	    char *tmp, *head = NULL, *tail = NULL, *p, *tr;
+
 	    R_CheckStack2(strlen(This) + 1);
 	    tmp = (char *) alloca(strlen(This) + 1);
 	    strcpy(tmp, This);
@@ -1107,28 +1190,32 @@ SEXP attribute_hidden do_gettext(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    for(p = tmp;
 		*p && (*p == ' ' || *p == '\t' || *p == '\n');
 		p++, ihead++) ;
+
 	    if(ihead > 0) {
 		R_CheckStack2(ihead + 1);
 		head = (char *) alloca(ihead + 1);
 		Rstrncpy(head, tmp, ihead + 1);
 		tmp += ihead;
-		}
+	    }
+
 	    if(strlen(tmp))
 		for(p = tmp+strlen(tmp)-1;
 		    p >= tmp && (*p == ' ' || *p == '\t' || *p == '\n');
 		    p--, itail++) ;
+
 	    if(itail > 0) {
 		R_CheckStack2(itail + 1);
 		tail = (char *) alloca(itail + 1);
 		strcpy(tail, tmp+strlen(tmp)-itail);
 		tmp[strlen(tmp)-itail] = '\0';
 	    }
+
 	    if(strlen(tmp)) {
 #ifdef DEBUG_GETTEXT
 		REprintf("translating '%s' in domain '%s'\n", tmp, domain);
 #endif
 		tr = dgettext(domain, tmp);
-		R_CheckStack2(strlen(tr) + ihead + itail + 1);
+ 		R_CheckStack2(        strlen(tr) + ihead + itail + 1);
 		tmp = (char *) alloca(strlen(tr) + ihead + itail + 1);
 		tmp[0] ='\0';
 		if(ihead > 0) strcat(tmp, head);
@@ -1140,20 +1227,15 @@ SEXP attribute_hidden do_gettext(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	UNPROTECT(1);
 	return ans;
-    } else return CADR(args);
-#else
-    return CADR(args);
+    } else
 #endif
+	// no NLS or no domain :
+	return CADR(args);
 }
 
 /* ngettext(n, msg1, msg2, domain) */
 SEXP attribute_hidden do_ngettext(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-#ifdef ENABLE_NLS
-    const char *domain = "", *cfn;;
-    char *buf;
-    SEXP ans, sdom = CADDDR(args);
-#endif
     SEXP msg1 = CADR(args), msg2 = CADDR(args);
     int n = asInteger(CAR(args));
 
@@ -1165,51 +1247,21 @@ SEXP attribute_hidden do_ngettext(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("'%s' must be a character string"), "msg2");
 
 #ifdef ENABLE_NLS
-    if(isNull(sdom)) {
-	RCNTXT *cptr;
-	SEXP rho = R_BaseEnv;
-	for (cptr = R_GlobalContext->nextcontext;
-	     cptr != NULL && cptr->callflag != CTXT_TOPLEVEL;
-	     cptr = cptr->nextcontext)
-	    if (cptr->callflag & CTXT_FUNCTION) {
-		/* stop() etc have internal call to .makeMessage */
-		cfn = CHAR(STRING_ELT(deparse1s(CAR(cptr->call)), 0));
-		if(streql(cfn, "stop") || streql(cfn, "warning")
-		   || streql(cfn, "message")) continue;
-		rho = cptr->cloenv;
-	    }
-	while(rho != R_EmptyEnv) {
-	    if (rho == R_GlobalEnv) break;
-	    else if (R_IsNamespaceEnv(rho)) {
-		domain = translateChar(STRING_ELT(R_NamespaceEnvSpec(rho), 0));
-		break;
-	    }
-	    rho = CDR(rho);
-	}
-	if(strlen(domain)) {
-	    size_t len = strlen(domain)+3;
-	    R_CheckStack2(len);
-	    buf = (char *) alloca(len);
-	    Rsnprintf(buf, len, "R-%s", domain);
-	    domain = buf;
-	}
-    } else if(isString(sdom))
-	domain = CHAR(STRING_ELT(sdom,0));
-    else if(isLogical(sdom) && LENGTH(sdom) == 1 && LOGICAL(sdom)[0] == NA_LOGICAL) ;
-    else error(_("invalid '%s' value"), "domain");
+    SEXP sdom = CADDDR(args);
+    char * domain = determine_domain_gettext(sdom, rho);
 
-    /* libintl seems to malfunction if given a message of "" */
-    if(strlen(domain) && length(STRING_ELT(msg1, 0))) {
-	char *fmt = dngettext(domain,
-			      translateChar(STRING_ELT(msg1, 0)),
-			      translateChar(STRING_ELT(msg2, 0)),
-			      n);
-	PROTECT(ans = mkString(fmt));
-	UNPROTECT(1);
-	return ans;
-    } else
+    if(domain && strlen(domain)) {
+	/* libintl seems to malfunction if given a message of "" */
+	if(length(STRING_ELT(msg1, 0))) {
+	    char *fmt = dngettext(domain,
+		      translateChar(STRING_ELT(msg1, 0)),
+		      translateChar(STRING_ELT(msg2, 0)),
+		      n);
+	    return mkString(fmt);
+	}
+    }
 #endif
-	return n == 1 ? msg1 : msg2;
+    return n == 1 ? msg1 : msg2;
 }
 
 
@@ -1364,7 +1416,7 @@ void NORET ErrorMessage(SEXP call, int which_error, ...)
     }
 
     va_start(ap, which_error);
-    Rvsnprintf(buf, BUFSIZE, _(ErrorDB[i].format), ap);
+    Rvsnprintf_mbcs(buf, BUFSIZE, _(ErrorDB[i].format), ap);
     va_end(ap);
     errorcall(call, "%s", buf);
 }
@@ -1388,7 +1440,7 @@ void WarningMessage(SEXP call, R_WARNING which_warn, ...)
       'va_start' has undefined behavior [-Wvarargs]
 */
     va_start(ap, which_warn);
-    Rvsnprintf(buf, BUFSIZE, _(WarningDB[i].format), ap);
+    Rvsnprintf_mbcs(buf, BUFSIZE, _(WarningDB[i].format), ap);
     va_end(ap);
     warningcall(call, "%s", buf);
 }
@@ -1618,6 +1670,30 @@ static SEXP mkHandlerEntry(SEXP klass, SEXP parentenv, SEXP handler, SEXP rho,
 #define ENTRY_HANDLER(e) VECTOR_ELT(e, 2)
 #define ENTRY_TARGET_ENVIR(e) VECTOR_ELT(e, 3)
 #define ENTRY_RETURN_RESULT(e) VECTOR_ELT(e, 4)
+#define CLEAR_ENTRY_CALLING_ENVIR(e) SET_VECTOR_ELT(e, 1, R_NilValue)
+#define CLEAR_ENTRY_TARGET_ENVIR(e) SET_VECTOR_ELT(e, 3, R_NilValue)
+
+SEXP attribute_hidden R_UnwindHandlerStack(SEXP target)
+{
+    SEXP hs;
+
+    /* check that the target is in the current stack */
+    for (hs = R_HandlerStack; hs != target && hs != R_NilValue; hs = CDR(hs))
+	if (hs == target)
+	    break;
+    if (hs != target)
+	return target; /* restoring a saved stack */
+
+    for (hs = R_HandlerStack; hs != target; hs = CDR(hs)) {
+	/* pop top handler; may not be needed */
+	R_HandlerStack = CDR(hs);
+
+	/* clear the two environments to reduce reference counts */
+	CLEAR_ENTRY_CALLING_ENVIR(CAR(hs));
+	CLEAR_ENTRY_TARGET_ENVIR(CAR(hs));
+    }
+    return target;
+}
 
 #define RESULT_SIZE 4
 
@@ -1723,7 +1799,7 @@ static void vsignalWarning(SEXP call, const char *format, va_list ap)
 	PROTECT(qfun);
 	PROTECT(qcall = LCONS(qfun, LCONS(call, R_NilValue)));
 	PROTECT(hcall = LCONS(qcall, R_NilValue));
-	Rvsnprintf(buf, BUFSIZE - 1, format, ap);
+	Rvsnprintf_mbcs(buf, BUFSIZE - 1, format, ap);
 	hcall = LCONS(mkString(buf), hcall);
 	PROTECT(hcall = LCONS(hooksym, hcall));
 	evalKeepVis(hcall, R_GlobalEnv);
@@ -1748,7 +1824,7 @@ static void vsignalError(SEXP call, const char *format, va_list ap)
     SEXP list, oldstack;
 
     PROTECT(oldstack = R_HandlerStack);
-    Rvsnprintf(localbuf, BUFSIZE - 1, format, ap);
+    Rvsnprintf_mbcs(localbuf, BUFSIZE - 1, format, ap);
     while ((list = findSimpleErrorHandler()) != R_NilValue) {
 	char *buf = errbuf;
 	SEXP entry = CAR(list);
@@ -1763,7 +1839,7 @@ static void vsignalError(SEXP call, const char *format, va_list ap)
 		/* if we are in the process of handling a C stack
 		   overflow, treat all calling handlers as failed */
 		if (R_OldCStackLimit)
-		    break;
+		    continue;
 		SEXP hooksym, hcall, qcall, qfun;
 		/* protect oldstack here, not outside loop, so handler
 		   stack gets unwound in case error is protect stack
@@ -1931,15 +2007,12 @@ R_InsertRestartHandlers(RCNTXT *cptr, const char *cname)
 
 SEXP attribute_hidden do_dfltWarn(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    const char *msg;
-    SEXP ecall;
-
     checkArity(op, args);
 
     if (TYPEOF(CAR(args)) != STRSXP || LENGTH(CAR(args)) != 1)
 	error(_("bad error message"));
-    msg = translateChar(STRING_ELT(CAR(args), 0));
-    ecall = CADR(args);
+    const char *msg = translateChar(STRING_ELT(CAR(args), 0));
+    SEXP ecall = CADR(args);
 
     warningcall_dflt(ecall, "%s", msg);
     return R_NilValue;
@@ -1947,15 +2020,12 @@ SEXP attribute_hidden do_dfltWarn(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 SEXP attribute_hidden NORET do_dfltStop(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    const char *msg;
-    SEXP ecall;
-
     checkArity(op, args);
 
     if (TYPEOF(CAR(args)) != STRSXP || LENGTH(CAR(args)) != 1)
 	error(_("bad error message"));
-    msg = translateChar(STRING_ELT(CAR(args), 0));
-    ecall = CADR(args);
+    const char *msg = translateChar(STRING_ELT(CAR(args), 0));
+    SEXP ecall = CADR(args);
 
     errorcall_dflt(ecall, "%s", msg);
 }
@@ -2415,7 +2485,7 @@ SEXP R_withCallingErrorHandler(SEXP (*body)(void *), void *bdata,
 			       SEXP (*handler)(SEXP, void *), void *hdata)
 {
     /* This defines the lambda expression for th handler. The `addr`
-       variable wil be defined in the closure environment and contain
+       variable will be defined in the closure environment and contain
        an external pointer to the callback data. */
     static const char* wceh_callback_source =
 	"function(cond) .Internal(C_tryCatchHelper(addr, 1L, cond))";
