@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2018  The R Core Team.
+ *  Copyright (C) 1997--2019  The R Core Team
  *  Copyright (C) 2003--2016  The R Foundation
+ *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,12 +28,20 @@
  *
  * Exports
  *	formatString
+ *	formatStringS
  *	formatLogical
+ *	formatLogicalS
  *	formatInteger
+ *	formatIntegerS
  *	formatReal
+ *	formatRealS
  *	formatComplex
+ *	formatComplexS
  *
  * These  formatFOO() functions determine the proper width, digits, etc.
+ *
+ * formatFOOS() functions behave identically to formatFOO
+ * except that they accept a SEXP rather than a data pointer
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +52,7 @@
 #include <float.h> /* for DBL_EPSILON */
 #include <Rmath.h>
 #include <Print.h>
+#include <R_ext/Itermacros.h> /* for ITERATE_BY_REGION */
 
 /* this is just for conformity with other types */
 attribute_hidden
@@ -52,6 +61,13 @@ void formatRaw(const Rbyte *x, R_xlen_t n, int *fieldwidth)
     *fieldwidth = 2;
 }
 
+attribute_hidden
+void formatRawS(SEXP x, R_xlen_t n, int *fieldwidth)
+{
+    *fieldwidth = 2;
+}
+
+
 attribute_hidden
 void formatString(const SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
 {
@@ -67,13 +83,32 @@ void formatString(const SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
     *fieldwidth = xmax;
 }
 
+/* currently there is no STRING_GET_REGION */
+
+attribute_hidden
+void formatStringS(SEXP x, R_xlen_t n, int *fieldwidth, int quote)
+{
+    int xmax = 0;
+    int l;
+
+    for (R_xlen_t i = 0; i < n; i++) {
+	if (STRING_ELT(x, i) == NA_STRING) {
+	    l = quote ? R_print.na_width : R_print.na_width_noquote;
+	} else l = Rstrlen(STRING_ELT(x, i), quote) + (quote ? 2 : 0);
+	if (l > xmax) xmax = l;
+    }
+    *fieldwidth = xmax;
+}
+
+
+
 void formatLogical(const int *x, R_xlen_t n, int *fieldwidth)
 {
     *fieldwidth = 1;
     for(R_xlen_t i = 0 ; i < n; i++) {
 	if (x[i] == NA_LOGICAL) {
 	    if(*fieldwidth < R_print.na_width)
-		*fieldwidth =  R_print.na_width;
+		*fieldwidth = R_print.na_width;
 	} else if (x[i] != 0 && *fieldwidth < 4) {
 	    *fieldwidth = 4;
 	} else if (x[i] == 0 && *fieldwidth < 5 ) {
@@ -84,6 +119,38 @@ void formatLogical(const int *x, R_xlen_t n, int *fieldwidth)
     }
 }
 
+void formatLogicalS(SEXP x, R_xlen_t n, int *fieldwidth) {
+    *fieldwidth = 1;
+    int tmpfieldwidth = 1;
+    ITERATE_BY_REGION_PARTIAL(x, px, idx, nb, int, LOGICAL, 0, n,
+			      {
+				  formatLogical(px, nb, &tmpfieldwidth);
+				  if( tmpfieldwidth > *fieldwidth )
+				      *fieldwidth = tmpfieldwidth;
+				  if( *fieldwidth == 5)
+				      break;  /* break iteration loop */
+			      });
+    return;
+}
+
+
+/* needed in 2 places so rolled out into macro
+   to avoid divergence
+*/
+#define FORMATINT_RETLOGIC do {					\
+	if (naflag) *fieldwidth = R_print.na_width;		\
+	else *fieldwidth = 1;					\
+								\
+	if (xmin < 0) {						\
+	    l = IndexWidth(-xmin) + 1;	/* +1 for sign */	\
+	    if (l > *fieldwidth) *fieldwidth = l;		\
+	}							\
+	if (xmax > 0) {						\
+	    l = IndexWidth(xmax);				\
+	    if (l > *fieldwidth) *fieldwidth = l;		\
+	}							\
+    } while(0)
+
 void formatInteger(const int *x, R_xlen_t n, int *fieldwidth)
 {
     int xmin = INT_MAX, xmax = INT_MIN, naflag = 0;
@@ -97,17 +164,72 @@ void formatInteger(const int *x, R_xlen_t n, int *fieldwidth)
 	    if (x[i] > xmax) xmax = x[i];
 	}
     }
+    FORMATINT_RETLOGIC;
+}
 
-    if (naflag) *fieldwidth = R_print.na_width;
-    else *fieldwidth = 1;
+void formatIntegerS(SEXP x, R_xlen_t n, int *fieldwidth)
+{
 
-    if (xmin < 0) {
-	l = IndexWidth(-xmin) + 1;	/* +1 for sign */
-	if (l > *fieldwidth) *fieldwidth = l;
+    int xmin = INT_MAX, xmax = INT_MIN, naflag = 0,
+	sorted;
+    SEXP tmpmin = NULL, tmpmax = NULL;
+    /*
+       min and max should be VERY cheap when sortedness
+       is known, so better to call them both than loop
+       through whole vector even once
+
+       Shouldn't need to check for ALTREPness here
+       because KNOWN_SORTED(sorted) will never be
+       true for non-ALTREPs or "exploded" ALTREPs
+    */
+    sorted = INTEGER_IS_SORTED(x);
+    /* if we're not formatting/printing the whole thing 
+       ALTINTEGER_MIN/MAX will give us the wrong thing
+       anyway */
+    if(n == XLENGTH(x) && KNOWN_SORTED(sorted)) {
+	tmpmin = ALTINTEGER_MIN(x, TRUE);
+	tmpmax = ALTINTEGER_MAX(x, TRUE);
+	naflag = KNOWN_NA_1ST(sorted) ?
+	    INTEGER_ELT(x, 0) == NA_INTEGER :
+	    INTEGER_ELT(x, XLENGTH(x) - 1) == NA_INTEGER;
     }
-    if (xmax > 0) {
-	l = IndexWidth(xmax);
-	if (l > *fieldwidth) *fieldwidth = l;
+
+    /*
+       If we don't have min/max methods or they need to punt
+       for some reason we will get NULL.
+
+       The data are integers, so the only reason we would not
+       get INTSXP answers is if we got Inf/-Inf because
+       everything was NA.
+
+       In both of the above cases we will
+       do things the hard way below
+    */
+    if(tmpmin != NULL && tmpmax != NULL &&
+       TYPEOF(tmpmin) == INTSXP && TYPEOF(tmpmax) == INTSXP) {
+	int l; /* only needed here so defined locally */
+	xmin = INTEGER_ELT(tmpmin, 0);
+	xmax = INTEGER_ELT(tmpmax, 0);
+	/* naflag set above */
+
+	/* this is identical logic to what formatInteger
+	   does */
+	FORMATINT_RETLOGIC;
+    } else {
+	/*
+	   no fastpass so just format using formatInteger
+	   by region. No need for FORMATINT_RETLOGIC
+	   here because it happens inside all the
+	   formatInteger calls.
+	*/
+	int tmpfw = 1;
+	*fieldwidth = 1;
+	ITERATE_BY_REGION_PARTIAL(x, px, idx, nb, int, INTEGER, 0, n,
+			  {
+			      formatInteger(px, nb, &tmpfw);
+			      if(tmpfw > *fieldwidth)
+				  *fieldwidth = tmpfw;
+			  });
     }
 }
 
@@ -217,7 +339,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
 #if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
         long double r_prec = r;
         /* use exact scaling factor in long double precision, if possible */
-        if (abs(kp) <= 27) {
+        if (abs(kp) <= KP_MAX) {
             if (kp > 0) r_prec /= tbl[kp+1]; else if (kp < 0) r_prec *= tbl[ -kp+1];
         }
 #ifdef HAVE_POWL
@@ -241,7 +363,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
 #else /* not using long doubles */
 	double r_prec = r;
         /* use exact scaling factor in double precision, if possible */
-        if (abs(kp) <= 22) {
+        if (abs(kp) <= KP_MAX) {
             if (kp >= 0) r_prec /= tbl[kp+1]; else r_prec *= tbl[ -kp+1];
         }
         /* For IEC60559 1e-308 is not representable except by gradual underflow.
@@ -304,34 +426,30 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
 
 void formatReal(const double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 {
-    int left, right, sleft;
-    int mnl, mxl, rgt, mxsl, mxns, wF;
-    Rboolean roundingwidens;
-    int neg_i, neg, kpower, nsig;
-    int naflag, nanflag, posinf, neginf;
-
-    nanflag = 0;
-    naflag = 0;
-    posinf = 0;
-    neginf = 0;
-    neg = 0;
-    rgt = mxl = mxsl = mxns = INT_MIN;
-    mnl = INT_MAX;
+    Rboolean
+	naflag = FALSE, nanflag = FALSE,
+	posinf = FALSE, neginf = FALSE;
+    int neg = 0;
+    int mnl = INT_MAX,
+	mxl, rgt, mxsl, mxns;
+    mxl = rgt = mxsl = mxns = INT_MIN;
 
     for (R_xlen_t i = 0; i < n; i++) {
 	if (!R_FINITE(x[i])) {
-	    if(ISNA(x[i])) naflag = 1;
-	    else if(ISNAN(x[i])) nanflag = 1;
-	    else if(x[i] > 0) posinf = 1;
-	    else neginf = 1;
+	    if(ISNA(x[i])) naflag = TRUE;
+	    else if(ISNAN(x[i])) nanflag = TRUE;
+	    else if(x[i] > 0) posinf = TRUE;
+	    else neginf = TRUE;
 	} else {
+	    int neg_i, kpower, nsig;
+	    Rboolean roundingwidens;
 	    scientific(&x[i], &neg_i, &kpower, &nsig, &roundingwidens);
 
-	    left = kpower + 1;
+	    int left = kpower + 1;
 	    if (roundingwidens) left--;
 
-	    sleft = neg_i + ((left <= 0) ? 1 : left); /* >= 1 */
-	    right = nsig - left; /* #{digits} right of '.' ( > 0 often)*/
+	    int sleft = neg_i + ((left <= 0) ? 1 : left); /* >= 1 */
+	    int right = nsig - left; /* #{digits} right of '.' ( > 0 often)*/
 	    if (neg_i) neg = 1;	 /* if any < 0, need extra space for sign */
 
 	    /* Infinite precision "F" Format : */
@@ -358,7 +476,7 @@ void formatReal(const double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 
     /* use nsmall only *after* comparing "F" vs "E": */
     if (rgt < 0) rgt = 0;
-    wF = mxsl + rgt + (rgt != 0);	/* width for F format */
+    int wF = mxsl + rgt + (rgt != 0);	/* width for F format */
 
     /*-- 'see' how "E" Exponential format would be like : */
     *e = (mxl > 100 || mnl <= -99) ? 2 /* 3 digit exponent */ : 1;
@@ -387,48 +505,73 @@ void formatReal(const double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
     if (neginf && *w < 4) *w = 4;
 }
 
+void formatRealS(SEXP x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
+{
+    /*
+     *  iterate by region and just take the most extreme
+     *  values across all the regions for final w, d, and e
+     */
+    int tmpw, tmpd, tmpe;
+
+    *w = 0;
+    *d = 0;
+    *e = 0;
+
+    ITERATE_BY_REGION_PARTIAL(x, px, idx, nb, double, REAL, 0, n,
+		      {
+			  formatReal(px, nb, &tmpw, &tmpd, &tmpe, nsmall);
+			  if(tmpw > *w) *w = tmpw;
+			  if(!*d && tmpd) *d = tmpd;
+			  if(tmpe > *e) *e = tmpe;
+		      });
+}
+
+#ifdef formatComplex_USING_signif
 /*   From complex.c. */
 void z_prec_r(Rcomplex *r, const Rcomplex *x, double digits);
+#endif
 
 /* As from 2.2.0 the number of digits applies to real and imaginary parts
    together, not separately */
-void formatComplex(const Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
-		   int *wi, int *di, int *ei, int nsmall)
+void formatComplex(const Rcomplex *x, R_xlen_t n,
+		   int *wr, int *dr, int *er, // (w,d,e) for Re(.)
+		   int *wi, int *di, int *ei, // (w,d,e) for Im(.)
+		   int nsmall)
 {
 /* format.info() for  x[1..n] for both Re & Im */
-    int left, right, sleft;
+    Rboolean all_re_zero = TRUE, all_im_zero = TRUE,
+	naflag = FALSE,
+	rnan = FALSE, rposinf = FALSE, rneginf = FALSE,
+	inan = FALSE, iposinf = FALSE;
+    int neg = 0;
     int rt, mnl, mxl, mxsl, mxns, wF, i_wF;
     int i_rt, i_mnl, i_mxl, i_mxsl, i_mxns;
-    Rboolean roundingwidens;
-    int neg_i, neg, kpower, nsig;
-    int naflag, rnanflag, rposinf, rneginf, inanflag, iposinf;
-    Rcomplex tmp;
-    Rboolean all_re_zero = TRUE, all_im_zero = TRUE;
-
-    naflag = 0;
-    rnanflag = 0;
-    rposinf = 0;
-    rneginf = 0;
-    inanflag = 0;
-    iposinf = 0;
-    neg = 0;
-
     rt	=  mxl =  mxsl =  mxns = INT_MIN;
     i_rt= i_mxl= i_mxsl= i_mxns= INT_MIN;
     i_mnl = mnl = INT_MAX;
 
     for (R_xlen_t i = 0; i < n; i++) {
+	Rcomplex tmp;
+#ifdef formatComplex_USING_signif
 	/* Now round */
 	z_prec_r(&tmp, &(x[i]), R_print.digits);
+#else
+	tmp.r = x[i].r;
+	tmp.i = x[i].i;
+#endif
 	if(ISNA(tmp.r) || ISNA(tmp.i)) {
-	    naflag = 1;
+	    naflag = TRUE;
 	} else {
+	    Rboolean roundingwidens;
+	    int left, right, sleft,
+		neg_i, kpower, nsig;
+
 	    /* real part */
 
 	    if(!R_FINITE(tmp.r)) {
-		if (ISNAN(tmp.r)) rnanflag = 1;
-		else if (tmp.r > 0) rposinf = 1;
-		else rneginf = 1;
+		if (ISNAN(tmp.r)) rnan = TRUE;
+		else if (tmp.r > 0) rposinf = TRUE;
+		else rneginf = TRUE;
 	    } else {
 		if(x[i].r != 0) all_re_zero = FALSE;
 		scientific(&(tmp.r), &neg_i, &kpower, &nsig, &roundingwidens);
@@ -452,8 +595,8 @@ void formatComplex(const Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 	    /* we explicitly put the sign in when we print */
 
 	    if(!R_FINITE(tmp.i)) {
-		if (ISNAN(tmp.i)) inanflag = 1;
-		else iposinf = 1;
+		if (ISNAN(tmp.i)) inan = TRUE;
+		else iposinf = TRUE;
 	    } else {
 		if(x[i].i != 0) all_im_zero = FALSE;
 		scientific(&(tmp.i), &neg_i, &kpower, &nsig, &roundingwidens);
@@ -527,7 +670,7 @@ void formatComplex(const Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 	    if (nsmall > rt) {rt = nsmall; wF = mxsl + rt + (rt != 0);}
 	    *dr = rt;
 	    *wr = wF;
-	    }
+	}
 	*ei = *di = 0;
 	*wi = i_wF;
     } else if(wF + i_wF < *wr + *wi + 2*R_print.scipen) {
@@ -548,14 +691,39 @@ void formatComplex(const Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
     if(*wi < 0) *wi = 0;
 
     /* Ensure space for Inf and NaN */
-    if (rnanflag && *wr < 3) *wr = 3;
-    if (rposinf &&  *wr < 3) *wr = 3;
-    if (rneginf &&  *wr < 4) *wr = 4;
-    if (inanflag && *wi < 3) *wi = 3;
-    if (iposinf  && *wi < 3) *wi = 3;
+    if (rnan    && *wr < 3) *wr = 3;
+    if (rposinf && *wr < 3) *wr = 3;
+    if (rneginf && *wr < 4) *wr = 4;
+    if (inan    && *wi < 3) *wi = 3;
+    if (iposinf && *wi < 3) *wi = 3;
 
     /* finally, ensure that there is space for NA */
 
     if (naflag && *wr+*wi+2 < R_print.na_width)
 	*wr += (R_print.na_width -(*wr + *wi + 2));
 }
+
+void formatComplexS(SEXP x, R_xlen_t n, int *wr, int *dr, int *er,
+		   int *wi, int *di, int *ei, int nsmall)
+{
+/* format.info() for  x[1..n] for both Re & Im */
+    int tmpwr, tmpdr, tmper, tmpwi, tmpdi, tmpei;
+
+    *wr = 0;
+    *wi = 0;
+    *dr = 0;
+    *di = 0;
+    *er = 0;
+    *ei = 0;
+    ITERATE_BY_REGION_PARTIAL(x, px, idx, nb, Rcomplex, COMPLEX, 0, n,
+		      {
+			  formatComplex(px, nb, &tmpwr, &tmpdr, &tmper,
+					&tmpwi, &tmpdi, &tmpei, nsmall);
+			  if(tmpwr > *wr) *wr = tmpwr;
+			  if(tmpdr && !*dr) *dr = tmpdr;
+			  if(tmper > *er) *er = tmper;
+			  if(tmpwi > *wi) *wi = tmpwi;
+			  if(tmpdi && !*di) *di = tmpdi;
+			  if(tmpei > *ei) *ei = tmpei;
+		      });
+}
