@@ -1,4 +1,4 @@
-/*
+/* 
    Based on gzio.c from zlib 1.2.3, but considerably modified!
 
    Copyright (C) 1995-2005 Jean-loup Gailly.
@@ -40,7 +40,7 @@
 #ifdef Win32
 # define OS_CODE  0x06
 #else
-# define OS_CODE  0x03
+# define OS_CODE  0x03 
 #endif
 
 /* R ADDITION */
@@ -245,7 +245,7 @@ gzFile R_gzopen (const char *path, const char *mode)
     if (s->mode == 'w') {
         /* Write a very simple .gz header */
         fprintf(s->file, "%c%c%c%c%c%c%c%c%c%c", gz_magic[0], gz_magic[1],
-		Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/,
+		Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, 
 		OS_CODE);
         s->start = 10L;
     } else {
@@ -539,10 +539,65 @@ int R_gzclose (gzFile file)
     gz_stream *s = (gz_stream*) file;
     if (s == NULL) return Z_STREAM_ERROR;
     if (s->mode == 'w') {
-        if (gz_flush (file, Z_FINISH) != Z_OK)
+        if (gz_flush (file, Z_FINISH) != Z_OK) 
 	    return destroy((gz_stream*) file);
         z_putLong (s->file, s->crc);
         z_putLong (s->file, (uLong) (s->in & 0xffffffff));
     }
     return destroy((gz_stream*) file);
 }
+
+/*
+static voidpf
+R_zlib_alloc(voidpf ptr, uInt items, uInt size)
+{
+    return R_alloc(items, size);
+}
+
+static void
+R_zlib_free(voidpf ptr, voidpf addr) {}
+*/
+
+/* added in 4.0.0, modified from uncompress[2] */
+static int
+R_uncompress(Bytef *dest, uLong *destLen, Bytef *source, uLong sourceLen,
+	     int opt)
+{
+    z_stream stream;
+    
+    stream.next_in = source;
+    stream.avail_in = 0;
+    stream.zalloc = (alloc_func)0;
+    stream.zfree = (free_func)0;
+    stream.opaque = (voidpf)0;
+
+    /* 
+       opt is the main difference from uncompress
+       0 means no headers (zlib)
+       16 selects gzip
+       32 allows auto-selection.
+    */
+    int err = inflateInit2( &stream, MAX_WBITS + opt );
+    if(err != Z_OK) return err;
+
+    const uInt max = (uInt)-1;  // could have used UINT_MAX
+    uLong len = sourceLen, left = *destLen;
+    stream.next_out = dest;
+    stream.avail_out = 0;
+
+    do { // do >= 2^32 bits in chunks
+        if (stream.avail_out == 0) {
+            stream.avail_out = left > (uLong)max ? max : (uInt)left;
+            left -= stream.avail_out;
+        }
+        if (stream.avail_in == 0) {
+            stream.avail_in = len > (uLong)max ? max : (uInt)len;
+            len -= stream.avail_in;
+        }
+        err = inflate(&stream, Z_NO_FLUSH);
+    } while (err == Z_OK);
+
+    *destLen = stream.total_out;
+    inflateEnd(&stream);
+    return err;
+}
