@@ -610,7 +610,7 @@ static SEXP fixcall(SEXP call, SEXP args)
 		if( !found ) {
 			SETCDR(s, allocList(1));
 			SET_TAG(CDR(s), TAG(t));
-			SETCAR(CDR(s), duplicate(CAR(t)));
+			SETCAR(CDR(s), lazy_duplicate(CAR(t)));
 		}
 	}
     }
@@ -694,7 +694,7 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
     if (cptr == NULL)
 	error(_("'NextMethod' called from outside a function"));
 
-    PROTECT(newcall = duplicate(cptr->call));
+    PROTECT(newcall = shallow_duplicate(cptr->call));
 
     /* eg get("print.ts")(1) or do.call() */
     if (TYPEOF(CAR(cptr->call)) != SYMSXP)
@@ -1246,7 +1246,7 @@ static SEXP dispatchNonGeneric(SEXP name, SEXP env, SEXP fdef)
 	cptr = cptr->nextcontext;
     }
 
-    PROTECT(e = duplicate(R_syscall(0, cptr)));
+    PROTECT(e = shallow_duplicate(R_syscall(0, cptr)));
     SETCAR(e, fun);
     /* evaluate a call the non-generic with the same arguments and from
        the same environment as the call to the generic version */
