@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2019	The R Core Team.
+ *  Copyright (C) 2000-2020	The R Core Team.
  *  Copyright (C) 1995-1998	Robert Gentleman and Ross Ihaka.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -77,15 +77,15 @@ static void PrintObject(SEXP, R_PrintData *);
 
 
 #define TAGBUFLEN 256
-#define TAGBUFLEN0 TAGBUFLEN + 6
+#define TAGBUFLEN0 (TAGBUFLEN + 6)
 static char tagbuf[TAGBUFLEN0 * 2]; /* over-allocate to allow overflow check */
 
 void PrintInit(R_PrintData *data, SEXP env)
 {
     data->na_string = NA_STRING;
     data->na_string_noquote = mkChar("<NA>");
-    data->na_width = (int) strlen(CHAR(data->na_string));
-    data->na_width_noquote = (int) strlen(CHAR(data->na_string_noquote));
+    data->na_width = Rstrlen(data->na_string, 0);
+    data->na_width_noquote = Rstrlen(data->na_string_noquote, 0);
     data->quote = 1;
     data->right = Rprt_adj_left;
     data->digits = GetOptionDigits();
@@ -146,7 +146,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid 'na.print' specification"));
 	R_print.na_string = R_print.na_string_noquote = STRING_ELT(naprint, 0);
 	R_print.na_width = R_print.na_width_noquote =
-	    (int) strlen(CHAR(R_print.na_string));
+	    Rstrlen(R_print.na_string, 0);
     }
 
     if (length(rowlab) == 0) rowlab = R_NilValue;
@@ -234,43 +234,48 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP orig = PROTECT(CONS(R_NilValue, args));
     SEXP prev = orig;
 
+    // The following relies on the *order* of args, as set inside (R level) print.default() :
     if(!isNull(CAR(args))) {
-	data.digits = asInteger(CAR(args));
-	if (data.digits == NA_INTEGER ||
-	    data.digits < R_MIN_DIGITS_OPT ||
-	    data.digits > R_MAX_DIGITS_OPT)
-	    error(_("invalid '%s' argument"), "digits");
+	data.digits = FixupDigits(CAR(args), iERROR);
     }
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // quote :
     data.quote = asLogical(CAR(args));
     if(data.quote == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "quote");
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // na.print :
     SEXP naprint = CAR(args);
     if(!isNull(naprint))  {
 	if(!isString(naprint) || LENGTH(naprint) < 1)
 	    error(_("invalid 'na.print' specification"));
 	data.na_string = data.na_string_noquote = STRING_ELT(naprint, 0);
 	data.na_width = data.na_width_noquote =
-	    (int) strlen(CHAR(data.na_string));
+	    Rstrlen(data.na_string, 0);
     }
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // print.gap :
     SEXP gap = CAR(args);
     if(!isNull(gap)) {
 	data.gap = asInteger(gap);
 	if (data.gap == NA_INTEGER || data.gap < 0)
 	    error(_("'gap' must be non-negative integer"));
+	static int gap_max = 1024;
+	if (data.gap > gap_max)
+	    error(_("'print.gap' must be less than %d"), gap_max);
     }
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // right :
     data.right = (Rprt_adj) asLogical(CAR(args)); /* Should this be asInteger()? */
     if(data.right == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "right");
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // max :
     SEXP max = CAR(args);
     if(!isNull(max)) {
 	data.max = asInteger(max);
@@ -280,6 +285,13 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
     advancePrintArgs(&args, &prev, &missingArg, &allMissing);
 
+    // width :
+    SEXP width = CAR(args);
+    if(!isNull(width))
+	data.width = FixupWidth(width, iERROR);
+    advancePrintArgs(&args, &prev, &missingArg, &allMissing);
+
+    // useSource :
     data.useSource = asLogical(CAR(args));
     if(data.useSource == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "useSource");
@@ -363,12 +375,21 @@ static void PrintObjectS3(SEXP s, R_PrintData *data)
     UNPROTECT(4); /* mask, fun, args, call */
 }
 
+/* may not be needed anymore but keep for now to be safe */
+static void save_tagbuf(char *save, size_t n)
+{
+    if (strlen(tagbuf) < n)
+	strcpy(save, tagbuf);
+    else
+	error("tagbuf overflow");
+}
+    
 static void PrintObject(SEXP s, R_PrintData *data)
 {
     /* Save the tagbuffer to restore indexing tags after evaluation
        because calling into base::print() resets the buffer */
-    char save[TAGBUFLEN0];
-    strcpy(save, tagbuf);
+    char save[sizeof tagbuf];
+    save_tagbuf(save, sizeof save);
 
     if (isMethodsDispatchOn() && IS_S4_OBJECT(s))
 	PrintObjectS4(s, data);
@@ -399,7 +420,30 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 	PROTECT(t = allocArray(STRSXP, dims));
 	/* FIXME: check (ns <= data->max +1) ? ns : data->max; */
 	for (i = 0; i < ns; i++) {
-	    switch(TYPEOF(PROTECT(tmp = VECTOR_ELT(s, i)))) {
+	    PROTECT(tmp = VECTOR_ELT(s, i));
+	    if(isObject(tmp)) {
+		const char *str;
+		Rboolean use_fmt = FALSE;
+		SEXP fun = PROTECT(findFun(install("format"),
+					   R_BaseNamespace));
+		SEXP call = PROTECT(lang2(fun, tmp));
+		SEXP ans = PROTECT(eval(call, data->env));
+		if(TYPEOF(ans) == STRSXP && LENGTH(ans) == 1) {
+		    str = translateChar(STRING_ELT(ans, 0));
+		    if(strlen(str) < 100)
+			use_fmt = TRUE;
+		}
+		if(use_fmt)
+		    snprintf(pbuf, 115, "%s", str);
+		else {
+		    SEXP cls = PROTECT(R_data_class2(tmp));
+		    Rsnprintf_mbcs(pbuf, 115, "%s,%d",
+				   translateChar(STRING_ELT(cls, 0)),
+				   length(tmp));
+		    UNPROTECT(1);
+		}
+		UNPROTECT(3);
+	    } else switch(TYPEOF(tmp)) {
 	    case NILSXP:
 		snprintf(pbuf, 115, "NULL");
 		break;
@@ -410,7 +454,7 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 		    snprintf(pbuf, 115, "%s",
 			     EncodeLogical(x[0], w));
 		} else
-		    snprintf(pbuf, 115, "Logical,%d", LENGTH(tmp));
+		    snprintf(pbuf, 115, "logical,%d", LENGTH(tmp));
 		break;
 	    case INTSXP:
 		/* factors are stored as integers */
@@ -423,7 +467,7 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 			snprintf(pbuf, 115, "%s",
 				 EncodeInteger(x[0], w));
 		    } else
-			snprintf(pbuf, 115, "Integer,%d", LENGTH(tmp));
+			snprintf(pbuf, 115, "integer,%d", LENGTH(tmp));
 		}
 		break;
 	    case REALSXP:
@@ -433,7 +477,7 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 		    snprintf(pbuf, 115, "%s",
 			     EncodeReal0(x[0], w, d, e, OutDec));
 		} else
-		    snprintf(pbuf, 115, "Numeric,%d", LENGTH(tmp));
+		    snprintf(pbuf, 115, "numeric,%d", LENGTH(tmp));
 		break;
 	    case CPLXSXP:
 		if (LENGTH(tmp) == 1) {
@@ -449,34 +493,34 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 					       wr, dr, er, wi, di, ei, OutDec));
 		    }
 		} else
-		snprintf(pbuf, 115, "Complex,%d", LENGTH(tmp));
+		snprintf(pbuf, 115, "complex,%d", LENGTH(tmp));
 		break;
 	    case STRSXP:
 		if (LENGTH(tmp) == 1) {
 		    const void *vmax = vmaxget();
-		    /* This can potentially overflow */
 		    const char *ctmp = translateChar(STRING_ELT(tmp, 0));
 		    int len = (int) strlen(ctmp);
 		    if(len < 100)
 			snprintf(pbuf, 115, "\"%s\"", ctmp);
 		    else {
-			snprintf(pbuf, 101, "\"%s\"", ctmp);
-			pbuf[100] = '"'; pbuf[101] = '\0';
+			Rsnprintf_mbcs(pbuf, 101, "\"%s\"", ctmp);
+			size_t pbuflen = strlen(pbuf);
+			pbuf[pbuflen] = '"'; pbuf[pbuflen+1] = '\0';
 			strcat(pbuf, " [truncated]");
 		    }
 		    vmaxset(vmax);
 		} else
-		snprintf(pbuf, 115, "Character,%d", LENGTH(tmp));
+		snprintf(pbuf, 115, "character,%d", LENGTH(tmp));
 		break;
 	    case RAWSXP:
-		snprintf(pbuf, 115, "Raw,%d", LENGTH(tmp));
+		snprintf(pbuf, 115, "raw,%d", LENGTH(tmp));
 		break;
 	    case LISTSXP:
 	    case VECSXP:
-		snprintf(pbuf, 115, "List,%d", length(tmp));
+		snprintf(pbuf, 115, "list,%d", length(tmp));
 		break;
 	    case LANGSXP:
-		snprintf(pbuf, 115, "Expression");
+		snprintf(pbuf, 115, "expression");
 		break;
 	    default:
 		snprintf(pbuf, 115, "?");
@@ -499,7 +543,7 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 	    printArray(t, dims, 0, Rprt_adj_left, names);
 	    UNPROTECT(1);
 	}
-	UNPROTECT(1);
+	UNPROTECT(2); /* dims, t */
     }
     else { // no dim()
 	PROTECT(names = getAttrib(s, R_NamesSymbol));
@@ -577,9 +621,10 @@ static void PrintGenericVector(SEXP s, R_PrintData *data)
 		    /* internal version of isClass() */
 		    char str[201];
 		    const char *ss = translateChar(STRING_ELT(klass, 0));
-		    snprintf(str, 200, ".__C__%s", ss);
-		    if(findVar(install(str), data->env) != R_UnboundValue)
-			className = ss;
+		    int res = Rsnprintf_mbcs(str, 200, ".__C__%s", ss);
+		    if(res > 0 && res < 200 &&
+		       findVar(install(str), data->env) != R_UnboundValue)
+		        className = ss;
 		}
 	    }
 	    if(className) {
@@ -623,32 +668,32 @@ static void printList(SEXP s, R_PrintData *data)
 		break;
 
 	    case LGLSXP:
-		snprintf(pbuf, 100, "Logical,%d", LENGTH(CAR(s)));
+		snprintf(pbuf, 100, "logical,%d", LENGTH(CAR(s)));
 		break;
 
 	    case INTSXP:
 	    case REALSXP:
-		snprintf(pbuf, 100, "Numeric,%d", LENGTH(CAR(s)));
+		snprintf(pbuf, 100, "numeric,%d", LENGTH(CAR(s)));
 		break;
 
 	    case CPLXSXP:
-		snprintf(pbuf, 100, "Complex,%d", LENGTH(CAR(s)));
+		snprintf(pbuf, 100, "complex,%d", LENGTH(CAR(s)));
 		break;
 
 	    case STRSXP:
-		snprintf(pbuf, 100, "Character,%d", LENGTH(CAR(s)));
+		snprintf(pbuf, 100, "character,%d", LENGTH(CAR(s)));
 		break;
 
 	    case RAWSXP:
-		snprintf(pbuf, 100, "Raw,%d", LENGTH(CAR(s)));
+		snprintf(pbuf, 100, "raw,%d", LENGTH(CAR(s)));
 		break;
 
 	    case LISTSXP:
-		snprintf(pbuf, 100, "List,%d", length(CAR(s)));
+		snprintf(pbuf, 100, "list,%d", length(CAR(s)));
 		break;
 
 	    case LANGSXP:
-		snprintf(pbuf, 100, "Expression");
+		snprintf(pbuf, 100, "expression");
 		break;
 
 	    default:
@@ -945,14 +990,14 @@ static void printAttributes(SEXP s, R_PrintData *data, Rboolean useSlots)
 {
     SEXP a;
     char *ptag;
-    char save[TAGBUFLEN0] = "\0";
+    char save[sizeof tagbuf] = "\0";
 
     a = ATTRIB(s);
     if (a != R_NilValue) {
 	/* guard against cycles through attributes on environments */
 	if (strlen(tagbuf) > TAGBUFLEN0)
 	    error(_("print buffer overflow"));
-	strcpy(save, tagbuf);
+	save_tagbuf(save, sizeof save);
 	/* remove the tag if it looks like a list not an attribute */
 	if (strlen(tagbuf) > 0 &&
 	    *(tagbuf + strlen(tagbuf) - 1) != ')')
@@ -983,10 +1028,16 @@ static void printAttributes(SEXP s, R_PrintData *data, Rboolean useSlots)
 	    if(TAG(a) == R_CommentSymbol || TAG(a) == R_SrcrefSymbol
 	       || TAG(a) == R_WholeSrcrefSymbol || TAG(a) == R_SrcfileSymbol)
 		goto nextattr;
-	    if(useSlots)
-		sprintf(ptag, "Slot \"%s\":", EncodeChar(PRINTNAME(TAG(a))));
-	    else
-		sprintf(ptag, "attr(,\"%s\")", EncodeChar(PRINTNAME(TAG(a))));
+	    if(useSlots) {
+		size_t space = TAGBUFLEN0 - strlen(tagbuf);
+		Rsnprintf_mbcs(ptag, space,
+			       "Slot \"%s\":", EncodeChar(PRINTNAME(TAG(a))));
+	    }
+	    else {
+		size_t space = TAGBUFLEN0 - strlen(tagbuf);
+		Rsnprintf_mbcs(ptag, space,
+			       "attr(,\"%s\")", EncodeChar(PRINTNAME(TAG(a))));
+	    }
 	    Rprintf("%s", tagbuf); Rprintf("\n");
 	    if (TAG(a) == R_RowNamesSymbol) {
 		/* need special handling AND protection */
@@ -1136,7 +1187,7 @@ void F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
 /* Fortran-callable error routine for lapack */
 
 #ifdef FC_LEN_T
-void NORET F77_NAME(xerbla)(const char *srname, int *info, 
+void NORET F77_NAME(xerbla)(const char *srname, int *info,
 			    const FC_LEN_T srname_len)
 #else
 void NORET F77_NAME(xerbla)(const char *srname, int *info)
