@@ -76,7 +76,7 @@ static int isum(SEXP sx, isum_INT *value, Rboolean narm, SEXP call)
 
     /**** assumes INTEGER(sx) and LOGICAL(sx) are identical!! */
     ITERATE_BY_REGION(sx, x, i, nbatch, int, INTEGER, {
-	    for (int k = 0; k < nbatch; k++) {
+	    for (R_xlen_t k = 0; k < nbatch; k++) {
 		if (x[k] != NA_INTEGER) {
 		    if(!updated) updated = 1;
 		    s += x[k];
@@ -101,7 +101,7 @@ static Rboolean isum(SEXP sx, isum_INT *value, Rboolean narm, SEXP call)
 
     /**** assumes INTEGER(sx) and LOGICAL(sx) are identical!! */
     ITERATE_BY_REGION(sx, x, i, nbatch, int, INTEGER, {
-	    for (int k = 0; k < nbatch; k++) {
+	    for (R_xlen_t k = 0; k < nbatch; k++) {
 		if (x[k] != NA_INTEGER) {
 		    if(!updated) updated = TRUE;
 		    s += x[k];
@@ -195,7 +195,7 @@ static Rboolean imin(SEXP sx, int *value, Rboolean narm)
     int s = 0;
 
     ITERATE_BY_REGION(sx, x, i, nbatch, int, INTEGER, {
-	    for (int k = 0; k < nbatch; k++) {
+	    for (R_xlen_t k = 0; k < nbatch; k++) {
 		if (x[k] != NA_INTEGER) {
 		    if (!updated || s > x[k]) {
 			s = x[k];
@@ -338,7 +338,7 @@ static Rboolean iprod(SEXP sx, double *value, Rboolean narm)
 
     /**** assumes INTEGER(sx) and LOGICAL(sx) are identical!! */
     ITERATE_BY_REGION(sx, x, i, nbatch, int, INTEGER, {
-	    for (int k = 0; k < nbatch; k++) {
+	    for (R_xlen_t k = 0; k < nbatch; k++) {
 		if (x[k] != NA_INTEGER) {
 		    s *= x[k];
 		    if(!updated) updated = TRUE;
@@ -537,6 +537,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
     /* match to foo(..., na.rm=FALSE) */
     PROTECT(args = fixup_NaRm(args));
     PROTECT(call2 = shallow_duplicate(call));
+    R_args_enable_refcnt(args);
     SETCDR(call2, args);
 
     if (DispatchGroup("Summary", call2, op, args, env, &ans)) {
@@ -972,6 +973,7 @@ SEXP attribute_hidden do_range(SEXP call, SEXP op, SEXP args, SEXP env)
 
     PROTECT(args = fixup_NaRm(args));
     PROTECT(call2 = shallow_duplicate(call));
+    R_args_enable_refcnt(args);
     SETCDR(call2, args);
 
     if (DispatchGroup("Summary", call2, op, args, env, &ans)) {
