@@ -45,6 +45,7 @@
 #  include <netdb.h>
 #  include <sys/socket.h>
 #  include <netinet/in.h>
+#  include <netinet/tcp.h>
 #  ifdef HAVE_FCNTL_H
 #    include <fcntl.h>
 #  endif
@@ -162,6 +163,12 @@ int R_set_nonblocking(SOCKET s)
     return status; /* 0 */
 }
 
+int R_set_nodelay(SOCKET s)
+{
+    int val = 1;
+    return setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char *) &val, sizeof(val));
+}
+
 #if defined(__hpux)
    extern int h_errno; /* HP-UX 9.05 forgets to declare this in netdb.h */
 #endif
