@@ -23,7 +23,7 @@
 /*----------- DEBUGGING -------------
  *
  *	make CFLAGS='-DDEBUG_pnch ....'
-(cd `R-devel RHOME`/src/nmath; gcc -I. -I../../src/include -I../../../R/src/include -I/usr/local/include -DHAVE_CONFIG_H -fopenmp -g -O0 -pedantic -Wall --std=gnu99 -DDEBUG_pnch -DDEBUG_q -Wcast-align -Wclobbered  -c ../../../R/src/nmath/pnchisq.c -o pnchisq.o )
+(cd `R-devel RHOME`/src/nmath; gcc -I. -I../../src/include -I../../../R/src/include -I/usr/local/include -DHAVE_CONFIG_H -fopenmp -g -O0 -pedantic -Wall -DDEBUG_pnch -DDEBUG_q -c ../../../R/src/nmath/pnchisq.c -o pnchisq.o )
 
  * -- Feb.6, 2000 (R pre0.99); M.Maechler:  still have
  * bad precision & non-convergence in some cases (x ~= f, both LARGE)
@@ -76,7 +76,7 @@ double pnchisq(double x, double df, double ncp, int lower_tail, int log_p)
      *   <---> "in principle"  this check should happen there, not here  */
     if (!log_p || ans < -1e-8)
 	return ans;
-    else { // log_p  &&  ans >= -1e-8
+    else { // log_p (==> ans <= 0) &&  -1e-8 <= ans <= 0
 	// prob. = exp(ans) is near one: we can do better using the other tail
 #ifdef DEBUG_pnch
 	REprintf("   pnchisq_raw(*, log_p): ans=%g => 2nd call, other tail\n", ans);
@@ -125,13 +125,13 @@ pnchisq_raw(double x, double f, double theta /* = ncp */,
 	   log(x) < M_LN2 + 2/f*(lgamma(f/2. + 1) + _dbl_min_exp)) {
 	    // all  pchisq(x, f+2*i, lower_tail, FALSE), i=0,...,110 would underflow to 0.
 	    // ==> work in log scale
-	    double lambda = 0.5 * theta;
-	    double sum, sum2, pr = -lambda;
+	    double lambda = 0.5 * theta; // < 40
+	    double sum, sum2, pr = -lambda, log_lam = log(lambda);
 	    sum = sum2 = ML_NEGINF;
 	    /* we need to renormalize here: the result could be very close to 1 */
-	    for(i = 0; i < 110;  pr += log(lambda) - log(++i)) {
+	    for(i = 0; i < 110;  pr += log_lam - log(++i)) {
 		sum2 = logspace_add(sum2, pr);
-		sum = logspace_add(sum, pr + pchisq(x, f+2*i, lower_tail, TRUE));
+		sum  = logspace_add(sum , pr + pchisq(x, f+2*i, lower_tail, TRUE));
 		if (sum2 >= -1e-15) /*<=> EXP(sum2) >= 1-1e-15 */ break;
 	    }
 	    ans = sum - sum2;
@@ -171,9 +171,7 @@ pnchisq_raw(double x, double f, double theta /* = ncp */,
     lam = .5 * theta; // = lambda = ncp/2
     lamSml = (-lam < _dbl_min_exp);
     if(lamSml) {
-	/* MATHLIB_ERROR(
-	   "non centrality parameter (= %g) too large for current algorithm",
-p 	   theta) */
+	// originally error: "non centrality parameter too large for current algorithm"
         u = 0;
         lu = -lam;/* == ln(u) */
         l_lam = log(lam);
@@ -231,7 +229,9 @@ p 	   theta) */
 
     for (n = 1, f_2n = f + 2., f_x_2n += 2.; n <= itrmax ; n++, f_2n += 2, f_x_2n += 2) {
 #ifdef DEBUG_pnch_n
-	REprintf("\n _OL_: n=%d",n);
+	if(n % 1000 == 0)
+	    REprintf("\n _OL_: n=%d,  f_x_2n = %g", n);
+	else REprintf(n % 100 == 0 ? ".\n" : ".");
 #endif
 #ifndef MATHLIB_STANDALONE
 	if(n % 1000 == 0) R_CheckUserInterrupt();
@@ -243,7 +243,9 @@ p 	   theta) */
 
 	    bound = (double) (t * x / f_x_2n);
 #ifdef DEBUG_pnch_n
-	    REprintf("\n L10: n=%d; term= %g; bound= %g",n,term,bound);
+	    if(n % 1000 == 0)
+		REprintf("\n L10: n=%d; term, ans = %g, %g; bound= %g",
+			 n, term, ans, bound);
 #endif
 	    is_r = FALSE;
 	    /* convergence only if BOTH absolute and relative error < 'bnd' */
@@ -251,9 +253,9 @@ p 	   theta) */
                  (is_r = (term <= reltol * ans))))
             {
 #ifdef DEBUG_pnch
-                REprintf("BREAK out of for(n = 1 ..): n=%d; bound= %g %s, rel.err= %g %s\n",
+		REprintf("BREAK from for(n=1 ..): n=%d; bound= %g %s; term=%g, rel.err= %g %s\n",
 			 n,
-			 bound, (is_b ? "<= errmax" : ""),
+			 bound, (is_b ? "<= errmax" : ""), term,
 			 term/ans, (is_r ? "<= reltol" : ""));
 #endif
 		break; /* out completely */
