@@ -30,6 +30,12 @@
 #include "nmath.h"
 #include "dpq.h"
 
+/**----------- DEBUGGING -------------
+ *
+ *	make CFLAGS='-DDEBUG_qbeta  ...'
+ *MM (w/ Debug, w/o Optimization):
+ (cd `R-devel RHOME`/src/nmath ; gcc -I. -I../../src/include -I../../../R/src/include  -DHAVE_CONFIG_H -fopenmp -g -pedantic -Wall --std=gnu99 -DDEBUG_q -DDEBUG_qbeta -Wcast-align -Wclobbered  -c ../../../R/src/nmath/qbeta.c -o qbeta.o; cd ../..; make R)
+*/
 #ifdef DEBUG_qbeta
 # define R_ifDEBUG_printf(...) REprintf(__VA_ARGS__)
 #else
@@ -58,7 +64,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
     if (ISNAN(p) || ISNAN(q) || ISNAN(alpha))
 	return p + q + alpha;
 #endif
-    if(p < 0. || q < 0.) ML_ERR_return_NAN;
+    if(p < 0. || q < 0.) ML_WARN_return_NAN;
     // allowing p==0 and q==0  <==> treat as one- or two-point mass
 
     double qbet[2];// = { qbeta(), 1 - qbeta() }
@@ -141,14 +147,14 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 	return_q_1;
     }
 
-    // check alpha {*before* transformation which may all accuracy}:
+    // check alpha {*before* transformation which may lose all accuracy}:
     if((log_p && alpha > 0) ||
        (!log_p && (alpha < 0 || alpha > 1))) { // alpha is outside
 	R_ifDEBUG_printf("qbeta(alpha=%g, %g, %g, .., log_p=%d): %s%s\n",
 			 alpha, p,q, log_p, "alpha not in ",
 			 log_p ? "[-Inf, 0]" : "[0,1]");
-	// ML_ERR_return_NAN :
-	ML_ERROR(ME_DOMAIN, "");
+	// ML_WARN_return_NAN :
+	ML_WARNING(ME_DOMAIN, "");
 	qb[0] = qb[1] = ML_NAN; return;
     }
 
@@ -478,8 +484,8 @@ L_Newton:
 #else
 	    if (errno)
 #endif
-		{ // ML_ERR_return_NAN :
-		    ML_ERROR(ME_DOMAIN, "");
+		{ // ML_WARN_return_NAN :
+		    ML_WARNING(ME_DOMAIN, "");
 		    qb[0] = qb[1] = ML_NAN; return;
 		}
 
@@ -527,7 +533,7 @@ L_Newton:
 
     /*-- NOT converged: Iteration count --*/
     warned = TRUE;
-    ML_ERROR(ME_PRECISION, "qbeta");
+    ML_WARNING(ME_PRECISION, "qbeta");
 
 L_converged:
     log_ = log_p || use_log_x; // only for printing
