@@ -35,7 +35,7 @@
 
 double rchisq(double df)
 {
-    if (!R_FINITE(df) || df < 0.0) ML_ERR_return_NAN;
+    if (!R_FINITE(df) || df < 0.0) ML_WARN_return_NAN;
 
     return rgamma(df / 2.0, 2.0);
 }
