@@ -602,8 +602,10 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
       product = (char *) read_string (h);
       if (product == NULL)
 	return 0;
-      if (inf)
-	strncpy (inf->product, product, 61);
+      if (inf) { // placate gcc 8
+	strncpy (inf->product, product, 60);
+	inf->product[60] = '\0';
+      }
     }
   else if (inf)
     inf->product[0] = 0;
@@ -616,8 +618,10 @@ read_version_data (struct file_handle *h, struct pfm_read_info *inf)
       subproduct = (char *) read_string (h);
       if (subproduct == NULL)
 	return 0;
-      if (inf)
-	strncpy (inf->subproduct, subproduct, 61);
+      if (inf) {
+	strncpy (inf->subproduct, subproduct, 60);
+	inf->subproduct[60] = '\0';
+      }
     }
   else if (inf)
     inf->subproduct[0] = 0;
@@ -876,13 +880,15 @@ parse_value (struct file_handle *h, union value *v, struct variable *vv)
       if (mv == NULL)
 	return 0;
 
-      strncpy ((char *) v->s, mv, 8);
+      // do it this way as strings might not be nul-terminated.
+      // strncpy ((char *) v->s, mv, 8);
+      /* Value labels are always padded with spaces. */
+      memset(v->s, ' ', 8);
       for (j = 0; j < 8; j++)
-	if (v->s[j])
-	  v->s[j] = spss2ascii[v->s[j]];
+	if (mv[j])
+	    v->s[j] = spss2ascii[(unsigned char)mv[j]];
 	else
-	  /* Value labels are always padded with spaces. */
-	  v->s[j] = ' ';
+	  break;
     }
   else
     {
