@@ -20,7 +20,7 @@
   (c) 1999, 2000, 2001, 2002 Thomas Lumley.
   2000 Saikat DebRoy
 
-  Patches by the R Core Team and Brian Quistorff, 2002, 2007-2015.
+  Patches by the R Core Team and Brian Quistorff, 2002, 2007-2020.
 
   Frozen 2013-10-07 at Stata version 10 aka 114 (11 is the same as 10,
   and 12 seems compatible despite being described separately as 115).
@@ -95,7 +95,7 @@ static int InIntegerBinary(FILE * fp, int naok, int swapends)
 	error(_("a binary read error occurred"));
     if (swapends)
 	reverse_int(i);
-    return ((i == STATA_INT_NA) & !naok ? NA_INTEGER : i);
+    return (((i == STATA_INT_NA) & !naok) ? NA_INTEGER : i);
 }
 /* read a 1-byte signed integer */
 static int InByteBinary(FILE * fp, int naok)
@@ -103,7 +103,7 @@ static int InByteBinary(FILE * fp, int naok)
     signed char i;
     if (fread(&i, sizeof(char), 1, fp) != 1)
 	error(_("a binary read error occurred"));
-    return  ((i == STATA_BYTE_NA) & !naok ? NA_INTEGER : (int) i);
+    return  (((i == STATA_BYTE_NA) & !naok) ? NA_INTEGER : (int) i);
 }
 /* read a single byte  */
 static int RawByteBinary(FILE * fp, int naok)
@@ -111,7 +111,7 @@ static int RawByteBinary(FILE * fp, int naok)
     unsigned char i;
     if (fread(&i, sizeof(char), 1, fp) != 1)
 	error(_("a binary read error occurred"));
-    return  ((i == STATA_BYTE_NA) & !naok ? NA_INTEGER : (int) i);
+    return  (((i == STATA_BYTE_NA) & !naok) ? NA_INTEGER : (int) i);
 }
 
 static int InShortIntBinary(FILE * fp, int naok,int swapends)
@@ -127,28 +127,34 @@ static int InShortIntBinary(FILE * fp, int naok,int swapends)
     result= (second<<8) | first;
   }
   if (result > STATA_SHORTINT_NA) result -= 65536;
-  return ((result == STATA_SHORTINT_NA) & !naok ? NA_INTEGER  : result);
+  return (((result == STATA_SHORTINT_NA) & !naok) ? NA_INTEGER  : result);
 }
 
 
 static double InDoubleBinary(FILE * fp, int naok, int swapends)
 {
+    unsigned char b[sizeof(double)];
     double i;
-    if (fread(&i, sizeof(double), 1, fp) != 1)
+
+    if (fread(b, sizeof(double), 1, fp) != 1)
 	error(_("a binary read error occurred"));
     if (swapends)
-	reverse_double(i);
-    return ((i == STATA_DOUBLE_NA) & !naok ? NA_REAL : i);
+	reverse_bytes_double(b);
+    i = *(double *) b;
+    return (((i == STATA_DOUBLE_NA) & !naok) ? NA_REAL : i);
 }
 
 static double InFloatBinary(FILE * fp, int naok, int swapends)
 {
+    unsigned char b[sizeof(float)];
     float i;
-    if (fread(&i, sizeof(float), 1, fp) != 1)
+
+    if (fread(b, sizeof(float), 1, fp) != 1)
 	error(_("a binary read error occurred"));
     if (swapends)
-	reverse_float(i);
-    return ((i == STATA_FLOAT_NA) & !naok ? NA_REAL :  (double) i);
+	reverse_bytes_float(b);
+    i = *(float *) b;
+    return (((i == STATA_FLOAT_NA) & !naok) ? NA_REAL :  (double) i);
 }
 
 static void InStringBinary(FILE * fp, int nchar, char* buffer)
@@ -595,7 +601,7 @@ SEXP do_readStata(SEXP call)
 
 static void OutIntegerBinary(int i, FILE * fp, int naok)
 {
-    i=((i == NA_INTEGER) & !naok ? STATA_INT_NA : i);
+    i = (((i == NA_INTEGER) & !naok) ? STATA_INT_NA : i);
     if (fwrite(&i, sizeof(int), 1, fp) != 1)
 	error(_("a binary write error occurred"));
 
@@ -721,6 +727,7 @@ writeStataValueLabel(const char *labelName, const SEXP theselabels,
 
 }
 
+// workhorse of R's write.dta() :
 void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
 {
     int i, j, k = 0, l, nvar, nobs, charlen;
@@ -760,7 +767,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
 
     PROTECT(dlabel = getAttrib(df, install("datalabel")));
     if(!isNull(dlabel) && isString(dlabel) && LENGTH(dlabel) == 1)
-	strncpy(datalabel, CHAR(STRING_ELT(dlabel, 0)), 81);
+	strncpy(datalabel, CHAR(STRING_ELT(dlabel, 0)), 80);
     UNPROTECT(1);
     datalabel[80] = '\0';
     OutStringBinary(datalabel, fp, 81);
@@ -896,7 +903,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
     PROTECT(vlabels = getAttrib(df, install("var.labels")));
     if(!isNull(vlabels) && isString(vlabels) && LENGTH(vlabels) == nvar){
         for(i = 0; i < nvar; i++) {
-	    strncpy(datalabel,CHAR(STRING_ELT(vlabels,i)),81);
+	    strncpy(datalabel,CHAR(STRING_ELT(vlabels,i)),80);
 	    datalabel[80] = '\0';
 	    OutStringBinary(datalabel, fp, 81);
         }
@@ -904,7 +911,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
     else{
         PROTECT(orig_names = getAttrib(df,install("orig.names")));
         for(i = 0; i < nvar; i++) {
-	    strncpy(datalabel,CHAR(STRING_ELT(orig_names,i)),81);
+	    strncpy(datalabel,CHAR(STRING_ELT(orig_names,i)),80);
 	    datalabel[80] = '\0';
 	    OutStringBinary(datalabel, fp, 81);
         }
