@@ -138,8 +138,8 @@ SEXP Csparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value)
 			 "zsparseVector",// 5      3
 			 ""};
 
-    int ctype_x = Matrix_check_class_etc(x,     valid_cM),
-	ctype_v = Matrix_check_class_etc(value, valid_spv);
+    int ctype_x = R_check_class_etc(x,     valid_cM),
+	ctype_v = R_check_class_etc(value, valid_spv);
     if (ctype_x < 0)
 	error(_("invalid class of 'x' in Csparse_subassign()"));
     if (ctype_v < 0)
@@ -186,12 +186,12 @@ SEXP Csparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value)
 	    case x_pattern:// "n"
 	    case x_logical:// "l"
 		if(ctype_v >= 3)
-		    warning(_("x[] <- val: val must be logical for \"%s\" x"),
+		    warning(_("x[] <- val: val is coerced to logical for \"%s\" x"),
 			    valid_cM[ctype_x]);
 		break;
 	    case x_integer:
 		if(ctype_v >= 4)
-		    error(_("x[] <- val: val must be integer or logical for \"%s\" x"),
+		    error(_("x[] <- val: val should be integer or logical, is coerced to integer, for \"%s\" x"),
 			  valid_cM[ctype_x]);
 		break;
 	    case x_double:
@@ -213,10 +213,10 @@ SEXP Csparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value)
     SEXP ans;
     /* Instead of simple "duplicate": PROTECT(ans = duplicate(x)) , build up: */
     // Assuming that ans will have the same basic Matrix type as x :
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(valid_cM[ctype_x])));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(valid_cM[ctype_x]));
     SET_SLOT(ans, Matrix_DimSym,      duplicate(dimslot));
-    SET_SLOT(ans, Matrix_DimNamesSym, duplicate(GET_SLOT(x, Matrix_DimNamesSym)));
-    SET_SLOT(ans, Matrix_pSym,        duplicate(GET_SLOT(x, Matrix_pSym)));
+    slot_dup(ans, x, Matrix_DimNamesSym);
+    slot_dup(ans, x, Matrix_pSym);
     SEXP r_pslot = GET_SLOT(ans, Matrix_pSym);
     // and assign the i- and x- slots at the end, as they are potentially modified
     // not just in content, but also in their *length*
@@ -419,8 +419,8 @@ SEXP Csparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value)
     }// for( jj )
 
     if(ctype_x == 1) { // triangularMatrix: copy the 'diag' and 'uplo' slots
-	SET_SLOT(ans, Matrix_uploSym, duplicate(GET_SLOT(x, Matrix_uploSym)));
-	SET_SLOT(ans, Matrix_diagSym, duplicate(GET_SLOT(x, Matrix_diagSym)));
+	slot_dup(ans, x, Matrix_uploSym);
+	slot_dup(ans, x, Matrix_diagSym);
     }
     // now assign the i- and x- slots,  free memory and return :
     Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_iSym,  INTSXP, nnz)), ri, nnz);
