@@ -1,5 +1,16 @@
 #### For both 'Extract' ("[") and 'Replace' ("[<-") Method testing
 ####    aka    subsetting     and  subassignment
+####           ~~~~~~~~~~          ~~~~~~~~~~~~~
+
+if(interactive()) {
+    options(error = recover, warn = 1)
+} else if(FALSE) { ## MM / developer  testing *manually* :
+    options(error = recover, Matrix.verbose = 2,   warn = 1)
+} else {
+    options(                 Matrix.verbose = 2, warn = 1)
+}
+## Matrix.verbose = .. (*before* loading 'Matrix' pkg)
+## ==> will also show method dispath ambiguity messages: getOption("ambiguousMethodSelection")
 
 #### suppressPackageStartupMessages(...)  as we have an *.Rout.save to Rdiff against
 stopifnot(suppressPackageStartupMessages(require(Matrix)))
@@ -7,14 +18,6 @@ stopifnot(suppressPackageStartupMessages(require(Matrix)))
 source(system.file("test-tools.R", package = "Matrix"), keep.source = FALSE)
 ##-> identical3() etc
 cat("doExtras:",doExtras,"\n")
-
-if(interactive()) {
-    options(error = recover, warn = 1)
-} else if(FALSE) { ## MM @ testing *manually* only
-    options(error = recover, Matrix.verbose = TRUE, warn = 1)
-} else {
-    options(Matrix.verbose = TRUE, warn = 1)
-}
 
 
 ### Dense Matrices
@@ -72,6 +75,16 @@ stopifnot(identical(mn["rc", "D"], mn[3,4]), mn[3,4] == 24,
 	  )
 showProc.time()
 
+## Bug found thanks to Timothy Mak, Feb 3, 2017:
+## sparseMatrix logical indexing with (partial) NA:
+a.m <- as(mn,"matrix")
+assert.EQ(as(ms,"matrix"), a.m) # incl. dimnames
+iN4 <- c(NA, TRUE, FALSE, TRUE)
+assert.EQ(as(mn[,iN4],"matrix"), a.m[,iN4]) # (incl. dimnames)
+##assert.EQ(as.matrix(ms[,iN4]), a.m[,iN4]) # ms[, <with_NA>]  fails still : _FIXME_
+try(ms[,iN4])
+try(ms[,iN4] <- 100) ## <- segfaulted in Matrix <= 1.2-8  (!)
+
 ## R-forge Matrix bug #2556: Subsetting a sparse matrix did remove  names(dimnames(.)) :
 m44 <- matrix(1:16, 4, 4, dimnames=list(row=c('a','b','c','d'), col=c('x','y','z','w')))
 ## Dense matrix: ------------------------------------------
@@ -99,6 +112,12 @@ chk.ndn(ts[i, i]); chk.ndn(ts[i, ])
 chk.ndn( b[ , 1, drop=FALSE]); chk.ndn( s[i, 2, drop=FALSE])
 chk.ndn(tb[ , 1, drop=FALSE]); chk.ndn(ts[i, 2, drop=FALSE])
 
+L0 <- logical(0)
+stopifnot(exprs = {
+    identical(dim(b[,L0]), c(4L, 0L))
+    identical(dim(b[L0,]), c(0L, 4L)) # failed till 2019-09-x
+})
+
 ## Printing sparse colnames:
 ms[sample(28, 20)] <- 0
 ms <- t(rbind2(ms, 3*ms))
@@ -115,7 +134,7 @@ m[1:2, 4] <- 200
 m[, 1] <- -1
 m[1:3,]
 
-m. <- as.matrix(m)
+m. <- .asmatrix(m)
 
 ## m[ cbind(i,j) ] indexing:
 iN <- ij <- cbind(1:6, 2:3)
@@ -134,8 +153,8 @@ nsel <-(20 >= m) | (m >= 150)
 stopifnot(is(sel, "lMatrix"), is(ssel, "lsparseMatrix"),
 	  identical3(as.mat(sel.), as.mat(sel), as.mat(ssel)),
 	  identical3(!sel, !ssel, nsel), # !<sparse> is typically dense
-	  identical3(m[ sel],  m[ ssel], as.matrix(m)[as.matrix( ssel)]),
-	  identical3(m[!sel],  m[!ssel], as.matrix(m)[as.matrix(!ssel)])
+	  identical3(m[ sel],  m[ ssel], .asmatrix(m)[.asmatrix( ssel)]),
+	  identical3(m[!sel],  m[!ssel], .asmatrix(m)[.asmatrix(!ssel)])
 	  )
 showProc.time()
 
@@ -207,11 +226,9 @@ getDuplIndex <- function(n, k) {
     i
 }
 
-## From package 'sfsmisc':
-repChar <- function (char, no) paste(rep.int(char, no), collapse = "")
-
+suppressWarnings(RNGversion("3.5.0")); set.seed(101)
 m <- 1:800
-set.seed(101) ; m[sample(800, 600)] <- 0
+m[sample(800, 600)] <- 0
 m0 <- Matrix(m, nrow = 40)
 m1 <- add.simpleDimnames(m0)
 for(kind in c("n", "l", "d")) {
@@ -238,8 +255,8 @@ for(kind in c("n", "l", "d")) {
     show(mC[,1])
     show(mC[1:2,])
     show(mC[7,  drop = FALSE])
-    assert.EQ.mat(mC[1:2,], mm[1:2,])
-    assert.EQ.mat(mC[0,], mm[0,])
+    assert.EQ.mat(mC[1:2,],   mm[1:2,])
+    assert.EQ.mat(mC[0,],     mm[0,])
     assert.EQ.mat(mC[,FALSE], mm[,FALSE])
     ##
     ## *repeated* (aka 'duplicated') indices - did not work at all ...
@@ -256,15 +273,15 @@ for(kind in c("n", "l", "d")) {
     ## set.seed(7)
     op <- options(Matrix.verbose = FALSE)
     cat(" for(): ")
-    for(n in 1:(if(doExtras) 50 else 5)) {
+    for(n in 1:(if(doExtras) 50 else 5)) { # (as chkAssign() is random)
 	chkAssign(mC, mm)
 	chkAssign(mC[-3,-2], mm[-3,-2])
         cat(".")
     }
     options(op)
-    cat(sprintf("\n[Ok]%s\n\n", repChar("-", 64)))
+    cat(sprintf("\n[Ok]%s\n\n", strrep("-", 64)))
  }
- cat(sprintf("\nok( %s )\n== ###%s\n\n", kind, repChar("=", 70)))
+ cat(sprintf("\nok( %s )\n== ###%s\n\n", kind, strrep("=", 70)))
 }## end{for}---------------------------------------------------------------
 showProc.time()
 
@@ -308,10 +325,10 @@ if(doExtras) {### {was ./AAA_index.R, MM-only}
     show(N1 <- t(N <- mkN1(N0)))    # transpose {for display reasons}
     C1 <- t(C <- mkN1(as(N0,"CsparseMatrix")))
     stopifnot(all(C == N))
-    assert.EQ.mat(C, mkN1(as.matrix(N0)))
+    assert.EQ.mat(C, mkN1(.asmatrix(N0)))
 
     C. <- C1
-    show(N <- N1) ; n <- as.matrix(N); str(N)
+    show(N <- N1) ; n <- .asmatrix(N); str(N)
     sort(i <- c(6,8,19,11,21,20,10,7,12,9,5,18,17,22,13))## == c(5:13, 17:22))
     sort(j <- c(3,8,6,15,10,4,14,13,16,2,11,17,7,5))## == c(2:8, 10:11, 13:17)
     val <- v.l <- 5*c(0,6,0,7,0,0,8:9, 0,0)
@@ -393,10 +410,11 @@ cb <- crossprod(b)
 cB <- crossprod(Matrix(b, sparse=TRUE))
 a <- matrix(0, 6, 6)
 a[1:4, 1:4] <- cb
-A1 <- A2 <- Matrix(0, 6, 6)#-> sparse
+A1 <- A2 <- Matrix(0, 6, 6)#-> ddiMatrix
 A1[1:4, 1:4] <- cb
 A2[1:4, 1:4] <- cB
 assert.EQ.mat(A1, a)# indeed
+## "must": symmetric and sparse, i.e., ds*Matrix:
 stopifnot(identical(A1, A2), is(A1, "dsCMatrix"))
 
 ## repeated ones ``the challenge'' (to do smartly):
@@ -482,16 +500,16 @@ diag(B.) <- 10 * diag(B.)
 diag(B.[,-1]) <- 5* diag(B.[,-1])
 diag(B.[-1,]) <- 4* diag(B.[-1,]) ; B.
 C <- B.; C[,2] <- C[,2];  C[1,] <- C[1,]; C[2:3,2:1] <- C[2:3,2:1]
-stopifnot(identical(unname(as.matrix(A)),
+stopifnot(identical(unname(.asmatrix(A)),
 		    local({a <- matrix(0,4,3); a[c(1,2,1), 2] <-  1 ; a})),
-	  identical(unname(as.matrix(B)),
+	  identical(unname(.asmatrix(B)),
 		    local({a <- matrix(0,4,3); a[c(1,2,1), 2] <- 1:3; a})),
 	  identical(C, drop0(B.)))
 ## <sparse>[<logicalSparse>] <- v  failed in the past
 T <- as(C,"TsparseMatrix"); C. <- C
 T[T>0] <- 21
 C[C>0] <- 21
-a. <- local({a <- as.matrix(C.); a[a>0] <- 21; a})
+a. <- local({a <- .asmatrix(C.); a[a>0] <- 21; a})
 assert.EQ.mat(C, a.)
 stopifnot(identical(C, as(T, "CsparseMatrix")))
 
@@ -578,6 +596,62 @@ stopifnot(identical(m2[1:3,], as(m1[1:3,], "CsparseMatrix")),
                     uniqTsparse(as(m2[, c(4,2)], "TsparseMatrix")))
           )## failed in 0.9975-11
 
+## 0-dimensional diagonal - subsetting ----------------------------
+## before that diagU2N() etc for 0-dim. dtC*:
+m0. <- m00 <- matrix(numeric(),0,0)
+dimnames(m0.) <- list(NULL, NULL)
+ tC0.<- new("dtCMatrix")
+ tC0 <- new("dtCMatrix", diag="U")
+(gC0 <- new("dgCMatrix")) # 0 x 0
+D0 <- Diagonal(0)
+stopifnot(exprs = {
+    identical(m0.,       as(tC0, "matrix")) # failed: Cholmod error 'invalid xtype' ..
+    identical(numeric(), as(tC0, "numeric"))#   "
+    identical(numeric(), tC0[ 0 ])# --> .M.vectorSub(x, i) failed in as(., "matrix")
+    identical(m00[TRUE ], tC0[TRUE ])# (worked already)
+    identical(m00[FALSE], tC0[FALSE])#    ditto
+    ##
+    identical(D0, D0[0,0]) # used to fail --> subCsp_ij  (..)
+    identical(D0, D0[ ,0]) #  (ditto)     --> subCsp_cols(..)
+    identical(D0, D0[0, ]) #     "        --> subCsp_rows(..)
+    identical(D0, D0[,])          # (worked already)
+    identical(m00[ 0 ],  D0[ 0 ] )#      ditto
+    identical(m00[TRUE ], D0[TRUE ])#      "
+    identical(m00[FALSE], D0[FALSE])#      "
+    ##
+    identical(tC0.,tC0[0,0]) # failed --> subCsp_ij  (..)
+    identical(gC0, tC0[ ,0]) #   "    --> subCsp_cols(..)
+    identical(gC0, tC0[0, ]) #   "    --> subCsp_rows(..)
+    identical(tC0, tC0[,])   # (worked already)
+    ## vector indexing
+})
+
+expr <- quote({ ## FIXME -- both 'TRUE' and 'FALSE'  should fail "out of bound",etc
+  D0[TRUE, TRUE ]
+  D0[    , TRUE ]
+  D0[TRUE,      ] # worked  but should *NOT*
+ tC0[TRUE, TRUE ]
+ tC0[    , TRUE ]
+ tC0[TRUE,      ] # worked  but should *NOT*
+  ##
+  D0[FALSE,FALSE] # fails --> subCsp_ij(..) -> intI()
+  D0[     ,FALSE] #    ditto ............
+  D0[FALSE,     ] #    ditto
+ tC0[FALSE,FALSE] #      "
+ tC0[FALSE,     ] #      "
+ tC0[     ,FALSE] #      "
+})
+EE <- lapply(expr[-1], function(e)
+    list(expr = e,
+         r    = tryCatch(eval(e), error = identity)))
+exR <- lapply(EE, `[[`, "r")
+stopifnot(exprs = {
+    vapply(exR, inherits, logical(1), what = "error")
+    unique( vapply(exR, `[[`, "<msg>", "message")
+           ) == "logical subscript too long (1, should be 0)"
+})
+
+
 (uTr <- new("dtTMatrix", Dim = c(3L,3L), diag="U"))
 uTr[1,] <- 0
 assert.EQ.mat(uTr, cbind(0, rbind(0,diag(2))))
@@ -637,24 +711,24 @@ L <- c(TRUE, rep(FALSE,8)) ; z <- c(50,99)
 ## these now work correctly {though not very efficiently; hence warnings}
 m[i] <- v # the role model: only first column is affected
 M[i] <- v; assert.EQ.mat(M,m) # dge
-D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
+D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtC (new! 2019-07; was dgT)
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
-m.L <- asLogical(m)
-C[i] <- v # - with a warning: C is nMatrix, v not T/F
-assert.EQ.mat(C,m.L); validObject(C)
-N[i] <- v # - with a warning
+m.L <- asLogical(m) ; assertWarning(
+C[i] <- v, verbose=TRUE) # warning: C is nMatrix, v not T/F
+assert.EQ.mat(C,m.L); validObject(C); assertWarning(
+N[i] <- v, verbose=TRUE)
 assert.EQ.mat(N,m.L); validObject(N)
-stopifnot(Q.C.identical(D,s, checkClass=FALSE))
+stopifnot(identical(D, as(as(s, "triangularMatrix"), "CsparseMatrix")))
 ## logical *vector* indexing
 eval(.iniDiag.example)
 m[L] <- z; m.L <- asLogical(m)
 M[L] <- z; assert.EQ.mat(M,m)
 D[L] <- z; assert.EQ.mat(D,m)
 s[L] <- z; assert.EQ.mat(s,m)
-S[L] <- z; assert.EQ.mat(S,m) ; S
-C[L] <- z; assert.EQ.mat(C,m.L) # with a good warning
-N[L] <- z; assert.EQ.mat(N,m.L)
+S[L] <- z; assert.EQ.mat(S,m) ; S ; assertWarning(
+C[L] <- z, verbose=TRUE); assert.EQ.mat(C,m.L)  ; assertWarning(
+N[L] <- z, verbose=TRUE); assert.EQ.mat(N,m.L)
 
 
 ## indexing [i]  vs  [i,] --- now ok
@@ -706,10 +780,10 @@ m[i] <- v; m.L <- asLogical(m)
 M[i] <- v; assert.EQ.mat(M,m) # dge
 D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
-S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
-N[i] <- v # with a good warning
-assert.EQ.mat(N,m.L); N
-C[i] <- v #  ..  .  ..  warning
+S[i] <- v; assert.EQ.mat(S,m); S ; assertWarning( # dtC -> dtT -> dgT -> dgC
+N[i] <- v, verbose=TRUE)
+assert.EQ.mat(N,m.L); N ; assertWarning(
+C[i] <- v, verbose=TRUE)
 assert.EQ.mat(C,m.L); C #
 
 options(warn = 2) #---------------------# NO WARNINGS from here -----------------
@@ -768,7 +842,7 @@ mt
 mc[1,4] <- -99 ; stopifnot(mc[1,4] == -99)
 mc[1,4] <-  00 ; stopifnot(mc[1,4] ==  00)
 mc[1,4] <- -99 ; stopifnot(mc[1,4] == -99)
-mc[1:2,4:3] <- 4:1; stopifnot(as.matrix(mc[1:2,4:3]) == 4:1)
+mc[1:2,4:3] <- 4:1; stopifnot(.asmatrix(mc[1:2,4:3]) == 4:1)
 
 mc[-1, 3] <- -2:1 # 0 should not be entered; 'value' recycled
 mt[-1, 3] <- -2:1
@@ -841,13 +915,13 @@ H[i,j] <- 0
 (H. <- round(as(H, "sparseMatrix"), 3)[ , 2:7])
 Hc. <- Hc
 Hc.[i,j] <- 0 ## now "works", but setting "non-structural" 0s
-stopifnot(as.matrix(Hc.[i,j]) == 0)
+stopifnot(.asmatrix(Hc.[i,j]) == 0)
 Hc.[, 1:6]
 
 ## an example that failed for a long time
 sy3 <- new("dsyMatrix", Dim = as.integer(c(2, 2)), x = c(14, -1, 2, -7))
 checkMatrix(dm <- kronecker(Diagonal(2), sy3))# now sparse with new kronecker
-dm <- Matrix(as.matrix(dm))# -> "dsyMatrix"
+dm <- Matrix(.asmatrix(dm))# -> "dsyMatrix"
 (s2 <- as(dm, "sparseMatrix"))
 checkMatrix(st <- as(s2, "TsparseMatrix"))
 stopifnot(is(s2, "symmetricMatrix"),
@@ -897,8 +971,20 @@ assert.EQ.mat(t2, m)# ok
 assert.EQ.mat(s2, m)# failed in 0.9975-8
 showProc.time()
 
+## sub-assign  RsparseMatrix -- Matrix bug [#6709] by David Cortes
+## https://r-forge.r-project.org/tracker/?func=detail&atid=294&aid=6709&group_id=61
+## simplified by MM
+X <- new("dgCMatrix", i = c(0L,3L), p = c(0L,2L,2L,2L), x = c(100, -20), Dim = c(12L,3L))
+R <- as(X, "RsparseMatrix")
+T <- as(R, "TsparseMatrix")
+T[, 2] <- 22 # works fine
+R[, 2] <- 22 # failed, as it called replTmat() giving narg() == 3
+## now R is Tsparse (as documented on ../man/RsparseMatrix-class.Rd),
+identical(R, T) ## but as this may change, rather R & T should have the same *content*
+assert.EQ.Mat(R, T)
 
-## m[cbind(i,j)] <- value: (2-column matrix subassignment):
+
+## m[cbind(i,j)] <- value: (2-column matrix subassignment): -------------------------
 m.[ cbind(3:5, 1:3) ] <- 1:3
 stopifnot(m.[3,1] == 1, m.[4,2] == 2)
 nt. <- nt ; nt[rbind(2:3, 3:4, c(3,3))] <- FALSE
@@ -921,21 +1007,31 @@ ii <- c(1:2, 4:5)
 d6[cbind(ii,ii)] <- 7*ii
 stopifnot(is(d6, "ddiMatrix"), identical(d6, Diagonal(x=c(7*1:2,1,7*4:5,1))))
 
-for(j in 3:6) { ## even and odd j used to behave differently
+sclass <- function(obj) as.vector(class(obj)) # as.v*(): drop attr(*,"package")
+show2cls <- function(C,D, chr = "")
+    cat(sprintf("%s & %s%s: %s %s\n",
+                deparse(substitute(C)), deparse(substitute(D)), chr,
+                sclass(C), sclass(D)))
+for(j in 2:6) { ## even and odd j used to behave differently
+    cat("j = ", j, ":\n-------\n")
     M <- Matrix(0, j,j); m <- matrix(0, j,j)
     T  <- as(M, "TsparseMatrix")
     TG <- as(T, "generalMatrix")
-    G <-  as(M, "generalMatrix")
+    G  <- as(M, "generalMatrix");  show2cls(TG, G)
+    stopifnot(is(TG, "TsparseMatrix"),
+              is(G,  "CsparseMatrix"))
     id <- cbind(1:j,1:j)
     i2 <- cbind(1:j,j:1)
     m[id] <- 1:j
-    M[id] <- 1:j ; stopifnot(is(M,"symmetricMatrix"))
-    T[id] <- 1:j ; stopifnot(is(T,"symmetricMatrix"))
+    M[id] <- 1:j
+    T[id] <- 1:j ; show2cls(M, T,' ("diag")')
+    stopifnot(is(M, "diagonalMatrix"), # since 2019-07 // FIXME (?!) for j=1
+              is(T,"triangularMatrix"), isDiagonal(T)) # was "symmetricMatrix"
     G[id] <- 1:j
     TG[id]<- 1:j
     m[i2] <- 10
-    M[i2] <- 10 ; stopifnot(is(M,"symmetricMatrix"))
-    T[i2] <- 10 ; stopifnot(is(T,"symmetricMatrix"))
+    M[i2] <- 10
+    T[i2] <- 10 ; show2cls(M, T,' ("symm")')
     G[i2] <- 10
     TG[i2]<- 10
     ##
@@ -999,6 +1095,7 @@ showProc.time()
 n <- 7000000
 m <-  100000
 nnz <- 20000
+op <- options(Matrix.verbose = 2, warn = 1)
 
 set.seed(12)
 f <- sparseMatrix(i = sample(n, size=nnz, replace=TRUE),
@@ -1009,30 +1106,32 @@ prod(dim(f)) # 699930301096 == 699'930'301'096  (~ 700'000 millions)
 str(thisCol <-  f[,5000])# logi [~ 7 mio....]
 sv <- as(thisCol, "sparseVector")
 str(sv) ## "empty" !
-validObject(spCol <- f[,5000, drop=FALSE])
+validObject(spCol <- f[,5000, drop=FALSE]) # "empty" [n x 1] ngCmatrix
 ##
 ## *not* identical(): as(spCol, "sparseVector")@length is "double"prec:
 stopifnot(all.equal(as(spCol, "sparseVector"),
                     as(sv,   "nsparseVector"), tolerance=0))
-if(doExtras) {#-----------------------------------------------------------------
-f[,5762] <- thisCol # now "fine" <<<<<<<<<< FIXME uses LARGE objects -- slow --
-## is using  replCmat() in ../R/Csparse.R, then
-##           replTmat() in ../R/Tsparse.R
+if(interactive())
+    selectMethod("[<-", c("ngCMatrix", "missing","numeric", "logical"))
+                                        # -> replCmat() in ../R/Csparse.R
+f[,5762] <- thisCol # now "fine" and fast thanks to replCmat() --> replCmat4()
 
 fx <- sparseMatrix(i = sample(n, size=nnz, replace=TRUE),
                    j = sample(m, size=nnz, replace=TRUE),
                    x = round(10*rnorm(nnz)))
 class(fx)## dgCMatrix
-fx[,6000] <- (tC <- rep(thisCol, length=nrow(fx)))# slow (as above)
+fx[,6000] <- (tC <- rep(thisCol, length=nrow(fx)))# fine
 thCol <- fx[,2000]
-fx[,5762] <- thCol# slow
+fx[,5762] <- thCol# fine
 stopifnot(is(f, "ngCMatrix"), is(fx, "dgCMatrix"),
 	  identical(thisCol, f[,5762]),# perfect
 	  identical(as.logical(fx[,6000]), tC),
 	  identical(thCol,  fx[,5762]))
 
 showProc.time()
+options(op)# revert
 ##
+if(doExtras) {#-----------------------------------------------------------------
 cat("checkMatrix() of all: \n---------\n")
 Sys.setlocale("LC_COLLATE", "C")# to keep ls() reproducible
 for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
@@ -1073,12 +1172,100 @@ assert.EQ.mat(x1, x)
 
 i <- 4:7
 x1 <- x0; x1[cbind(i, i+10)] <- i^2
-x2 <- x0; x2[cbind(i, i+10)] <- as.matrix(i^2)
+x2 <- x0; x2[cbind(i, i+10)] <- .asmatrix(i^2)
 ## failed: nargs() = 4 ... please report
 
 stopifnot(isValid(x1, "dgTMatrix"), identical(x1, x2))
 
 
+iv <- c(rep(0,100), 3, 0,0,7,0,0,0)
+sv0  <- sv  <- as(iv, "sparseVector")
+sv.0 <- sv. <- as(as.integer(iv), "sparseVector")
+stopifnot(canCoerce("integer", "sparseVector"))
+sv2 <- as(sv, "isparseVector")
+stopifnot(validObject(sv), validObject(sv2), identical(sv., sv2),
+          sv == sv.)
+n0 <- sv. != 0 # -> is "lsparseV.."
+if(FALSE)
+    debug(Matrix:::replSPvec) ## --> ../R/sparseVector.R : replSPvec()
+##
+sv [n0] <- sv [n0]
+sv.[n0] <- sv.[n0] # gave error
+stopifnot(identical(sv , sv0),
+          identical(sv., sv.0))
+sv [3:7] <- 0
+sv.[3:7] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+sv [2:4] <- 2:4
+sv.[2:4] <- 2:4
+stopifnot(which(sv != 0) == (which(sv. != 0) -> in0),
+          in0 == c(2:4, 101L, 104L))
+sv [2:6] <- 0L
+sv.[2:6] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+## the next six *all* gave an error -- but should be no-op's:
+for(vv in list(sv, sv.0))
+    for(ind in list(0, FALSE, logical(length(vv))))
+        vv[ind] <- NA
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+## <sparseVector>[i] <- val -- failed to resort @i sometimes:  (R-forge Matrix bug #6659)
+y1 <- sparseVector(1:3, 13:15, 16)
+y2 <- sparseVector(1:6, c(5, 6, 7, 9, 14, 15), 16)
+i <- 1:16*12 # 12 24 36 ... 192
+x <- sparseVector(numeric(1), 1, length=200)
+x[i] <- y1     ; validObject(x[i]) # TRUE
+N <- x[i] + y2 ; validObject( N  ) # TRUE
+x[i] <- N ## <== bug was here ..
+validObject(x)
+## gave 'Error' invalid ..“dsparseVector”.. 'i' must be sorted strictly increasingly
+stopifnot(all.equal(x[i] ,  y1+y2, tolerance=0),
+		    x[i] == y1+y2)
 showProc.time()
 
 if(!interactive()) warnings()
+
+## [matrix-Bugs][#6720] Subsetting with empty indices does not drop -- 17 Apr 2021, by David Cortes
+##  https://r-forge.r-project.org/tracker/?func=detail&atid=294&aid=6720&group_id=61
+
+## extended by MM to all versions of "empty" :
+x <- c(1,8)
+(m1 <-  rbind(x))
+m1[]             # remains matrix
+m1[,,drop=FALSE] # ditto
+m1[,] # [1] 1 2 -- drops (as default drop=TRUE !)
+
+## Sparse Matrix and  actually *any* Matrix-extending class did not work
+(M1 <- as(m1, "denseMatrix")) # "dgeMatrix"
+S1 <- as(m1, "CsparseMatrix")
+R1 <- as(m1, "RsparseMatrix")
+stopifnot(exprs = {
+    identical(M1[], M1) # remains
+    identical(S1[], S1) # remains
+    identical(R1[], R1) # remains
+    identical(M1[,,drop=FALSE], M1) # ditto
+    identical(S1[,,drop=FALSE], S1) #  "
+    identical(R1[,,drop=FALSE], R1) #  "
+    ## but drop=TRUE  which is the *default* much be obeyed (also for *empty* (i,j):
+    identical(m1[,], x)
+    identical(M1[,], x) # should drop, but did not
+    identical(S1[,], x) #  "
+    identical(R1[,], x) #  "
+    identical(m1[,,drop=TRUE], x)
+    identical(M1[,,drop=TRUE], x) # should drop, but did not
+    identical(S1[,,drop=TRUE], x) #  "
+    identical(R1[,,drop=TRUE], x)  # "
+})
+
+
+## [matrix-Bugs][#6721] Assignment to 'dgRMatrix' with missing index takes only first element
+## MM: This has been fixed already!
+X <- rbind(0, 1:3, 0, c(0,1,0))
+Rx <- as(X, "RsparseMatrix")
+Cx <- as(X, "CsparseMatrix")
+X [2,] <- 0
+Cx[2,] <- 0
+Rx[2,] <- 0
+stopifnot(all(Cx == X),
+          all(Rx == X))
